/***************************************************************************

  osdepend.c

  OS dependant stuff (display handling, keyboard scan...)
  This is the only file which should me modified in order to port the
  emulator to a different system.

***************************************************************************/
#ifdef PROFILE_GENERATE
//extern void __libc_init_array(void);
#endif

#include "driver.h"
#include "datafile.h"
#include "stdlib.h"
#include <time.h>
#include <ctype.h>
#include <stdarg.h>
#include <string.h>
#include <strings.h>
#include "minimal.h"

int  msdos_init_sound(void);
void msdos_init_input(void);
int  frontend_help (int argc, char **argv);
void parse_cmdline (int argc, char **argv, int game);
void init_inpdir(void);
extern char     *_app_path;         /* from Native SDK entry_point.c */
extern void free_decompose_rom_sample_path(void);
extern void _memsys_cleanup();

//static FILE *errorlog;

/* put here anything you need to do when the program is started. Return 0 if */
/* initialization was successful, nonzero otherwise. */
int osd_init(void)
{
    if (msdos_init_sound())
	    return 1;
    msdos_init_input();
    dingoo_init_malloc();
    return 0;
}


/* put here cleanup routines to be executed when the program is terminated. */
void osd_exit(void)
{
    free_datafile_indices();
    free_decompose_rom_sample_path();
    //_memsys_cleanup();
    return;
}

/* fuzzy string compare, compare short string against long string        */
/* e.g. astdel == "Asteroids Deluxe". The return code is the fuzz index, */
/* we simply count the gaps between maching chars.                       */
int fuzzycmp (const char *s, const char *l)
{
	int gaps = 0;
	int match = 0;
	int last = 1;

	for (; *s && *l; l++)
	{
		if (*s == *l)
			match = 1;
		else if (*s >= 'a' && *s <= 'z' && (*s - 'a') == (*l - 'A'))
			match = 1;
		else if (*s >= 'A' && *s <= 'Z' && (*s - 'A') == (*l - 'a'))
			match = 1;
		else
			match = 0;

		if (match)
			s++;

		if (match != last)
		{
			last = match;
			if (!match)
				gaps++;
		}
	}

	/* penalty if short string does not completely fit in */
	for (; *s; s++)
		gaps++;

	return gaps;
}

int main_run_game (int argc, char **argv)
{
	int res, i, j = 0, game_index;
	unsigned int use_fame=0;
	unsigned int dingoo_clock = DINGOO_DEFAULT_MHZ;
	unsigned int dingoo_brightness = DINGOO_DEFAULT_BRIGHT;
	unsigned int profilemode = 0;
	extern int video_scale;
	extern int video_border;
	extern int video_aspect;
	extern int throttle;
#if (HAS_M68000_FAME)
	extern int using_always_musashi;
#endif
	
	memset(&options,0,sizeof(options));

	/* set option defaults, set in the next piece of code */
	//errorlog = 0;
	game_index = -1; /* game not found */
	use_fame=0;
	video_scale=0;
	video_aspect=0;
	throttle=1;

	/* Removed the first arv which was the name of the executable */
	/* So we start at 0 */
	for (i = 1;i < argc;i++)
	{
		if (strcasecmp(argv[i],"-fame") == 0)
			use_fame=1;
		if (strcasecmp(argv[i],"-horizscale") == 0)
			video_scale=1;
		if (strcasecmp(argv[i],"-halfscale") == 0)
			video_scale=2;
		if (strcasecmp(argv[i],"-bestscale") == 0)
			video_scale=3;
		if (strcasecmp(argv[i],"-fastscale") == 0)
			video_scale=4;
		if (strcasecmp(argv[i],"-border") == 0)
			video_border=1;
		if (strcasecmp(argv[i],"-aspect") == 0)
			video_aspect=1;
		if (strcasecmp(argv[i],"-nothrottle") == 0)
			throttle=0;
		if (strcasecmp(argv[i],"-profilemode") == 0)
			profilemode=1;
		if (strcasecmp(argv[i],"-brightness") == 0)
			dingoo_brightness=atoi(argv[i+1]);
		if ((strcasecmp(argv[i],"-clock") == 0) && (i<argc-1))
			dingoo_clock=atoi(argv[i+1]);
	}

	dingoo_set_clock(dingoo_clock);

#if 0        
	/* check for frontend options */
	res = frontend_help (argc, argv);
#endif

	/* Determine game to play*/
	if (game_index == -1)
	{
	    /* take the first commandline argument without "-" as the game name */
	    for (j = 1; j < argc; j++)
	    {
		if (argv[j][0] != '-') break;
	    }
		    /* do we have a driver for this? */
	    {
		for (i = 0; drivers[i] && (game_index == -1); i++)
		{
		    if (strcasecmp(argv[j],drivers[i]->name) == 0)
		    {
			game_index = i;
			break;
		    }
		}

		/* educated guess on what the user wants to play */
		if (game_index == -1)
		{
		    int fuzz = 9999; /* best fuzz factor so far */

		    for (i = 0; (drivers[i] != 0); i++)
		    {
			int tmp;
			tmp = fuzzycmp(argv[j], drivers[i]->description);
			/* continue if the fuzz index is worse */
			if (tmp > fuzz)
			    continue;

			/* on equal fuzz index, we prefer working, original games */
			if (tmp == fuzz)
			{
			    /* game is a clone */
			    if (drivers[i]->clone_of != 0
				    && !(drivers[i]->clone_of->flags & NOT_A_DRIVER))
			    {
				/* if the game we already found works, why bother. */
				/* and broken clones aren't very helpful either */
				if ((!drivers[game_index]->flags & GAME_NOT_WORKING) ||
				    (drivers[i]->flags & GAME_NOT_WORKING))
				    continue;
			    }
			    else continue;
			}

			/* we found a better match */
			game_index = i;
			fuzz = tmp;
		    }

		    if (game_index != -1)
			printf("fuzzy name compare, running %s\n",drivers[game_index]->name);
		}
	    }

	    if (game_index == -1)
	    {
		printf("Game \"%s\" not supported\n", argv[j]);
		return 1;
	    }
	}

	/* parse generic (os-independent) options */
	parse_cmdline (argc, argv, game_index);

	{	/* Mish:  I need sample rate initialised _before_ rom loading for optional rom regions */
		extern int soundcard;

		if (soundcard == 0) {    /* silence, this would be -1 if unknown in which case all roms are loaded */
			Machine->sample_rate = 0; /* update the Machine structure to show that sound is disabled */
			options.samplerate=0;
		}
	}

#if (HAS_M68000_FAME)
    using_always_musashi = 0;	// default core
    if (use_fame == 1)
    {
	printf("Using FAME/C (fast)\n");
    }
#if (HAS_M68000)
    else
    {
	printf("Using Musashi (Compat)\n");
	using_always_musashi = 1;
    }
#endif
#endif

    // Remove the mouse usage for certain games
    if ( (strcasecmp(drivers[game_index]->name,"hbarrel")==0) || (strcasecmp(drivers[game_index]->name,"hbarrelw")==0) ||
         (strcasecmp(drivers[game_index]->name,"midres")==0) || (strcasecmp(drivers[game_index]->name,"midresu")==0) ||
         (strcasecmp(drivers[game_index]->name,"midresj")==0) || (strcasecmp(drivers[game_index]->name,"tnk3")==0) ||
         (strcasecmp(drivers[game_index]->name,"tnk3j")==0) || (strcasecmp(drivers[game_index]->name,"ikari")==0) ||
         (strcasecmp(drivers[game_index]->name,"ikarijp")==0) || (strcasecmp(drivers[game_index]->name,"ikarijpb")==0) ||
         (strcasecmp(drivers[game_index]->name,"victroad")==0) || (strcasecmp(drivers[game_index]->name,"dogosoke")==0) ||
         (strcasecmp(drivers[game_index]->name,"gwar")==0) || (strcasecmp(drivers[game_index]->name,"gwarj")==0) ||
         (strcasecmp(drivers[game_index]->name,"gwara")==0) || (strcasecmp(drivers[game_index]->name,"gwarb")==0) ||
         (strcasecmp(drivers[game_index]->name,"bermudat")==0) || (strcasecmp(drivers[game_index]->name,"bermudaj")==0) ||
         (strcasecmp(drivers[game_index]->name,"bermudaa")==0) || (strcasecmp(drivers[game_index]->name,"mplanets")==0) ||
         (strcasecmp(drivers[game_index]->name,"forgottn")==0) || (strcasecmp(drivers[game_index]->name,"lostwrld")==0) ||
         (strcasecmp(drivers[game_index]->name,"gondo")==0) || (strcasecmp(drivers[game_index]->name,"makyosen")==0) ||
         (strcasecmp(drivers[game_index]->name,"topgunr")==0) || (strcasecmp(drivers[game_index]->name,"topgunbl")==0) ||
         (strcasecmp(drivers[game_index]->name,"tron")==0) || (strcasecmp(drivers[game_index]->name,"tron2")==0) ||
         (strcasecmp(drivers[game_index]->name,"kroozr")==0) ||(strcasecmp(drivers[game_index]->name,"crater")==0) ||
         (strcasecmp(drivers[game_index]->name,"dotron")==0) || (strcasecmp(drivers[game_index]->name,"dotrone")==0) ||
         (strcasecmp(drivers[game_index]->name,"zwackery")==0) || (strcasecmp(drivers[game_index]->name,"ikari3")==0) ||
         (strcasecmp(drivers[game_index]->name,"searchar")==0) || (strcasecmp(drivers[game_index]->name,"sercharu")==0) ||
         (strcasecmp(drivers[game_index]->name,"timesold")==0) || (strcasecmp(drivers[game_index]->name,"timesol1")==0) ||
         (strcasecmp(drivers[game_index]->name,"btlfield")==0) || (strcasecmp(drivers[game_index]->name,"aztarac")==0))
    {
        extern int use_mouse;
	printf("Disabling mouse.\n");
        use_mouse=0;
    }

    /* go for it */
    printf ("%s (%s)...\n",drivers[game_index]->description,drivers[game_index]->name);
    res = run_game (game_index);

    /* close open files */
#if 0
    if (errorlog) fclose (errorlog);
    if (options.playback) osd_fclose (options.playback);
    if (options.record)   osd_fclose (options.record);
    if (options.language_file) osd_fclose (options.language_file);
#endif

    if (res!=0)
    {
	    /* wait a key press */
	    dingoo_joystick_press();
    }


	/* Exit to menu */
  return EXIT_SUCCESS;
}

void CLIB_DECL logerror(const char *text,...)
{
#if 0
    char strOut[512];
    va_list marker;

    va_start(marker, text);
    vsprintf(strOut, text, marker);
    va_end(marker);
    
    debug_printf(strOut);
#endif
}

void GamePathInit(const char* inPath) {
    uintptr_t i, j;
    for(i = 0, j = 0; inPath[i] != '\0'; i++) {
	    if((inPath[i] == '\\') || (inPath[i] == '/'))
		    j = i + 1;
    }
    strncpy(gameDir, inPath, j);
    gameDir[j] = '\0';
}

int main(int argc, char** argv) {
  int ref;
  debug_printf("Log file for %s\n", _app_path);
  GamePathInit(argv[0]);
  ref = mainMame (argc, argv);
#ifdef DEBUG_MALLOC
  printAddress();
#endif
  dingoo_deinit();
  //return ref;
  exit(ref);
}
