#include <sys/cachectl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sched.h>
#include <dirent.h>
#include "minimal.h"
#include "dingoo_native_version.h"
#include "dingoo_wrapper.h"
#include "dingoo/fsys.h"
#include "dingoo_frontend_list.h"
#include <time.h>
#include <sml/control.h>
#include <sml/timer.h>

#define CACHE_FLUSH dingoo_flip_video();
#define BITMAP_BUFFER bBuffer
#define BITMAP_SZ 77878
#define BITMAP_SPLASH "dingoosplash"
#define BITMAP_MENU "dingoomenu"

#if   MODULE_VectSega
#define MAMECFG "mameVectSega.cfg"
#define MAMELIST "mameVectSega.lst"
#elif   MODULE_Capcom
#define MAMECFG "mameCapcom.cfg"
#define MAMELIST "mameCapcom.lst"
#elif   MODULE_Konami
#define MAMECFG "mameKonami.cfg"
#define MAMELIST "mameKonami.lst"
#elif   MODULE_Classic
#define MAMECFG "mameClassic.cfg"
#define MAMELIST "mameClassic.lst"
#elif   MODULE_DeasIrem
#define MAMECFG "mameDeasIrem.cfg"
#define MAMELIST "mameDeasIrem.lst"
#elif   MODULE_AtarNamco
#define MAMECFG "mameAtarNamco.cfg"
#define MAMELIST "mameAtarNamco.lst"
#elif   MODULE_Others
#define MAMECFG "mameOthers.cfg"
#define MAMELIST "mameOthers.lst"
#elif   MODULE_Taito
#define MAMECFG "mameTaito.cfg"
#define MAMELIST "mameTaito.lst"
#elif   MODULE_Williams
#define MAMECFG "mameWilliams.cfg"
#define MAMELIST "mameWilliams.lst"
#elif   MODULE_Jaleco
#define MAMECFG "mameJaleco.cfg"
#define MAMELIST "mameJaleco.lst"
#elif   MODULE_Neogeo
#define MAMECFG "mameNeogeo.cfg"
#define MAMELIST "mameNeogeo.lst"
#elif   MODULE_M92
#define MAMECFG "mameM92.cfg"
#define MAMELIST "mameM92.lst"
#elif   MODULE_Darkmist
#define MAMECFG "mameDarkmist.cfg"
#define MAMELIST "mameDarkmist.lst"
#elif   MODULE_Test
#define MAMECFG "mameTest.cfg"
#define MAMELIST "mameTest.lst"
#endif


//unsigned char bBuffer[BITMAP_SZ];
unsigned char *bBuffer=0;

int game_num_avail = 0;
int dingoo_freq;
int dingoo_video_depth;
int dingoo_video_aspect;
int dingoo_video_sync;
int dingoo_brightness;
int dingoo_sound;
int dingoo_clock_cpu;
int dingoo_clock_sound;
int dingoo_cpu_cores;
int dingoo_volume;
int dingoo_cheat;
int last_game_selected;

static void alloc_bitmap_buffer()
{
    if (!BITMAP_BUFFER)
	BITMAP_BUFFER = malloc(BITMAP_SZ);
}


static void free_bitmap_buffer()
{
    if (BITMAP_BUFFER)
    {
	free(BITMAP_BUFFER);
	BITMAP_BUFFER = 0;
    }
}

static void draw_background() 
{
	int i,x,y;
	unsigned char r,g,b,c;
	unsigned char* in=BITMAP_BUFFER;
	unsigned short* out=dingoo_screen15;

	in+=14; /* Skip HEADER */
	in+=40; /* Skip INFOHD */
	/* Set Palette */
	for (i=0;i<DINGOO_PALETTE_SZ;i++) {
		b=in[0];
		g=in[1];
		r=in[2];
		c=in[3];
		in+=4;
		dingoo_video_color8(i,r,g,b);
	}
	/* Set Bitmap */	
	for (y=239;y>=0;y--)
	{
		for (x=0;x<DINGOO_SCREEN_WIDTH;x++)
		{
			*out++=dingoo_palette[in[x+(y*DINGOO_SCREEN_WIDTH)]];
		}
	}
}

static void load_background(char *filename)
{
	char bitmap_filename[256];
	FILE *f;

	alloc_bitmap_buffer();

	sprintf(bitmap_filename,"%sskins\\%s.bmp",gameDir, filename);
	f=fopen(bitmap_filename,"rb");
	if (f)
	{
	    if (fread(BITMAP_BUFFER,1,BITMAP_SZ,f) != BITMAP_SZ)
	    {
		printf("ERROR: Bad bitmap size. Should be 320x240 256 color Windows/OS2 format bitmap %d bytes\n", BITMAP_SZ);
	    }
	    fclose(f);
	}
	else
	{
	    printf("ERROR: Loading bitmap %s\n", filename);
	    memset(BITMAP_BUFFER, 0, BITMAP_SZ);
	}
	draw_background();
}

static void game_list_init_nocache(char *mamelist)
{
	int i;
	char game[32];
	char rom_filename[256];
	fsys_file_info_t fd;

	/* initialize */
	game_num_avail=0;
	for (i=0;i<NUMGAMES;i++)
	{
		driversfe[i].available=0;
	}

	/* Search for ROMS as .zip files */
	sprintf(rom_filename, "%sroms\\*.zip",gameDir);
	if (fsys_findfirst(rom_filename, FSYS_FIND_FILE, &fd) == 0)
	{
	    do
	    { 
		for (i=0;i<NUMGAMES;i++)
		{
		    if (driversfe[i].available==0)
		    {
			sprintf(game,"%s.zip",driversfe[i].name);
			if (strcasecmp(fd.name,game)==0)
			{
			    driversfe[i].available=1;
			    game_num_avail++;
			    break;
			}
		    }
		}
	    } while (fsys_findnext(&fd) == 0);
	}
	fsys_findclose(&fd);

	/* Search for ROMS as directory names */
	sprintf(rom_filename, "%sroms\\*",gameDir);
	if (fsys_findfirst(rom_filename, FSYS_FIND_DIR, &fd) == 0)
	{
	    do
	    { 
		for (i=0;i<NUMGAMES;i++)
		{
		    if (driversfe[i].available==0)
		    {
			if (strcasecmp(fd.name,driversfe[i].name)==0)
			{
			    driversfe[i].available=1;
			    game_num_avail++;
			    break;
			}
		    }
		}
	    } while (fsys_findnext(&fd) == 0);
	}
	fsys_findclose(&fd);

	printf("Number of games found for this module=%d\n", game_num_avail);

	/* Write out the cache file */
	if (game_num_avail)
	{
	    FILE *f;
		remove(mamelist);
		f=fopen(mamelist,"w");
		if (f)
		{
			for (i=0;i<NUMGAMES;i++)
			{
				fputc(driversfe[i].available,f);
			}
			fclose(f);
		}
	}
	else
	{
		printf("No games found or Update Jukebox\n");
	}
}

static void game_list_init_cache(char *mamelist)
{
	FILE *f;
	int i;
	struct stat buf;
	f=fopen(mamelist,"r");
	if (f)
	{
		stat(mamelist, &buf);
		if (buf.st_size == NUMGAMES)
		{
			for (i=0;i<NUMGAMES;i++)
			{
				driversfe[i].available=fgetc(f);
				if (driversfe[i].available)
					game_num_avail++;
			}
			fclose(f);
			return; /* Success */
		}
	}

	/* If the wrong size or not exist, create a new cache file */
	game_list_init_nocache(mamelist);
}

static void game_list_init(int use_cache, char *mamelist)
{
	if (use_cache)
		game_list_init_cache(mamelist);
	else
		game_list_init_nocache(mamelist);
}

static void game_list_view(char* game, char* desc) {

	int i;
	int view_pos;
	int aux_pos=0;
	int screen_y = 45;
	int screen_x = 38;

	/* Draw background image */
	draw_background();

	/* Set View Pos */
	if (last_game_selected<10) {
		view_pos=0;
	} else {
		if (last_game_selected>game_num_avail-11) {
			view_pos=game_num_avail-21;
			view_pos=(view_pos<0?0:view_pos);
		} else {
			view_pos=last_game_selected-10;
		}
	}

	/* Show List */
	for (i=0;i<NUMGAMES;i++) {
		if (driversfe[i].available==1) {
			if (aux_pos>=view_pos && aux_pos<=view_pos+20) {
				dingoo_gamelist_text_out( screen_x, screen_y, driversfe[i].description);
				if (aux_pos==last_game_selected) {
					dingoo_gamelist_text_out( screen_x-10, screen_y,">" );
					dingoo_gamelist_text_out( screen_x-13, screen_y-1,"-" );
					strcpy(game,driversfe[i].name);
					strcpy(desc,driversfe[i].description);
				}
				screen_y+=8;
			}
			aux_pos++;
		}
	}
}

#if 0
static void game_list_select (int index, char *game) {
	int i;
	int aux_pos=0;
	for (i=0;i<NUMGAMES;i++)
	{
		if (driversfe[i].available==1)
		{
			if(aux_pos==index)
			{
				strcpy(game,driversfe[i].name);
				break;
			}
			aux_pos++;
		}
	}
}

static char *game_list_description (int index)
{
	int i;
	int aux_pos=0;
	for (i=0;i<NUMGAMES;i++) {
		if (driversfe[i].available==1) {
			if(aux_pos==index) {
				return(driversfe[i].description);
			}
			aux_pos++;
		   }
	}
	return ((char *)0);
}
#endif

static int show_options(char *game, char *desc)
{
	unsigned long ExKey=0;
	int selected_option=0;
	int x_Pos = 41;
	int y_Pos = 38;
	unsigned int options_count = 9;
	char config_filename[128];
	char text[128];
	FILE *f;

	/* Set game configuration name */
	sprintf(config_filename,"%sfrontend\\%s.cfg",gameDir, game);

	/* Read game configuration */
	f=fopen(config_filename,"r");
	if (f)
	{
	    int i=0; // dummy
	    fread(&dingoo_freq,sizeof(int),1,f);
	    fread(&dingoo_video_depth,sizeof(int),1,f);
	    fread(&dingoo_video_aspect,sizeof(int),1,f);
	    fread(&dingoo_video_sync,sizeof(int),1,f);
	    fread(&dingoo_brightness,sizeof(int),1,f);
	    fread(&dingoo_sound,sizeof(int),1,f);
	    fread(&dingoo_clock_cpu,sizeof(int),1,f);
	    fread(&dingoo_clock_sound,sizeof(int),1,f);
	    fread(&dingoo_cpu_cores,sizeof(int),1,f);
	    fread(&dingoo_volume,sizeof(int),1,f);
	    fread(&i,sizeof(int),1,f);
	    fread(&dingoo_cheat,sizeof(int),1,f);
	    fclose(f);
	}
	else
	{
	    printf("Using MAME4ALL default config\n");
	    dingoo_freq=336;		/* Default dingoo Mhz */
	    dingoo_video_depth=8;	/* MAME video depth */
	    dingoo_video_aspect=0;	/* Normal, no scaling or rotation */
	    dingoo_video_sync=0;	/* No vsync */
	    dingoo_brightness=70;	/* Screen Brightness */
	    dingoo_sound = 6;		/* 16000hz, Mono */
	    dingoo_clock_cpu=100;	/* Emulated Main CPU clock % */
	    dingoo_clock_sound=100;	/* Emulated Sound CPU clock % */
	    dingoo_cpu_cores=1;		/* Fast FAME M68K core */
	    dingoo_volume=3;		/* Dingoo Volume = Medium */
	    dingoo_cheat=0;		/* Disable MAME cheat modes */
	}

	while(1)
	{
		/* Draw background image */
		draw_background();

		/* Draw the options */
		strncpy (text,desc,33);
		text[32]='\0';
		dingoo_gamelist_text_out(x_Pos,y_Pos+10,text);
		dingoo_gamelist_text_out(x_Pos,y_Pos+20,game);

		/* (0) Dingoo Clock */
		dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+40, "Dingoo Clock  %d MHz", dingoo_freq);

		/* (1) Video Depth */
		switch (dingoo_video_depth)
		{
			case -1: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+50,"Video Depth   Auto"); break;
			case 8:  dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+50,"Video Depth   8-bit"); break;
			case 16: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+50,"Video Depth   16-bit"); break;
		}

		/* (2) Video Aspect */
		switch (dingoo_video_aspect)
		{
			case 0: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Normal"); break;
			case 1: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Horiz Resize"); break;
			case 2: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Scale Quality"); break;
			case 3: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Scale Fast"); break;
			case 4: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Scale Halfsize"); break;
			case 5: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Rot90"); break;
			case 6: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Rot90 Vert Resize"); break;
			case 7: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Rot90 Scale Qlty"); break;
			case 8: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Rot90 Scale Fast"); break;
			case 9: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+60,"Video Aspect  Rot90 Halfsize"); break;
		}
		
		/* (3) Video Sync */
		switch (dingoo_video_sync)
		{
			case 0: dingoo_gamelist_text_out(x_Pos,y_Pos+70, "Video Sync    Normal"); break;
			case 1: dingoo_gamelist_text_out(x_Pos,y_Pos+70, "Video Sync    DblBuf"); break;
			case -1: dingoo_gamelist_text_out(x_Pos,y_Pos+70,"Video Sync    OFF"); break;
		}
		
		/* (4) Screeen Brightness REMOVED */
		//dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+70,"Brightness    %d%%",dingoo_brightness);

		/* (5) Sound - Max is 48000hz, multiples of this is best */
		switch(dingoo_sound)
		{
			case 0: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","OFF"); break;
			case 1: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 8000hz fast"); break;
			case 2: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 16000hz fast"); break;
			case 3: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 24000hz fast"); break;
			case 4: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 48000hz fast"); break;
			case 5: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 8000hz mono"); break;
			case 6: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 16000hz mono"); break;
			case 7: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 24000hz mono"); break;
			case 8: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 48000hz mono"); break;
			case 9: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 8000hz stereo"); break;
			case 10: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 16000hz stereo"); break;
			case 11: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 24000hz stereo"); break;
			case 12: dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+80,"Sound         %s","ON 48000hz stereo"); break;
		}

		/* (6) CPU Clock */
		dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+90,"CPU Clock     %d%%",dingoo_clock_cpu);

		/* (7) Audio Clock */
		dingoo_gamelist_text_out_fmt(x_Pos,y_Pos+100,"Audio Clock   %d%%",dingoo_clock_sound);

		/* (8) Volume */
		if (dingoo_sound == 0)
		{
			dingoo_gamelist_text_out(x_Pos,y_Pos+110,"Volume        OFF");
		}
		else
		{
			switch (dingoo_volume)
			{
				case 4: dingoo_gamelist_text_out(x_Pos,y_Pos+110,"Volume        Max"); break;
				case 3: dingoo_gamelist_text_out(x_Pos,y_Pos+110,"Volume        Medium"); break;
				case 2: dingoo_gamelist_text_out(x_Pos,y_Pos+110,"Volume        Low"); break;
				case 1: dingoo_gamelist_text_out(x_Pos,y_Pos+110,"Volume        Quiet"); break;
			}
		}

		/* (9) Cheats */
		if (dingoo_cheat)
			dingoo_gamelist_text_out(x_Pos,y_Pos+120,"Cheats        ON");
		else
			dingoo_gamelist_text_out(x_Pos,y_Pos+120,"Cheats        OFF");
	
		dingoo_gamelist_text_out(x_Pos,y_Pos+150,"Press A to confirm, B to return\0");

		/* (10) CPU cores MOVED */
#ifdef MODULE_Taito
		options_count = 10;
		if (dingoo_cpu_cores == 2) dingoo_cpu_cores = 1;
		switch (dingoo_cpu_cores)
		{
			case 0: dingoo_gamelist_text_out(x_Pos,y_Pos+130, "M68000 core   Musashi Compatible"); break;
			case 1: dingoo_gamelist_text_out(x_Pos,y_Pos+130, "M68000 core   FAME Fast"); break;
		}
#endif
		/* Show currently selected item */
		dingoo_gamelist_text_out(x_Pos-16,y_Pos+(selected_option*10)+40," >");

		CACHE_FLUSH
		while(dingoo_joystick_read(1)) { dingoo_timer_delay(DINGOO_DELAY); }
		while(!(ExKey=dingoo_joystick_read(1))) { /*sched_yield();*/ }
		if(ExKey & DINGOO_DOWN){
			selected_option++;
			selected_option = selected_option % options_count;
		}
		else if(ExKey & DINGOO_UP){
			selected_option--;
			if(selected_option<0)
				selected_option = options_count - 1;
		}
		else if(ExKey & DINGOO_RIGHT || ExKey & DINGOO_LEFT)
		{
		    switch(selected_option)
		    {
		    case 0:
			    /* Dingoo Clock */
			    if(ExKey & DINGOO_RIGHT){
				    switch (dingoo_freq) {
					    case 300: dingoo_freq=312;break;
					    case 312: dingoo_freq=324;break;
					    case 324: dingoo_freq=336;break;
					    case 336: dingoo_freq=348;break;
					    case 348: dingoo_freq=360;break;
					    case 360: dingoo_freq=372;break;
					    case 372: dingoo_freq=384;break;
					    case 384: dingoo_freq=396;break;
					    case 396: dingoo_freq=408;break;
					    case 408: dingoo_freq=420;break;
					    case 420: dingoo_freq=432;break;
					    case 432: dingoo_freq=444;break;
					    case 444: dingoo_freq=456;break;
					    case 456: dingoo_freq=300;break;
				    }
			    } else {
				    switch (dingoo_freq) {
					    case 456: dingoo_freq=444;break;
					    case 444: dingoo_freq=432;break;
					    case 432: dingoo_freq=420;break;
					    case 420: dingoo_freq=408;break;
					    case 408: dingoo_freq=396;break;
					    case 396: dingoo_freq=384;break;
					    case 384: dingoo_freq=372;break;
					    case 372: dingoo_freq=360;break;
					    case 360: dingoo_freq=348;break;
					    case 348: dingoo_freq=336;break;
					    case 336: dingoo_freq=324;break;
					    case 324: dingoo_freq=312;break;
					    case 312: dingoo_freq=300;break;
					    case 300: dingoo_freq=456;break;
				    }
			    }
			    break;
		    case 1:
			    switch (dingoo_video_depth)
			    {
				    case -1: dingoo_video_depth=8; break;
				    case 8: dingoo_video_depth=16; break;
				    case 16: dingoo_video_depth=-1; break;
			    }
			    break;
		    case 2:
			    if(ExKey & DINGOO_RIGHT)
			    {
				    dingoo_video_aspect++;
				    if (dingoo_video_aspect>9)
					    dingoo_video_aspect=0;
			    }
			    else
			    {
				    dingoo_video_aspect--;
				    if (dingoo_video_aspect<0)
					    dingoo_video_aspect=9;
			    }
			    break;
		    case 3:
			    dingoo_video_sync=dingoo_video_sync+1;
			    if (dingoo_video_sync>1)
				    dingoo_video_sync=-1;
			    break;
		    case 4:
			    if(ExKey & DINGOO_RIGHT)
			    {
				    dingoo_sound ++;
				    if (dingoo_sound>12)
					    dingoo_sound=0;
			    }
			    else
			    {
				    dingoo_sound--;
				    if (dingoo_sound<0)
					    dingoo_sound=12;
			    }
			    break;
		    case 5:
			    /* "CPU Clock" */
			    if(ExKey & DINGOO_RIGHT)
			    {
				    dingoo_clock_cpu += 10; /* Add 10% */
				    if (dingoo_clock_cpu > 200) /* 200% is the max */
					    dingoo_clock_cpu = 200;
			    }
			    else
			    {
				    dingoo_clock_cpu -= 10; /* Subtract 10% */
				    if (dingoo_clock_cpu < 10) /* 10% is the min */
					    dingoo_clock_cpu = 10;
			    }
			    break;
		    case 6:
			    /* "Audio Clock" */
			    if(ExKey & DINGOO_RIGHT)
			    {
				    dingoo_clock_sound += 10; /* Add 10% */
				    if (dingoo_clock_sound > 200) /* 200% is the max */
					    dingoo_clock_sound = 200;
			    }
			    else{
				    dingoo_clock_sound -= 10; /* Subtract 10% */
				    if (dingoo_clock_sound < 10) /* 10% is the min */
					    dingoo_clock_sound = 10;
			    }
			    break;
		    case 7:
			    /* Volume */
			    if(ExKey & DINGOO_RIGHT)
			    {
				    dingoo_volume++;
				    if (dingoo_volume > 4)
					    dingoo_volume = 1;
			    }
			    else {
				    dingoo_volume--;
				    if (dingoo_volume < 1)
					    dingoo_volume = 4;
			    }
			    break;
		    case 8:
			    dingoo_cheat=!dingoo_cheat;
			    break;
		    case 9:
			    dingoo_cpu_cores=(!dingoo_cpu_cores);
			    break;
		    }
		}

		if ((ExKey & DINGOO_A) || (ExKey & DINGOO_X) || (ExKey & DINGOO_START)) 
		{
			dingoo_gamelist_text_out(x_Pos,y_Pos+170,"Starting game...\0");
			CACHE_FLUSH

			if (ExKey & DINGOO_START)
			{
				/* Do not write game config in profile mode */
			}
			else
			{
			    /* Write game configuration */
			    f=fopen(config_filename,"w");
			    if (f) {
			      fwrite(&dingoo_freq,sizeof(int),1,f);
			      fwrite(&dingoo_video_depth,sizeof(int),1,f);
			      fwrite(&dingoo_video_aspect,sizeof(int),1,f);
			      fwrite(&dingoo_video_sync,sizeof(int),1,f);
			      fwrite(&dingoo_brightness,sizeof(int),1,f);
			      fwrite(&dingoo_sound,sizeof(int),1,f);
			      fwrite(&dingoo_clock_cpu,sizeof(int),1,f);
			      fwrite(&dingoo_clock_sound,sizeof(int),1,f);
			      fwrite(&dingoo_cpu_cores,sizeof(int),1,f);
			      fwrite(&dingoo_volume,sizeof(int),1,f);
			      fwrite(&last_game_selected,sizeof(int),1,f); // Not used though
			      fwrite(&dingoo_cheat,sizeof(int),1,f);
			      fclose(f);
			    }
			}

			/* Selected game will be run */
			return 1;
		}
		else if ((ExKey & DINGOO_B) || (ExKey & DINGOO_Y) || (ExKey & DINGOO_SELECT))
		{
			/* Return To Menu */
			return 0;
		}
	}
}

static int select_game(char *game, char *mamelist)
{
    char desc[96];

    /* No Selected game */
    game[0]='\0';
    desc[0]='\0';

    /* load background */
    load_background(BITMAP_MENU);
  
    /* Wait until user selects a game */
    while(1)
    {
	unsigned long ExKey;
	game_list_view(game,desc);
	CACHE_FLUSH

	if( dingoo_joystick_read(1)) dingoo_timer_delay(DINGOO_DELAY); 
	while(!(ExKey=dingoo_joystick_read(1))) { dingoo_timer_delay(1); }
	/* Check for game selection movement */
	if (ExKey & DINGOO_UP) last_game_selected--;
	if (ExKey & DINGOO_DOWN) last_game_selected++;
	if (ExKey & DINGOO_LEFT) last_game_selected-=21;
	if (ExKey & DINGOO_RIGHT) last_game_selected+=21;

	/* Check Limits */
	if (last_game_selected<0)
		last_game_selected=game_num_avail-1;
	if (last_game_selected>(game_num_avail-1))
		last_game_selected=0;
					   
	if ((ExKey & DINGOO_L) && (ExKey & DINGOO_R)) return 1; /* exit */

	if ((ExKey & DINGOO_SELECT))	/* Regenerate list cache files */
	{
	    /* Draw background image */
	    load_background(BITMAP_SPLASH);
	    CACHE_FLUSH
	    game_list_init_nocache(mamelist);
	    last_game_selected=0;
	    dingoo_gamelist_text_out(114, 225, "PRESS A KEY");
	    CACHE_FLUSH
	    dingoo_joystick_press();
	    load_background(BITMAP_MENU);
	}

	if ((ExKey & DINGOO_A) || (ExKey & DINGOO_START))
	{
	    /* Select the game */
	    //game_list_select(last_game_selected, game);

	    /* Emulation Options */
	    if(show_options(game, desc))
	    {
		    break;
	    }
	}
    }
  return 0; /* game selected */
}

void execute_game (char *execute_game)
{
	char *args[32];
	char str[12][64];
	int n=0;
	int i=0;

	/* executable */
	strcpy(str[i],"mame");
	args[n]=str[i]; i++; n++;
	
	/* game to play */
	args[n]=execute_game; n++;

	/* dingoo_freq */
	args[n]="-clock"; n++;
	sprintf(str[i],"%d",dingoo_freq);
	args[n]=str[i]; i++; n++;

	/*
	 * This is NOT the Dingoo's video depth!
	 * This is MAME's internal video depth.
	 * The Dingoo LCD minimum color display is 16-bit so MAME's 8-bit video
	 * is mapped to Dingoo's 16-bit when blitting (see blit.c).
	 */
	if (dingoo_video_depth==8)
	{
		args[n]="-depth"; n++;
		args[n]="8"; n++;
	}
	if (dingoo_video_depth==16)
	{
		args[n]="-depth"; n++;
		args[n]="16"; n++;
	}

	/* dingoo_video_aspect */
	if ((dingoo_video_aspect==1) || (dingoo_video_aspect==6))
	{
		args[n]="-horizscale"; n++;
		args[n]="-nodirty"; n++;
	}
	if ((dingoo_video_aspect==2) || (dingoo_video_aspect==7))
	{
		args[n]="-bestscale"; n++;
		args[n]="-nodirty"; n++;
	}
	if ((dingoo_video_aspect==3) || (dingoo_video_aspect==8))
	{
		args[n]="-fastscale"; n++;
		args[n]="-nodirty"; n++;
	}
	if ((dingoo_video_aspect==4) || (dingoo_video_aspect==9))
	{
		args[n]="-halfscale"; n++;
		args[n]="-nodirty"; n++;
	}
	if ((dingoo_video_aspect>=5) && (dingoo_video_aspect<=9))
	{
		args[n]="-rotatecontrols"; n++;
		args[n]="-ror"; n++;
	}
	
	/* dingoo_video_sync */
	if (dingoo_video_sync==1)
	{
		args[n]="-nodirty"; n++;
		args[n]="-dblbuf"; n++;
	}
	else if (dingoo_video_sync==-1)
	{
		args[n]="-nothrottle"; n++;
	}
	
	/* dingoo_brightness */
	if (dingoo_brightness)
	{
		args[n]="--brightness"; n++;
		sprintf(str[i],"%d",dingoo_brightness);
		args[n]=str[i]; i++; n++;
	}

	/* dingoo_sound */
	if (dingoo_sound==0)
	{
		args[n]="-soundcard"; n++;
		args[n]="0"; n++;
	}
	if ((dingoo_sound==1) || (dingoo_sound==5) || (dingoo_sound==9))
	{
		args[n]="-samplerate"; n++;
		args[n]="8000"; n++;
	}
	if ((dingoo_sound==2) || (dingoo_sound==6) || (dingoo_sound==10))
	{
		args[n]="-samplerate"; n++;
		args[n]="16000"; n++;
	}
	if ((dingoo_sound==3) || (dingoo_sound==7) || (dingoo_sound==11))
	{
		args[n]="-samplerate"; n++;
		args[n]="24000"; n++;
	}
	if ((dingoo_sound==4) || (dingoo_sound==8) || (dingoo_sound==12))
	{
		args[n]="-samplerate"; n++;
		args[n]="48000"; n++;
	}
	if ((dingoo_sound>=1) && (dingoo_sound<=4))
	{
		args[n]="-fastsound"; n++;
	}
	if (dingoo_sound>=9)
	{
		args[n]="-stereo"; n++;
	}

	/* dingoo_clock_cpu */
	if (dingoo_clock_cpu!=100)
	{
		args[n]="-uclock"; n++;
		sprintf(str[i],"%d",100-dingoo_clock_cpu);
		args[n]=str[i]; i++; n++;
	}

	/* dingoo_clock_sound */
	if (dingoo_clock_cpu!=100)
	{
		args[n]="-uclocks"; n++;
		sprintf(str[i],"%d",100-dingoo_clock_sound);
		args[n]=str[i]; i++; n++;
	}
	
	/* dingoo_cpu_cores */
	if (dingoo_cpu_cores==1)
	{
		args[n]="-fame"; n++;
	}

	switch (dingoo_volume)
	{
		case 4: break; /* nothing, default to maximum volume */
		case 3: args[n]="-volume"; n++; args[n]="-4"; n++; break;
		case 2: args[n]="-volume"; n++; args[n]="-8"; n++; break;
		case 1: args[n]="-volume"; n++; args[n]="-10"; n++; break;
	}
	
	if (dingoo_cheat)
	{
		args[n]="-cheat"; n++;
	}

	args[n]=NULL;
	
#if 0
	for (i=0; i<n; i++)
	{
		printf("%s ",args[i]);
	}
	printf("\n");
#endif

	main_run_game (n,args);
}


int mainMame (int argc, char **argv)
{
    FILE *f;
    char config_filename[128];
    char list_filename[128];
    char playgame[16];

    /* Cache list file */
    sprintf(list_filename,"%sfrontend\\%s",gameDir, MAMELIST);

    /* Dingoo Initialization */
    dingoo_init(DINGOO_DEFAULT_MHZ, 0);

    /* Show intro screen */
    load_background(BITMAP_SPLASH);
    CACHE_FLUSH

    /* Initialize list of available games */
    game_list_init(1, list_filename);

    /* Write version string and press any key */
    dingoo_gamelist_text_out(16, 2, DINGOO_NATIVE_VERSION);

    if (game_num_avail==0)
    {
	dingoo_gamelist_text_out(108, 225, "NO GAMES FOUND");
	CACHE_FLUSH
	dingoo_joystick_press();
    }
    else
    {
	dingoo_gamelist_text_out(114, 225, "PRESS A KEY");
	CACHE_FLUSH
	dingoo_joystick_press();

	/* Get last game played configuration */
	last_game_selected=0;
	sprintf(config_filename,"%sfrontend\\%s",gameDir, MAMECFG);
	f=fopen(config_filename,"r");
	if (f)
	{
	    fread(&last_game_selected,sizeof(int),1,f);
	    fclose(f);
	    if (last_game_selected >= game_num_avail)
		last_game_selected=0;
	}


	/* Select Game */
	while(1)
	{
	    int res = select_game(playgame, list_filename); 

	    /* Save last game played configuration */
	    f=fopen(config_filename,"w");
	    if (f)
	    {
		fwrite(&last_game_selected,sizeof(int),1,f);
		fclose(f);
	    }

	    /* Write configuration */
	    if (!res)
	    {
		/* Execute Game */
		free_bitmap_buffer();
		execute_game (playgame);
	    
		/* wait releasing key */
		while(dingoo_joystick_read(1)) { dingoo_timer_delay(DINGOO_DELAY); }
	    }
	    else
	    {
		free_bitmap_buffer();
		break;	/* exit from game selection loop */
	    }
	}
    }

  return EXIT_SUCCESS;
}
