#include <stdlib.h>
#include <stdio.h>

#define MIN_ALLOC_SIZE	(1024*1024*1)
#define EXTRA_MEMORY_SZ (1024*1024*2)	// default, can be overridden by EXTRA_MEM_SIZE
#define HEAP_START_ADDR	0x81000000
#ifdef EXTRA_MEM_SIZE
#undef EXTRA_MEMORY_SZ
#define EXTRA_MEMORY_SZ (EXTRA_MEM_SIZE)
#endif
#ifdef EXTRA_MEM_MIN_SIZE
#undef MIN_ALLOC_SIZE
#define MIN_ALLOC_SIZE (EXTRA_MEM_MIN_SIZE)
#endif
#define MAX_ALLOCATIONS (EXTRA_MEMORY_SZ/MIN_ALLOC_SIZE)

// Dingoo Native A320 allocates a maximum of
// //   6MB to static declarations ie. program and other statically declared types
// //  10MB to dynamic memory heap ie. malloc()
// //
// // ie.
// // Program starts as 0x80A00000
// // mallocs start at  0x81000000
// //
//
static unsigned char   *dingoo_extra_memory_ptr;
static unsigned int	dingoo_extra_memory_remaining;
static unsigned int	dingoo_extra_memory_table[MAX_ALLOCATIONS];
unsigned char	dingoo_extra_memory[EXTRA_MEMORY_SZ]  __attribute__ ((__aligned__ (32)));  // Extra RAM :)

static int _insertAllocation(void * ptr, unsigned int size)
{
    unsigned int i;
    for (i=0; i<MAX_ALLOCATIONS; i++)
    {
	    if (dingoo_extra_memory_table[i] == 0)
	    {
		    dingoo_extra_memory_table[i] = (unsigned int)ptr;
		    debug_printf("Extra memory allocation pos=%u ptr=%x size=%u\n", i, ptr, size);
		    break;
	    }
    }
    return i;
}

static inline int _allFree()
{
    unsigned int i;
    for (i=0; i<MAX_ALLOCATIONS; i++)
    {
	if (dingoo_extra_memory_table[i] != 0)
	{
	    return 0;
	}
    }
    return 1;
}

static inline int _freeAllocation(void * ptr)
{
    unsigned int i;
    for (i=0; i<MAX_ALLOCATIONS; i++)
    {
	if (dingoo_extra_memory_table[i] == (unsigned int)ptr)
	{
	    debug_printf("Extra memory free pos=%u ptr=%x\n", i, ptr);
	    dingoo_extra_memory_table[i] = 0;
	    if (_allFree())
	    {
		dingoo_init_malloc();
	    }
	    return 1;
	}
    }
    return 0;
}


static inline void _findRemaining(unsigned int size)
{
    unsigned int i;
    unsigned char * ptr;
    for (i=0; i<MAX_ALLOCATIONS; i++)
    {
	size-=MIN_ALLOC_SIZE;
	ptr=_malloc(size);
	if (ptr)
	{
	    debug_printf("Remaining memory = %u\n", size);
	    free(ptr);
	    return;
	}
    }
    debug_printf("Close to zero remaining memory\n");
}


void dingoo_init_malloc(void)
{
	
	memset(dingoo_extra_memory_table, 0, MAX_ALLOCATIONS*sizeof(unsigned int));
	dingoo_extra_memory_remaining = EXTRA_MEMORY_SZ;
	dingoo_extra_memory_ptr = &dingoo_extra_memory[0];
	debug_printf("Extra Dingoo RAM size = %u\n", EXTRA_MEMORY_SZ);
}

void * dingoo_malloc(const char *func, unsigned int size)
{
	void *ptr;

#if 0
	// Attempt to allocate various possible unused areas in memory map unsuccessful
	static int test_alloc=0;
	if (size == (1024*1024*4) && (test_alloc == 0))
	{
	    ptr=0x80600000;
	    test_alloc=1;
	}
	else
#endif

	if (size >= (MIN_ALLOC_SIZE) && dingoo_extra_memory_remaining >= size)
	{
	    dingoo_extra_memory_remaining -= size;
	    ptr = dingoo_extra_memory_ptr;
	    dingoo_extra_memory_ptr += size;
	    _insertAllocation(ptr, size);
	}
	else
	{
	    ptr=_malloc(size);
	}

	if (ptr)
	{
#ifdef EXTRA_DEBUG
		debug_printf("%s() size=%u at %x\n", func, size, ptr);
#endif
		return (ptr);
	}
	else
	{
		_findRemaining(size);
		debug_printf("ERROR: dingoo_malloc() fail size=%u\n", size);
		return((void *)0);
	}
}

void dingoo_free(void * ptr)
{
	if (ptr)
	{
		if(!_freeAllocation(ptr))
		{
		    _free(ptr);
		}
	}
}
