#include <stdlib.h>
#include <stdio.h>

static unsigned int mAddress[256];

unsigned int insertAddress(void * ptr)
{
    unsigned int i;

    /* check for dupes */
    for (i=0; i<1024; i++)
    {
	    if (mAddress[i] == (unsigned int)ptr)
		    debug_printf("MALLOC: Already allocated %x\n", ptr);
    }

    /* Allocate */
    for (i=0; i<1024; i++)
    {
	    if (mAddress[i] == 0)
	    {
		    mAddress[i] = (unsigned int)ptr;
		    break;
	    }
    }
    if (i==1024) debug_printf("MALLOC: Out of space in malloc list for %x\n", ptr);
    return i;
}

unsigned int removeAddress(void * ptr)
{
    unsigned int i;
    for (i=0; i<1024; i++)
    {
	    if (mAddress[i] == (unsigned int)ptr)
	    {
		    mAddress[i] = 0;
		    break;
	    }
    }
    if (i==1024) debug_printf("MALLOC: Not in malloc list %x\n", ptr);
    return i;

}

void printAddress(void)
{
    unsigned int i;
    for (i=0; i<1024; i++)
    {
	    if (mAddress[i] != 0)
	    {
		    debug_printf("MALLOC: Ptr %x\n", mAddress[i]);
		    break;
	    }
    }
}

void * debug_malloc(unsigned int size, const char *func)
{
	void *ptr=(void *)0;

	
#if 0
	// Extra memory hack attempt
	static i = 0;
	if (size == 2196480 && i == 0) // megasys1.c needs 2 chunks of this sized memory.
	{
	    extern unsigned int * alloc(unsigned int);
	    ptr=alloc(size);
	    i++;
	}
	else
#endif
	ptr=_malloc(size);

	if (ptr)
	{
		insertAddress(ptr);
		debug_printf ("%s malloc %d at %x\n", func, size, ptr);
		return (ptr);
	}
	else
	{
		debug_printf ("%s malloc %d failed\n", func, size);
		return((void *)0);
	}
}

void debug_free(void * ptr, const char *func)
{
	debug_printf ("%s free at %x\n", func, ptr);
	if (ptr)
	{
		removeAddress(ptr);
		_free(ptr);
	}
	else
	{
		debug_printf("free failed\n");
	}
}

void * debug_realloc(void * oldptr, unsigned int size, const char *func)
{
	void *newptr=(void *)0;

	removeAddress(oldptr);

	newptr=_realloc(oldptr, size);

	if (newptr)
	{
		insertAddress(newptr);
		debug_printf ("%s realloc %d from %x to %x\n", func, size, oldptr, newptr);
		return (newptr);
	}
	else
	{
		debug_printf ("%s realloc %d failed\n", func, size);
		return((void *)0);
	}
}

void * debug_calloc(unsigned int nbr, unsigned int size, const char *func)
{
	void *ptr=(void *)0;

	ptr=_malloc(nbr*size);

	if (ptr)
	{
		memset(ptr,0,nbr*size);
		insertAddress(ptr);
		debug_printf ("%s malloc %d at %x\n", func, size, ptr);
		return (ptr);
	}
	else
	{
		debug_printf ("%s malloc %d failed\n", func, size);
		return((void *)0);
	}
}

void * regular_calloc(unsigned int nbr, unsigned int size)
{
	void *ptr=(void *)0;

	ptr=_malloc(nbr*size);

	if (ptr)
	{
		memset(ptr,0,nbr*size);
		return (ptr);
	}
	else
	{
		return((void *)0);
	}
}
