#include "dingoo_sound.h"

OS_STK	TaskStkPlay[TASK_STK_SIZE_PLAY];
static	waveout_inst*	woHandle = 0;
static  volatile  int curReadBuffer;
static  volatile  unsigned int soundThreadPause = 1;;
static  volatile  unsigned int ReadBufferID = 0;
static  volatile  unsigned int WriteBufferID = 0;
static  volatile  unsigned int dingooSoundStarted = 0;
static  volatile  unsigned int soundThreadExitReq = 0;
unsigned int dingooSoundVolume;
unsigned int nbr_samples_per_frame;
unsigned int nbr_channels;


static short dingooSoundBufferTotal[SND_BUFFER_SAMPLES * SND_NBR_BUFFERS];// Sound buffer
void *dingooSoundBuffers[SND_NBR_BUFFERS] = {	    // Sound buffer fragments
	(void *)(dingooSoundBufferTotal+(SND_BUFFER_SAMPLES*0)),
	(void *)(dingooSoundBufferTotal+(SND_BUFFER_SAMPLES*1)),
	(void *)(dingooSoundBufferTotal+(SND_BUFFER_SAMPLES*2)),
	(void *)(dingooSoundBufferTotal+(SND_BUFFER_SAMPLES*3))
};
static short silence[SND_BUFFER_SAMPLES];// Sound buffer

void dingoo_sound_play(void *stream, int len)
{
	static int prev = -1;
	while (prev == ReadBufferID) { /* wait until ReadBufferID changes; updated in _dingooSoundPlay() */ }
	unsigned int buff = (curReadBuffer+1)&(SND_NBR_BUFFERS-1);
	buff = (buff+1)&(SND_NBR_BUFFERS-1); 
	memcpy(dingooSoundBuffers[buff],stream,len);
	prev = ReadBufferID;
	WriteBufferID++;
}

int dingoo_sound_thread_start(unsigned int sample_rate, unsigned int fps, int is_stereo)
{
    dingooSoundStarted = 0;
    WriteBufferID = 0;
    curReadBuffer = 0;
    soundThreadExitReq = 0;
    soundThreadPause = 0;
    dingooSoundClearBuffers();

    if (is_stereo) is_stereo = 1; // ensure it's 0 or 1
    nbr_channels = is_stereo + 1;
    if (nbr_channels > SND_MAX_CHANNELS) nbr_channels = SND_MAX_CHANNELS;
    nbr_samples_per_frame = sample_rate / fps;

    printf("Start sound thread samples:%d fps:%d channels:%d\n", sample_rate, fps, nbr_channels);

    waveout_args woArgs = {
	sample_rate,		// frequency
	SND_BIT_SZ,		// 16
	nbr_channels,		// nbr of channels 
	dingooSoundVolume	// init volume from config.c (frontend selects starting volume)
    };

    woHandle = waveout_open(&woArgs);
    if (!woHandle)
    {
	printf("ERROR: Could not open waveout device\n");
	return 0;
    }

    memset(TaskStkPlay, 0, TASK_STK_SIZE_PLAY * sizeof(OS_STK));

    if ( OSTaskCreate(_dingooSoundPlay, (void *)0,  (void *)&TaskStkPlay[TASK_STK_SIZE_PLAY -1], TASK_START_PRIO) != OS_NO_ERR)
    {
	printf("ERROR: Could not create Dingoo Sound Play thread\n");
	return 0;
    }
    
    dingooSoundStarted = 1;

    return nbr_samples_per_frame;
}


void dingoo_sound_thread_stop()
{
    if (dingooSoundStarted)
    {
	printf("Request Sound Play thread exit...\n");

	soundThreadPause = 1;
	soundThreadExitReq = 1;

	__dcache_writeback_all();

	while (OSTaskDel(TASK_START_PRIO))
	{
	    OSTimeDly(1);
	}

	printf("Sound Play thread stopped\n");
    }

    waveout_close(woHandle);
    woHandle = 0;
}

void dingooSoundPause(unsigned int b)
{
    soundThreadPause = b ? 1 : 0;
	    
    WriteBufferID = 0;
}

unsigned int dingooSoundIsPaused()
{
    return soundThreadPause;
}

void dingooSoundClearBuffers()
{
    memset(dingooSoundBufferTotal, 0, SND_BUFFER_SAMPLES * SND_NBR_BUFFERS * sizeof(short));
    memset(silence, 0, SND_BUFFER_SAMPLES * sizeof(short));
}

void dingooSoundVolumeIncrease()
{
    if (dingooSoundVolume < SND_VOLUME_MAX)
	dingooSoundVolume += SND_VOLUME_STEP;

    if (dingooSoundVolume > SND_VOLUME_MAX)
	dingooSoundVolume = SND_VOLUME_MAX;

    waveout_set_volume(dingooSoundVolume);
}

void dingooSoundVolumeDecrease()
{
    if (dingooSoundVolume > 0)
	dingooSoundVolume -= SND_VOLUME_STEP;

    if (dingooSoundVolume < 0)
	dingooSoundVolume = 0;

    waveout_set_volume(dingooSoundVolume);
}

unsigned int dingooSoundGetVolume()
{
    return dingooSoundVolume;
}

void dingoo_sound_volume(int l, int r)
{
    //extern int master_volume;
    l=l<0?0:l; l=l>100?100:l; r=r<0?0:r; r=r>100?100:r;
    l = ((l+r)*5)/33; // Dingoo volume 0 <=> 30
    //if (l>0) master_volume=l;     /* Dont set if muting */
    if (woHandle)
	dingooSoundSetVolume(l);
    else
	dingooSoundVolume = l;
}


void dingooSoundSetVolume(unsigned int v)
{
    if (v < 0)
	dingooSoundVolume = 0;
    else if (v > SND_VOLUME_MAX)
	dingooSoundVolume = SND_VOLUME_MAX;
    else
	dingooSoundVolume = v;

    waveout_set_volume(dingooSoundVolume);
}

void _dingooSoundPlay(void *none)
{
    //printf("Dingoo Sound Play thread started\n");
    do
    {
	if (soundThreadPause == 0)  // Not Paused
	{
	    curReadBuffer = (curReadBuffer+1)&(SND_NBR_BUFFERS-1);      // Update the current sound buffer
	    waveout_write(woHandle, dingooSoundBuffers[curReadBuffer], nbr_samples_per_frame * SND_BYTE_SZ * nbr_channels );
	}
	else
	{
	    waveout_write(woHandle, (void *)silence, nbr_samples_per_frame * SND_BYTE_SZ);
	}

	ReadBufferID++;

    } while (1);
}
