/*
  Dingoo minimal library by Alekmaul
  Base on Slaanesh's dingux port
  Based on Rlyeh's minimal.c for GP2X
*/
#include <stddef.h>
#include <stdint.h>

#include <dingoo/keyboard.h>
#include <dingoo/slcd.h>
#include <dingoo/cache.h>
#include <dingoo/jz4740.h>

#include <time.h>
#include <sml/control.h>
#include <sml/timer.h>
#include "minimal.h"
#include "dingoo_wrapper.h"
#include "dingoo_sound.c"
#include "dingoo_cpu.c"
#include "osinline.h"

char		gameDir[256];
unsigned short	dingoo_palette[DINGOO_PALETTE_SZ];
display		*gameDisplay  = NULL;
unsigned short	*dingoo_screen15;
int		rotate_controls=0;
uint16_t *g_pGameDecodeBuf;	    /* Not used */
extern char	*_app_path;	    /* from Native SDK entry_point.c */

#if 0
void* _m4alldisplay_addr() {
	return g_pGameDecodeBuf;
}

unsigned short **GetVideoBufferPointer()
{
    unsigned long *LP, LV;
    short *SP;
    LP = (unsigned long *)_lcd_get_frame;
    LV = *LP;
    LV = ((LV & 0x03FFFFFF) << 2) | 0x80000000;
    LP = (unsigned long *)LV;
    LV = LP[2];
    LV = ((LV & 0x03FFFFFF) << 2) | 0x80000000;
    LP = (unsigned long *)LV;
    LV = LP[10];
    LV = ((LV & 0x03FFFFFF) << 2) | 0x80000000;
    LP = (unsigned long *)LV;
    SP = (short *)LV;
    LV = (LP[0] << 16) + ((long)(SP[4]));
    return((unsigned short **)LV);
}
#endif


//
// see jz4740.h for details on timers
// timers 0 and 1 used by O/S
//
#ifdef DINGOO_HIRES_TIMER
#define XTAL	  12000000
#define TIMER_NBR 5   
#define MAX_TICKS   0xffff
static unsigned int master_ticks;
void dingoo_reset_timer(void)
{
    __tcu_select_rtcclk(TIMER_NBR);
    __tcu_select_clk_div1(TIMER_NBR);
    REG_TCU_TDFR(TIMER_NBR) = MAX_TICKS;
    REG_TCU_TDHR(TIMER_NBR) = MAX_TICKS;
    __tcu_set_count(TIMER_NBR, 0);
}


void dingoo_stop_timer(void)
{
    __tcu_stop_counter(TIMER_NBR);
    __tcu_set_pwm_output_shutdown_abrupt(TIMER_NBR);
    __tcu_disable_pwm_output(TIMER_NBR);
    __tcu_mask_half_match_irq(TIMER_NBR);
    __tcu_mask_full_match_irq(TIMER_NBR);
}

void dingoo_start_timer(void)
{
    __tcu_start_counter(TIMER_NBR);
    __tcu_clear_full_match_flag(TIMER_NBR);
}

unsigned int tick_hist[256];
unsigned int current_hist[256];
unsigned int master_hist[256];
void dingoo_update_timer(int hist)
{
    static unsigned x = 0;

    static unsigned int prev_ticks = 0;
    unsigned int current_ticks = __tcu_get_count(TIMER_NBR);
    //unsigned int pclk = __cpm_get_pclk();
    unsigned int ticks;

    // master_ticks is in micro seconds (1,000,000x per second)
    // Find ticks since last update and divide by div prescaler
    // The timer wraps around after MAX_TICKS
    //
    if (prev_ticks < current_ticks)
	ticks = current_ticks - prev_ticks;
    else
	ticks = MAX_TICKS - prev_ticks + current_ticks;

    prev_ticks = current_ticks;

    master_ticks += (ticks * 1000000)/(JZ_EXTAL2); 

    if (hist)
    {
	x++;
	x=x&0xff;
	tick_hist[x] = ticks;
	current_hist[x] = current_ticks;
	master_hist[x] = master_ticks;
	__tcu_clear_full_match_flag(TIMER_NBR);
    }

}


void dingoo_init_timer(void)
{
    debug_printf("__cpm_get_pclk = %d\n", __cpm_get_pclk());
    master_ticks = 0;
    dingoo_stop_timer();
    dingoo_reset_timer();
    dingoo_start_timer();
}

#endif

void dingoo_clear_buffer(void)
{
    memset32(dingoo_screen15, 0, DINGOO_SCREEN_WIDTH * DINGOO_SCREEN_HEIGHT * sizeof(short));
}

void dingoo_clear_video(void)
{
    memset32((uint32_t*) dingoo_screen15, 0, DINGOO_SCREEN_WIDTH * DINGOO_SCREEN_HEIGHT * sizeof(short));
}

void dingoo_flip_video(void)
{
    __dcache_writeback_all();
    _lcd_set_frame();
}

#if 0
void dingoo_buffer_to_video()
{
    uint32_t* out = (uint32_t*) _lcd_get_frame();
    uint32_t* in = (uint32_t*) dingoo_screen15;

    /* Copy to LCD */
    memcpy32(out, in, DINGOO_SCREEN_WIDTH * DINGOO_SCREEN_HEIGHT * sizeof(short));

    /* Flush cache */
    __dcache_writeback_all();

    /* Flip the draw buffer with LCD display  */
    _lcd_set_frame();

    /* Update the new draw buffer */
    dingoo_screen15 = g_pGameDecodeBuf;
}
#endif


unsigned long dingoo_joystick_read(unsigned int use_virtual)
{
    KEY_STATUS key_curr;

    _kbd_get_status( &key_curr);
    unsigned long gpio3 = key_curr.status;

    return gpio3;
}

void dingoo_joystick_press (void)
{
    while(!dingoo_joystick_read(1));
}

void dingoo_timer_delay(unsigned int milliseconds)
{
    unsigned int now=dingoo_timer_read();
    while (dingoo_timer_read()-now<(milliseconds*1000));
}


unsigned int dingoo_timer_read()
{
#ifdef DINGOO_HIRES_TIMER
    if (sound_thread_started)
    {
	dingoo_update_timer(0);
	return master_ticks;
    }
    else
    {
	return OSTimeGet()*10010;
    }
#else
	return OSTimeGet()*10010;   // 99.9hz, too low res really. Want 1000hz min.
#endif
}

void dingoo_timer_profile(void)
{
}

void dingoo_init(unsigned int mhz, unsigned short *buffer)
{
	/* init main graphics and controls */
	//gameDisplay = display_create_fastest(); // 320x240x16 (RGB565) with static buffer

	srand(OSTimeGet());

	/* Set frame buffer */
	if (buffer)
	{
	    dingoo_screen15 = buffer;	/* Buffer passed in */
	}
	else
	{
	    dingoo_screen15 = _lcd_get_frame();	/* straight to LCD screen */
	}

	/* clear lcd buffer */
	dingoo_clear_buffer();

	/* set dingoo cpu speed */
	dingoo_set_clock(mhz);

#ifdef DINGOO_HIRES_TIMER
	/* Start Hi-Resolution timer */
	dingoo_init_timer();
	unsigned int t1, t2, t3;
	mdelay(10);
	dingoo_update_timer(0);
	t1 = master_ticks;
	mdelay(10);
	dingoo_update_timer(0);
	t2 = master_ticks;
	mdelay(10);
	dingoo_update_timer(0);
	t3 = master_ticks;
	debug_printf("Timer accuracy: %u %u %u\n", t1, t2, t3);
#endif

}

void dingoo_deinit(void)
{
#ifdef DINGOO_HIRES_TIMER
	unsigned int x=0;
	dingoo_stop_timer();
	for(x=0; x<256; x++)
	{
	    debug_printf("%u: %u %u %u\n", x, tick_hist[x], current_hist[x], master_hist[x]);
	}
#endif

    debug_printf("dingoo_deinit\n");
    dingoo_set_clock(DINGOO_DEFAULT_MHZ);
    __dcache_writeback_all();

    /* close devices */
    //display_delete(gameDisplay);
}


void dingoo_set_video_mode(int bpp,int width,int height)
{
	dingoo_clear_buffer();

	/*
	 * Not much else to do here as we're are always using
	 * a 320 x 240 x 16bit screen
	 */
}

void dingoo_video_wait_vsync(void)
{
	/* Looks like this is impossible with the Dingoo's LCD */
	/* TV out uses the built in controller so it may be useful for that */
}


static unsigned char fontdata8x8[] =
{
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x3C,0x42,0x99,0xBD,0xBD,0x99,0x42,0x3C,0x3C,0x42,0x81,0x81,0x81,0x81,0x42,0x3C,
	0xFE,0x82,0x8A,0xD2,0xA2,0x82,0xFE,0x00,0xFE,0x82,0x82,0x82,0x82,0x82,0xFE,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0x64,0x74,0x7C,0x38,0x00,0x00,
	0x80,0xC0,0xF0,0xFC,0xF0,0xC0,0x80,0x00,0x01,0x03,0x0F,0x3F,0x0F,0x03,0x01,0x00,
	0x18,0x3C,0x7E,0x18,0x7E,0x3C,0x18,0x00,0xEE,0xEE,0xEE,0xCC,0x00,0xCC,0xCC,0x00,
	0x00,0x00,0x30,0x68,0x78,0x30,0x00,0x00,0x00,0x38,0x64,0x74,0x7C,0x38,0x00,0x00,
	0x3C,0x66,0x7A,0x7A,0x7E,0x7E,0x3C,0x00,0x0E,0x3E,0x3A,0x22,0x26,0x6E,0xE4,0x40,
	0x18,0x3C,0x7E,0x3C,0x3C,0x3C,0x3C,0x00,0x3C,0x3C,0x3C,0x3C,0x7E,0x3C,0x18,0x00,
	0x08,0x7C,0x7E,0x7E,0x7C,0x08,0x00,0x00,0x10,0x3E,0x7E,0x7E,0x3E,0x10,0x00,0x00,
	0x58,0x2A,0xDC,0xC8,0xDC,0x2A,0x58,0x00,0x24,0x66,0xFF,0xFF,0x66,0x24,0x00,0x00,
	0x00,0x10,0x10,0x38,0x38,0x7C,0xFE,0x00,0xFE,0x7C,0x38,0x38,0x10,0x10,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x1C,0x1C,0x18,0x00,0x18,0x18,0x00,
	0x6C,0x6C,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x28,0x7C,0x28,0x7C,0x28,0x00,0x00,
	0x10,0x38,0x60,0x38,0x0C,0x78,0x10,0x00,0x40,0xA4,0x48,0x10,0x24,0x4A,0x04,0x00,
	0x18,0x34,0x18,0x3A,0x6C,0x66,0x3A,0x00,0x18,0x18,0x20,0x00,0x00,0x00,0x00,0x00,
	0x30,0x60,0x60,0x60,0x60,0x60,0x30,0x00,0x0C,0x06,0x06,0x06,0x06,0x06,0x0C,0x00,
	0x10,0x54,0x38,0x7C,0x38,0x54,0x10,0x00,0x00,0x18,0x18,0x7E,0x18,0x18,0x00,0x00,
	0x00,0x00,0x00,0x00,0x18,0x18,0x30,0x00,0x00,0x00,0x00,0x00,0x3E,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x18,0x18,0x00,0x00,0x00,0x04,0x08,0x10,0x20,0x40,0x00,0x00,
	0x38,0x4C,0xC6,0xC6,0xC6,0x64,0x38,0x00,0x18,0x38,0x18,0x18,0x18,0x18,0x7E,0x00,
	0x7C,0xC6,0x0E,0x3C,0x78,0xE0,0xFE,0x00,0x7E,0x0C,0x18,0x3C,0x06,0xC6,0x7C,0x00,
	0x1C,0x3C,0x6C,0xCC,0xFE,0x0C,0x0C,0x00,0xFC,0xC0,0xFC,0x06,0x06,0xC6,0x7C,0x00,
	0x3C,0x60,0xC0,0xFC,0xC6,0xC6,0x7C,0x00,0xFE,0xC6,0x0C,0x18,0x30,0x30,0x30,0x00,
	0x78,0xC4,0xE4,0x78,0x86,0x86,0x7C,0x00,0x7C,0xC6,0xC6,0x7E,0x06,0x0C,0x78,0x00,
	0x00,0x00,0x18,0x00,0x00,0x18,0x00,0x00,0x00,0x00,0x18,0x00,0x00,0x18,0x18,0x30,
	0x1C,0x38,0x70,0xE0,0x70,0x38,0x1C,0x00,0x00,0x7C,0x00,0x00,0x7C,0x00,0x00,0x00,
	0x70,0x38,0x1C,0x0E,0x1C,0x38,0x70,0x00,0x7C,0xC6,0xC6,0x1C,0x18,0x00,0x18,0x00,
	0x3C,0x42,0x99,0xA1,0xA5,0x99,0x42,0x3C,0x38,0x6C,0xC6,0xC6,0xFE,0xC6,0xC6,0x00,
	0xFC,0xC6,0xC6,0xFC,0xC6,0xC6,0xFC,0x00,0x3C,0x66,0xC0,0xC0,0xC0,0x66,0x3C,0x00,
	0xF8,0xCC,0xC6,0xC6,0xC6,0xCC,0xF8,0x00,0xFE,0xC0,0xC0,0xFC,0xC0,0xC0,0xFE,0x00,
	0xFE,0xC0,0xC0,0xFC,0xC0,0xC0,0xC0,0x00,0x3E,0x60,0xC0,0xCE,0xC6,0x66,0x3E,0x00,
	0xC6,0xC6,0xC6,0xFE,0xC6,0xC6,0xC6,0x00,0x7E,0x18,0x18,0x18,0x18,0x18,0x7E,0x00,
	0x06,0x06,0x06,0x06,0xC6,0xC6,0x7C,0x00,0xC6,0xCC,0xD8,0xF0,0xF8,0xDC,0xCE,0x00,
	0x60,0x60,0x60,0x60,0x60,0x60,0x7E,0x00,0xC6,0xEE,0xFE,0xFE,0xD6,0xC6,0xC6,0x00,
	0xC6,0xE6,0xF6,0xFE,0xDE,0xCE,0xC6,0x00,0x7C,0xC6,0xC6,0xC6,0xC6,0xC6,0x7C,0x00,
	0xFC,0xC6,0xC6,0xC6,0xFC,0xC0,0xC0,0x00,0x7C,0xC6,0xC6,0xC6,0xDE,0xCC,0x7A,0x00,
	0xFC,0xC6,0xC6,0xCE,0xF8,0xDC,0xCE,0x00,0x78,0xCC,0xC0,0x7C,0x06,0xC6,0x7C,0x00,
	0x7E,0x18,0x18,0x18,0x18,0x18,0x18,0x00,0xC6,0xC6,0xC6,0xC6,0xC6,0xC6,0x7C,0x00,
	0xC6,0xC6,0xC6,0xEE,0x7C,0x38,0x10,0x00,0xC6,0xC6,0xD6,0xFE,0xFE,0xEE,0xC6,0x00,
	0xC6,0xEE,0x3C,0x38,0x7C,0xEE,0xC6,0x00,0x66,0x66,0x66,0x3C,0x18,0x18,0x18,0x00,
	0xFE,0x0E,0x1C,0x38,0x70,0xE0,0xFE,0x00,0x3C,0x30,0x30,0x30,0x30,0x30,0x3C,0x00,
	0x60,0x60,0x30,0x18,0x0C,0x06,0x06,0x00,0x3C,0x0C,0x0C,0x0C,0x0C,0x0C,0x3C,0x00,
	0x18,0x3C,0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,
	0x30,0x30,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0x06,0x3E,0x66,0x66,0x3C,0x00,
	0x60,0x7C,0x66,0x66,0x66,0x66,0x7C,0x00,0x00,0x3C,0x66,0x60,0x60,0x66,0x3C,0x00,
	0x06,0x3E,0x66,0x66,0x66,0x66,0x3E,0x00,0x00,0x3C,0x66,0x66,0x7E,0x60,0x3C,0x00,
	0x1C,0x30,0x78,0x30,0x30,0x30,0x30,0x00,0x00,0x3E,0x66,0x66,0x66,0x3E,0x06,0x3C,
	0x60,0x7C,0x76,0x66,0x66,0x66,0x66,0x00,0x18,0x00,0x38,0x18,0x18,0x18,0x18,0x00,
	0x0C,0x00,0x1C,0x0C,0x0C,0x0C,0x0C,0x38,0x60,0x60,0x66,0x6C,0x78,0x6C,0x66,0x00,
	0x38,0x18,0x18,0x18,0x18,0x18,0x18,0x00,0x00,0xEC,0xFE,0xFE,0xFE,0xD6,0xC6,0x00,
	0x00,0x7C,0x76,0x66,0x66,0x66,0x66,0x00,0x00,0x3C,0x66,0x66,0x66,0x66,0x3C,0x00,
	0x00,0x7C,0x66,0x66,0x66,0x7C,0x60,0x60,0x00,0x3E,0x66,0x66,0x66,0x3E,0x06,0x06,
	0x00,0x7E,0x70,0x60,0x60,0x60,0x60,0x00,0x00,0x3C,0x60,0x3C,0x06,0x66,0x3C,0x00,
	0x30,0x78,0x30,0x30,0x30,0x30,0x1C,0x00,0x00,0x66,0x66,0x66,0x66,0x6E,0x3E,0x00,
	0x00,0x66,0x66,0x66,0x66,0x3C,0x18,0x00,0x00,0xC6,0xD6,0xFE,0xFE,0x7C,0x6C,0x00,
	0x00,0x66,0x3C,0x18,0x3C,0x66,0x66,0x00,0x00,0x66,0x66,0x66,0x66,0x3E,0x06,0x3C,
	0x00,0x7E,0x0C,0x18,0x30,0x60,0x7E,0x00,0x0E,0x18,0x0C,0x38,0x0C,0x18,0x0E,0x00,
	0x18,0x18,0x18,0x00,0x18,0x18,0x18,0x00,0x70,0x18,0x30,0x1C,0x30,0x18,0x70,0x00,
	0x00,0x00,0x76,0xDC,0x00,0x00,0x00,0x00,0x10,0x28,0x10,0x54,0xAA,0x44,0x00,0x00,
};

static void dingoo_text(unsigned short *screen, int x, int y, char *text, int color)
{
	unsigned int i,l,len;
	unsigned short *fb;

	if (y>239-8) y = 231;	// Last available position for text

	len=strlen(text);	// Maximum 40 characters
	if (len > 40) len = 40;

        fb = screen + (y*DINGOO_SCREEN_WIDTH+x);

	for (i=0;i<len;i++) {
		
		for (l=0;l<8;l++) {
			fb[l*DINGOO_SCREEN_WIDTH+0]=(fontdata8x8[((text[i])*8)+l]&0x80)?color:fb[l*DINGOO_SCREEN_WIDTH+0];
			fb[l*DINGOO_SCREEN_WIDTH+1]=(fontdata8x8[((text[i])*8)+l]&0x40)?color:fb[l*DINGOO_SCREEN_WIDTH+1];
			fb[l*DINGOO_SCREEN_WIDTH+2]=(fontdata8x8[((text[i])*8)+l]&0x20)?color:fb[l*DINGOO_SCREEN_WIDTH+2];
			fb[l*DINGOO_SCREEN_WIDTH+3]=(fontdata8x8[((text[i])*8)+l]&0x10)?color:fb[l*DINGOO_SCREEN_WIDTH+3];
			fb[l*DINGOO_SCREEN_WIDTH+4]=(fontdata8x8[((text[i])*8)+l]&0x08)?color:fb[l*DINGOO_SCREEN_WIDTH+4];
			fb[l*DINGOO_SCREEN_WIDTH+5]=(fontdata8x8[((text[i])*8)+l]&0x04)?color:fb[l*DINGOO_SCREEN_WIDTH+5];
			fb[l*DINGOO_SCREEN_WIDTH+6]=(fontdata8x8[((text[i])*8)+l]&0x02)?color:fb[l*DINGOO_SCREEN_WIDTH+6];
			fb[l*DINGOO_SCREEN_WIDTH+7]=(fontdata8x8[((text[i])*8)+l]&0x01)?color:fb[l*DINGOO_SCREEN_WIDTH+7];
		}
		fb+=8;
	} 
}

void dingoo_gamelist_text_out(int x, int y, char *eltexto)
{
	char texto[33];
	strncpy(texto,eltexto,32);
	texto[32]=0;
	if (texto[0]!='-')
		dingoo_text(dingoo_screen15,x+1,y+1,texto,DINGOO_BLACK);
	dingoo_text(dingoo_screen15,x,y,texto,DINGOO_WHITE);
}

/* Variadic functions guide found at http://www.unixpapa.com/incnote/variadic.html */
void dingoo_gamelist_text_out_fmt(int x, int y, char* fmt, ...)
{
	char strOut[128];
	va_list marker;
	
	va_start(marker, fmt);
	vsprintf(strOut, fmt, marker);
	va_end(marker);	

	dingoo_gamelist_text_out(x, y, strOut);
}


void debug_printf(char *fmt, ...)
{
    va_list ap;
    char tmp_buf[256];
    int mesg_len=0;
    FILE *fptr=NULL;

    va_start(ap, fmt);
    mesg_len = vsnprintf(tmp_buf, sizeof(tmp_buf)-1, fmt, ap);
    va_end(ap);
    fptr = fsys_fopen(DINGOO_NATIVE_LOGFILE, "a");
    if (fptr == NULL)
	fptr = fsys_fopen(DINGOO_NATIVE_LOGFILE, "w");
    if (fptr != NULL)
    {
        fsys_fwrite(tmp_buf, mesg_len, 1, fptr);
        fsys_fclose(fptr);
    }
}
