/*
  Dingoo minimal library by Slaanesh
*/

#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <sys/time.h>
#include <unistd.h>
#include <stdarg.h>
#include <sml/graphics.h>

#ifndef __MINIMAL_H__
#define __MINIMAL_H__

/* System defines */
//ALEK #define KEYBOARD 1
//#define DINGOO_HIRES_TIMER	1
#define DINGOO_DELAY 100
#define DINGOO_NATIVE_LOGFILE   "b:\\mame.txt"
#define DINGOO_DEFAULT_MHZ	336
#define DINGOO_DEFAULT_BRIGHT	70
#define DINGOO_SCREEN_WIDTH	320
#define DINGOO_SCREEN_HEIGHT	240
#define DINGOO_BLACK		0x0000	/* Black 16-bit pixel */
#define DINGOO_WHITE		0xFFFF	/* White 16-bit pixel */
#define DINGOO_PALETTE_SZ	256     /* To emulate 8-bit paletted games */
/* These are for Dingoo controller inputs */
#define MASK_GPIO2 	0x00020000 /*00000000000000100000000000000000*/
#define MASK_GPIO3	0x280EC067 /*00101000000011101100000001100111*/
/* This mask is used to select buttons which when combined with the SELECT
 * key create a "new" key */
#define MASK_SEL_SHIFT  0x0009C007 /*00000000000010011100000000000111*/


/* Handy Macros */
#define dingoo_video_color8(C,R,G,B) (dingoo_palette[C] = ((((R)&0xF8)<<8)|(((G)&0xFC)<<3)|(((B)&0xF8)>>3)))
#define dingoo_video_color15(R,G,B) ((((R)&0xF8)<<8)|(((G)&0xFC)<<3)|(((B)&0xF8)>>3))
#define dingoo_video_getr15(C) (((C)>>8)&0xF8)
#define dingoo_video_getg15(C) (((C)>>3)&0xFC)
#define dingoo_video_getb15(C) (((C)<<3)&0xF8)

enum  {
  DINGOO_UP =	1<<20,
	DINGOO_DOWN =	1<<27,
	DINGOO_LEFT =	1<<28,
	DINGOO_RIGHT =	1<<18,
	DINGOO_B =	1<<21, //1<<31,	/* remapped A to match GP2X */
	DINGOO_X =	1<<16,	/* remapped B to match GP2X */
	DINGOO_Y =	1<<6,	/* remapped X to match GP2X */
  DINGOO_A =	1<<31,	/* remapped Y to match GP2X */
	DINGOO_L = 	1<<8,
	DINGOO_R =	1<<29,
	DINGOO_START =	1<<11,	/* actually pin 17, merged gpio2 and gpio3 */
	DINGOO_SELECT =	1<<10,
	DINGOO_POWER =  1<<7,
				/* Virtual keys
				 * SELECT + 'button'
				 * Same as base values except shifted << 7 */
	DINGOO_SEL_B =	1<<28,//(DINGOO_B)<<7,	
	DINGOO_SEL_X =	1<<23,//(DINGOO_X)<<7,
	DINGOO_SEL_Y =	1<<13,//(DINGOO_Y)<<7,
  DINGOO_SEL_A =	1<<7,//(DINGOO_A)<<7,
	DINGOO_SEL_L = 	1<<15,//(DINGOO_L)<<7,
	DINGOO_SEL_R =	1<<5,//(DINGOO_R)<<7,
	DINGOO_SEL_START =1<<18//(DINGOO_START)<<7
};

void dingoo_reset_timer(void);
void dingoo_stop_timer(void);
void dingoo_start_timer(void);
void dingoo_update_timer(int hist);
void dingoo_init_timer(void);

extern unsigned short	dingoo_palette[256];
extern unsigned short 	*dingoo_screen15;
extern char		gameDir[256];
extern display*		gameDisplay;

extern void dingoo_init(unsigned int, unsigned short*);
extern void dingoo_deinit(void);
extern void dingoo_timer_delay(unsigned int milliseconds);
extern void dingoo_video_flip(void);
extern void dingoo_video_flip_single(void);
extern void dingoo_video_setpalette(void);
extern unsigned long dingoo_joystick_read(unsigned int use_virtual);
extern void dingoo_joystick_press (void);
extern unsigned int dingoo_timer_read(void);
extern void dingoo_timer_profile(void);
extern void dingoo_set_video_mode(int bpp,int width,int height);
extern void dingoo_set_clock(unsigned int mhz);
extern void dingoo_video_wait_vsync(void);
extern void dingoo_clear_video(void);
extern void dingoo_flush_video(void);

extern void debug_printf(char* fmt, ...);
extern void debug_printf_init(void);
extern void dingoo_gamelist_text_out(int x, int y, char *eltexto);
extern void dingoo_gamelist_text_out_fmt(int x, int y, char* fmt, ...);



#endif
