#include "../vidhrdw/cabal.c"

/******************************************************************
Cabal Bootleg
(c)1998 Red Corp

driver by Carlos A. Lozano Baides

68000 + Z80

The original uses 2xYM3931 for sound
The bootleg uses YM2151 + 2xZ80 used as ADPCM players

MEMORY MAP
0x000000 - 0x03ffff   ROM
0x040000 - 0x0437ff   RAM
0x043800 - 0x0437ff   VRAM (Sprites)
0x060000 - 0x0607ff   VRAM (Tiles)
0x080000 - 0x0803ff   VRAM (Background)
0x0A0000 - 0xA0000f   Input Ports
0x0C0040 - 0x0c0040   Watchdog??
0x0C0080 - 0x0C0080   Watchdog??
0x0E0000 - 0x0E07ff   COLORRAM (----BBBBGGGGRRRR)
0x0E8000 - 0x0E800f   Output Ports/Input Ports

VRAM(Background)
0x80000 - 32 bytes (16 tiles)
0x80040 - 32 bytes (16 tiles)
0x80080 - 32 bytes (16 tiles)
0x800c0 - 32 bytes (16 tiles)
0x80100 - 32 bytes (16 tiles)
...
0x803c0 - 32 bytes (16 tiles)

VRAM(Tiles)
0x60000-0x607ff (1024 tiles 8x8 tiles, 2 bytes every tile)

VRAM(Sprites)
0x43800-0x43bff (128 sprites, 8 bytes every sprite)

COLORRAM(Colors)
0xe0000-0xe07ff (1024 colors, ----BBBBGGGGRRRR)

******************************************************************/

#include "driver.h"
#include "vidhrdw/generic.h"
#include "cpu/z80/z80.h"

enum
{
    VECTOR_INIT,
    RST10_ASSERT,
    RST10_CLEAR,
    RST18_ASSERT,
    RST18_CLEAR
};


static int cabal_sound_command1, cabal_sound_command2;
static int sound_cpu=1;
static UINT8 main2sub[2],sub2main[2];
static int last[4];
static int main2sub_pending,sub2main_pending;

extern void cabal_vh_screenrefresh( struct osd_bitmap *bitmap, int fullrefresh );

static void cabal_init_machine( void ) {
	cabal_sound_command1 = cabal_sound_command2 = 0xff;
}

READ_HANDLER( seibu_main_word_r );
WRITE_HANDLER( seibu_main_word_w );

static READ_HANDLER( cabal_background_r ){
	return READ_WORD (&videoram[offset]);
}

static WRITE_HANDLER( cabal_background_w ){
	int oldword = READ_WORD(&videoram[offset]);
	int newword = COMBINE_WORD(oldword,data);
	if( oldword != newword ){
		WRITE_WORD(&videoram[offset],newword);
		dirtybuffer[offset/2] = 1;
	}
}

/******************************************************************************************/

static void setvector_callback(int param)
{
    static int irq1,irq2;

    switch(param)
    {
	case VECTOR_INIT:
	    irq1 = irq2 = 0xff;
	    break;

	case RST10_ASSERT:
	    irq1 = 0xd7;
	    break;

	case RST10_CLEAR:
	    irq1 = 0xff;
	    break;

	case RST18_ASSERT:
	    irq2 = 0xdf;
	    break;

	case RST18_CLEAR:
	    irq2 = 0xff;
	    break;
    }

    cpu_irq_line_vector_w(sound_cpu,0,irq1 & irq2);
    if ((irq1 & irq2) == 0xff)      /* no IRQs pending */
	cpu_set_irq_line(sound_cpu,0,CLEAR_LINE);
    else    /* IRQ pending */
	cpu_set_irq_line(sound_cpu,0,ASSERT_LINE);
}



void seibu_ym3812_irqhandler(int linestate)
{
    if (linestate)
	timer_set(TIME_NOW,RST10_ASSERT,setvector_callback);
    else
	timer_set(TIME_NOW,RST10_CLEAR,setvector_callback);
}


static struct YM2151interface ym2151_interface =
{
	1,                      /* 1 chip */
	3579580,        /* 3.58 MHz ? */ /* 4 MHz in raine */
	{ YM3012_VOL(80,MIXER_PAN_LEFT,80,MIXER_PAN_RIGHT) },
	{ seibu_ym3812_irqhandler },
};

static struct YM2151interface ym2151_interface_bl =
{
 	1,			/* 1 chip */
 	3579545,	/* 3.58 MHZ ? */ /* 4 MHZ in raine */
	{ YM3012_VOL(80,MIXER_PAN_LEFT,80,MIXER_PAN_RIGHT) },
 	{ 0 }
};

struct ADPCMinterface adpcm_interface =
{
	2,			/* total number of ADPCM decoders in the machine */
	8000,		/* playback frequency */
	REGION_SOUND1, /* memory region where the samples come from */
	0,			/* initialization function */
	{40,40}
};

static void cabal_play_adpcm( int channel, int which ){
	if( which!=0xff ){
		unsigned char *RAM = memory_region(REGION_SOUND1);
		int offset = channel*0x10000;
		int start, len;

		which = which&0x7f;
		if( which ){
			which = which*2+0x100;
			start = RAM[offset+which] + 256*RAM[offset+which+1];
			len = (RAM[offset+start]*256 + RAM[offset+start+1])*2;
			start+=2;
			ADPCM_play( channel,offset+start,len );
		}
	}
}

static READ_HANDLER( cabal_coin_r ) {
	static int coin = 0;
	int val = readinputport( 3 );

	if ( !( val & 0x04 ) ){
		if ( coin == 0 ){
			coin = 1;
			return val;
		}
	} else {
		coin = 0;
	}

 	return val | 0x04;
}

static READ_HANDLER( cabal_io_r ) {
	switch (offset){
		case 0x0: return readinputport(4) + (readinputport(5)<<8); /* DIPSW */
		case 0x8: return 0xff + (readinputport(0)<<8);/* IN0 */
		case 0x10: return readinputport(1) + (readinputport(2)<<8); /* IN1,IN2 */
		default: return (0xffff);
	}
}



static WRITE_HANDLER( track_reset_w )
{
    int i;

    for (i = 0;i < 4;i++)
    last[i] = readinputport(3+i);
}

static READ_HANDLER( track_r )
{
    switch (offset>>1)
    {
	default:
	case 0: return ((readinputport(3) - last[0]) & 0x00ff) | ((readinputport(5) - last[2]) << 8);           /* X lo */
	case 1: return ((readinputport(3) - last[0]) >> 8)     | ((readinputport(5) - last[2]) & 0xff00);       /* X hi */
	case 2: return ((readinputport(4) - last[1]) & 0x00ff) | ((readinputport(6) - last[3]) << 8);           /* Y lo */
	case 3: return ((readinputport(4) - last[1]) >> 8)     | ((readinputport(6) - last[3]) & 0xff00);       /* Y hi */
    }
}


static WRITE_HANDLER( cabal_sndcmd_w ) {
	switch (offset) {
		case 0x0:
			cabal_sound_command1 = data;
			cpu_cause_interrupt( 1, Z80_NMI_INT );
		break;

		case 0x2: /* ?? */
			cabal_sound_command2 = data & 0xff;
		break;

		case 0x8: /* ?? */
		break;
	}
}

WRITE_HANDLER( cabal_sound_irq_trigger_word_w )
{
	seibu_main_word_w(8,data);

	/* spin for a while to let the Z80 read the command, otherwise coins "stick" */
	cpu_spinuntil_time(TIME_IN_USEC(50));
}


static struct MemoryReadAddress readmem_cpu[] = {
	{ 0x00000, 0x3ffff, MRA_ROM },
	{ 0x40000, 0x437ff, MRA_RAM },
	{ 0x43800, 0x43fff, MRA_RAM },
	{ 0x44000, 0x4ffff, MRA_RAM },
	{ 0x60000, 0x607ff, MRA_BANK1 },  /* text layer */
	{ 0x80000, 0x801ff, cabal_background_r }, /* background layer */
	{ 0x80200, 0x803ff, MRA_BANK2 },
	{ 0xa0000, 0xa0001, input_port_0_r },
	{ 0xa0008, 0xa000f, track_r },
	{ 0xa0010, 0xa0011, input_port_1_r },
	{ 0xe0000, 0xe07ff, paletteram_word_r },
	{ 0xe8000, 0xe800d, seibu_main_word_r },
	{ -1 }
};
static struct MemoryWriteAddress writemem_cpu[] = {
	{ 0x00000, 0x3ffff, MWA_ROM },
	{ 0x40000, 0x437ff, MWA_RAM },
	{ 0x43800, 0x43fff, MWA_RAM, &spriteram, &spriteram_size },
	{ 0x44000, 0x4ffff, MWA_RAM },
	{ 0x60000, 0x607ff, MWA_BANK1, &colorram },
	{ 0x80000, 0x801ff, cabal_background_w, &videoram, &videoram_size },
	{ 0x80200, 0x803ff, MWA_BANK2 },
	{ 0xc0000, 0xc0001, track_reset_w },
	{ 0xc0040, 0xc0041, MWA_NOP }, /* ??? */
	{ 0xc0080, 0xc0081, MWA_NOP }, /* cabal_flipscreen_w */
	{ 0xe0000, 0xe07ff, paletteram_xxxxBBBBGGGGRRRR_word_w, &paletteram},
	{ 0xe8008, 0xe8009, cabal_sound_irq_trigger_word_w },   // fix coin insertion
	{ 0xe8000, 0xe800d, seibu_main_word_w },
	{ -1 }
};

static struct MemoryReadAddress cabalbl_readmem_cpu[] = {
	{ 0x00000, 0x3ffff, MRA_ROM },
	{ 0x40000, 0x437ff, MRA_RAM },
	{ 0x43c00, 0x4ffff, MRA_RAM },
	{ 0x43800, 0x43bff, MRA_RAM },
	{ 0x60000, 0x607ff, MRA_BANK1 },  /* text layer */
	{ 0x80000, 0x801ff, cabal_background_r }, /* background layer */
	{ 0x80200, 0x803ff, MRA_BANK2 },
	{ 0xa0000, 0xa0012, cabal_io_r },
	{ 0xe0000, 0xe07ff, paletteram_word_r },
	{ 0xe8000, 0xe800f, cabal_coin_r },
	{ -1 }
};
static struct MemoryWriteAddress cabalbl_writemem_cpu[] = {
	{ 0x00000, 0x3ffff, MWA_ROM },
	{ 0x40000, 0x437ff, MWA_RAM },
	{ 0x43800, 0x43bff, MWA_RAM, &spriteram, &spriteram_size },
	{ 0x43c00, 0x4ffff, MWA_RAM },
	{ 0x60000, 0x607ff, MWA_BANK1, &colorram },
	{ 0x80000, 0x801ff, cabal_background_w, &videoram, &videoram_size },
	{ 0x80200, 0x803ff, MWA_BANK2 },
	{ 0xc0040, 0xc0041, MWA_NOP }, /* ??? */
	{ 0xc0080, 0xc0081, MWA_NOP }, /* ??? */
	{ 0xe0000, 0xe07ff, paletteram_xxxxBBBBGGGGRRRR_word_w, &paletteram},
	{ 0xe8000, 0xe800f, cabal_sndcmd_w },
	{ -1 }
};

/*********************************************************************/

static READ_HANDLER( cabal_snd_r )
{
	switch(offset){
		case 0x08: return cabal_sound_command2;
		case 0x0a: return cabal_sound_command1;
		default: return(0xff);
	}
}

static WRITE_HANDLER( cabal_snd_w )
{
	switch( offset ){
		case 0x00: cabal_play_adpcm( 0, data ); break;
		case 0x02: cabal_play_adpcm( 1, data ); break;
     }
}


/* Use this if the sound cpu is cpu 1 */
void seibu_sound_init_1(void)
{
	sound_cpu=1;
	setvector_callback(VECTOR_INIT);
}

READ_HANDLER( seibu_soundlatch_r )
{
	return main2sub[offset];
}

WRITE_HANDLER( seibu_main_data_w )
{
	sub2main[offset] = data;
}

WRITE_HANDLER( seibu_coin_w )
{
	coin_counter_w(0,data & 1);
	coin_counter_w(1,data & 2);
}

READ_HANDLER( seibu_main_data_pending_r )
{
	return sub2main_pending ? 1 : 0;
}

WRITE_HANDLER( seibu_irq_clear_w )
{
	setvector_callback(VECTOR_INIT);
}

WRITE_HANDLER( seibu_rst10_ack_w )
{
    /* Unused for now */
}

WRITE_HANDLER( seibu_rst18_ack_w )
{
    timer_set(TIME_NOW,RST18_CLEAR,setvector_callback);
}


READ_HANDLER( seibu_main_word_r )
{
    switch (offset)
    {
    case 4:
    case 6:
	return sub2main[(offset/2)-2];
    case 10:
	return main2sub_pending ? 1 : 0;
    default:
	return 0xffff;
    }
}

WRITE_HANDLER( seibu_main_word_w )
{
    if (ACCESSING_LSB)
    {
	switch (offset)
	{
	    case 0:
	    case 2:
		main2sub[offset/2] = data;
		break;
	    case 8:
		timer_set(TIME_NOW,RST18_ASSERT,setvector_callback);
		break;
	    case 12:
		/* just a guess */
		sub2main_pending = 0;
		main2sub_pending = 1;
		break;
	}
    }
}

static int start, end, start1, end1;

WRITE_HANDLER( seibu_adpcm_adr_1_w )
{
    if (offset)
    {
	end = data<<8;
    }
    else
    {
	start = data<<8;
    }
}

WRITE_HANDLER( seibu_adpcm_ctl_1_w )
{
    switch (data)
    {
	case 0:
	    ADPCM_stop(0);
	    break;
	case 2:
	    break;
	case 1:
	    ADPCM_play(0, start, end-start);
	    break;
    }
}

WRITE_HANDLER( seibu_adpcm_adr_2_w )
{
    if (offset)
    {
	end1 = data<<8;
    }
    else
    {
	start1 = data<<8;
    }
}

WRITE_HANDLER( seibu_adpcm_ctl_2_w )
{
    switch (data)
    {
	case 0:
	    ADPCM_stop(1);
	    break;
	case 2:
	    break;
	case 1:
	    start1 += 0x10000;
	    end1 += 0x10000;
	    ADPCM_play(1, start1, end1-start1);
	    break;
    }
}

static struct MemoryReadAddress readmem_sound[] =
{
	{ 0x0000, 0x1fff, MRA_ROM },
	{ 0x2000, 0x27ff, MRA_RAM },
	{ 0x4009, 0x4009, YM2151_status_port_0_r },
	{ 0x4010, 0x4011, seibu_soundlatch_r },
	{ 0x4012, 0x4012, seibu_main_data_pending_r },
	{ 0x4013, 0x4013, input_port_2_r },
	{ 0x8000, 0xffff, MRA_ROM },
	{ -1 }
};

static struct MemoryWriteAddress writemem_sound[] =
{
	{ 0x0000, 0x1fff, MWA_ROM },
	{ 0x2000, 0x27ff, MWA_RAM },
	{ 0x4001, 0x4001, seibu_irq_clear_w },
	{ 0x4002, 0x4002, seibu_rst10_ack_w },
	{ 0x4003, 0x4003, seibu_rst18_ack_w },
	{ 0x4005, 0x4006, seibu_adpcm_adr_1_w},
	{ 0x401a, 0x401a, seibu_adpcm_ctl_1_w},
	{ 0x4008, 0x4008, YM2151_register_port_0_w },
	{ 0x4009, 0x4009, YM2151_data_port_0_w },
	{ 0x4018, 0x4019, seibu_main_data_w },
	{ 0x401b, 0x401b, seibu_coin_w },
	{ 0x6005, 0x6006, seibu_adpcm_adr_2_w},
	{ 0x601a, 0x601a, seibu_adpcm_ctl_2_w},
	{ 0x8000, 0xffff, MWA_ROM },
	{ -1 }
};

static struct MemoryReadAddress cabalbl_readmem_sound[] =
{
	{ 0x0000, 0x1fff, MRA_ROM },
	{ 0x2000, 0x2fff, MRA_RAM },
	{ 0x4000, 0x400d, cabal_snd_r },
	{ 0x400f, 0x400f, YM2151_status_port_0_r },
	{ 0x8000, 0xffff, MRA_ROM },
	{ -1 }
};

static struct MemoryWriteAddress cabalbl_writemem_sound[] =
{
	{ 0x0000, 0x1fff, MWA_ROM },
	{ 0x2000, 0x2fff, MWA_RAM },
	{ 0x4000, 0x400d, cabal_snd_w },
	{ 0x400e, 0x400e, YM2151_register_port_0_w },
	{ 0x400f, 0x400f, YM2151_data_port_0_w },
	{ 0x6000, 0x6000, MWA_NOP },  /*???*/
	{ 0x8000, 0xffff, MWA_ROM },
	{ -1 }
};



/***************************************************************************/

INPUT_PORTS_START( cabal )
            PORT_START
	    /* Coin Mode 1, todo Mode 2 */
	    PORT_DIPNAME( 0x000f, 0x000f, DEF_STR( Coinage ) )
            PORT_DIPSETTING(      0x000a, DEF_STR( 6C_1C ) )
            PORT_DIPSETTING(      0x000b, DEF_STR( 5C_1C ) )
            PORT_DIPSETTING(      0x000c, DEF_STR( 4C_1C ) )
            PORT_DIPSETTING(      0x000d, DEF_STR( 3C_1C ) )
            PORT_DIPSETTING(      0x0001, DEF_STR( 8C_3C ) )
            PORT_DIPSETTING(      0x000e, DEF_STR( 2C_1C ) )
            PORT_DIPSETTING(      0x0002, DEF_STR( 5C_3C ) )
            PORT_DIPSETTING(      0x0003, DEF_STR( 3C_2C ) )
            PORT_DIPSETTING(      0x000f, DEF_STR( 1C_1C ) )
            PORT_DIPSETTING(      0x0004, DEF_STR( 2C_3C ) )
            PORT_DIPSETTING(      0x0009, DEF_STR( 1C_2C ) )
            PORT_DIPSETTING(      0x0008, DEF_STR( 1C_3C ) )
            PORT_DIPSETTING(      0x0007, DEF_STR( 1C_4C ) )
            PORT_DIPSETTING(      0x0006, DEF_STR( 1C_5C ) )
            PORT_DIPSETTING(      0x0005, DEF_STR( 1C_6C ) )
            PORT_DIPSETTING(      0x0000, DEF_STR( Free_Play ) )

            PORT_DIPNAME( 0x0010, 0x0010, "Coin Mode" )
	    PORT_DIPSETTING(      0x0010, "Mode 1" )
	    PORT_DIPSETTING(      0x0000, "Mode 2" )
	    PORT_DIPNAME( 0x0020, 0x0020, "Invert Buttons" )
	    PORT_DIPSETTING(      0x0020, DEF_STR( Off ) )
            PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
            PORT_DIPNAME( 0x0040, 0x0040, DEF_STR( Flip_Screen ) )
            PORT_DIPSETTING(      0x0040, DEF_STR( Off ) )
            PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
            PORT_DIPNAME( 0x0080, 0x0080, "Trackball" )
	    PORT_DIPSETTING(      0x0080, "Small" )
	    PORT_DIPSETTING(      0x0000, "Large" )
	    PORT_DIPNAME( 0x0300, 0x0300, DEF_STR( Lives ) )
            PORT_DIPSETTING(      0x0200, "2" )
	    PORT_DIPSETTING(      0x0300, "3" )
	    PORT_DIPSETTING(      0x0100, "5" )
	    PORT_BITX( 0,         0x0000, IPT_DIPSWITCH_SETTING | IPF_CHEAT, "121", IP_KEY_NONE, IP_JOY_NONE )
	    PORT_DIPNAME( 0x0c00, 0x0c00, DEF_STR( Bonus_Life ) )
            PORT_DIPSETTING(      0x0c00, "20k 50k" )
	    PORT_DIPSETTING(      0x0800, "30k 100k" )
	    PORT_DIPSETTING(      0x0400, "50k 150k" )
	    PORT_DIPSETTING(      0x0000, "70K" )
	    PORT_DIPNAME( 0x3000, 0x3000, DEF_STR( Difficulty ) )
            PORT_DIPSETTING(      0x3000, "Easy" )
	    PORT_DIPSETTING(      0x2000, "Normal" )
	    PORT_DIPSETTING(      0x1000, "Hard" )
	    PORT_DIPSETTING(      0x0000, "Very Hard" )
	    PORT_DIPNAME( 0x4000, 0x4000, DEF_STR( Unknown ) )
	    PORT_DIPSETTING(      0x4000, DEF_STR( Off ) )
            PORT_DIPSETTING(      0x0000, DEF_STR( On ) )
            PORT_DIPNAME( 0x8000, 0x8000, DEF_STR( Demo_Sounds ) )
            PORT_DIPSETTING(      0x0000, DEF_STR( Off ) )
            PORT_DIPSETTING(      0x8000, DEF_STR( On ) )

            PORT_START
	    PORT_BIT( 0x0001, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER1 )
            PORT_BIT( 0x0002, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_PLAYER1 )
            PORT_BIT( 0x0004, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER2 )
            PORT_BIT( 0x0008, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_PLAYER2 )
            PORT_BIT( 0x0ff0, IP_ACTIVE_LOW, IPT_UNUSED )
            PORT_BIT( 0x1000, IP_ACTIVE_LOW, IPT_SERVICE1 )
            PORT_BIT( 0x2000, IP_ACTIVE_LOW, IPT_UNKNOWN )
            PORT_BIT( 0x4000, IP_ACTIVE_LOW, IPT_START2 )
            PORT_BIT( 0x8000, IP_ACTIVE_LOW, IPT_START1 )

            PORT_START
	    PORT_BIT_IMPULSE( 0x01, IP_ACTIVE_HIGH, IPT_COIN1, 4 )  /* read through sound cpu */
            PORT_BIT_IMPULSE( 0x02, IP_ACTIVE_HIGH, IPT_COIN2, 4 )  /* read through sound cpu */

            PORT_START
	    PORT_ANALOG( 0x0fff, 0x0000, IPT_TRACKBALL_X | IPF_PLAYER1, 100, 30, 0, 0 )

            PORT_START
	    PORT_ANALOG( 0x0fff, 0x0000, IPT_TRACKBALL_Y | IPF_REVERSE | IPF_PLAYER1, 100, 30, 0, 0 )

            PORT_START
	    PORT_ANALOG( 0x0fff, 0x0000, IPT_TRACKBALL_X | IPF_PLAYER2, 100, 30, 0, 0 )

            PORT_START
	    PORT_ANALOG( 0x0fff, 0x0000, IPT_TRACKBALL_Y | IPF_PLAYER2, 100, 30, 0, 0 )
INPUT_PORTS_END



INPUT_PORTS_START( cabalbl )
	PORT_START	/* IN0 */
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_8WAY)
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_8WAY)
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_8WAY)
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY)
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_8WAY | IPF_PLAYER2 )
	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_8WAY | IPF_PLAYER2 )
	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_8WAY | IPF_PLAYER2 )
	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_PLAYER2 )

	PORT_START	/* IN1 */
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_BUTTON1 )
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_BUTTON2 )
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER2 )
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_PLAYER2 )
	PORT_BIT( 0xf0, IP_ACTIVE_LOW, IPT_UNKNOWN )

	PORT_START	/* IN2 */
	PORT_BIT( 0x0f, IP_ACTIVE_LOW, IPT_UNKNOWN )
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON3 | IPF_PLAYER2)
	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON3 )
	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_START2 )
	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_START1 )

	PORT_START	/* IN3 */
	PORT_BIT( 0x03, IP_ACTIVE_LOW, IPT_UNKNOWN )
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_COIN1 )
	PORT_BIT( 0xf8, IP_ACTIVE_LOW, IPT_UNKNOWN )

	PORT_START	/* DIPSW1 */
	PORT_DIPNAME( 0x0f, 0x0e, DEF_STR( Coinage ) )
	PORT_DIPSETTING(    0x0a, DEF_STR( 3C_1C ) )
	PORT_DIPSETTING(    0x0b, "3 Coins/1 Credit 5/2" )
	PORT_DIPSETTING(    0x0c, DEF_STR( 2C_1C ) )
	PORT_DIPSETTING(    0x0d, "2 Coins/1 Credit 3/2" )
	PORT_DIPSETTING(    0x01, DEF_STR( 4C_3C ) )
	PORT_DIPSETTING(    0x0e, DEF_STR( 1C_1C ) )
	PORT_DIPSETTING(    0x03, "2 Coins/2 Credits 3/4" )
	PORT_DIPSETTING(    0x02, DEF_STR( 3C_3C ) )
	PORT_DIPSETTING(    0x0f, DEF_STR( 1C_2C ) )
	PORT_DIPSETTING(    0x04, DEF_STR( 1C_3C ) )
	PORT_DIPSETTING(    0x09, DEF_STR( 1C_4C ) )
	PORT_DIPSETTING(    0x08, DEF_STR( 1C_6C ) )
	PORT_DIPSETTING(    0x07, DEF_STR( 1C_8C ) )
	PORT_DIPSETTING(    0x06, "1 Coin/10 Credits" )
	PORT_DIPSETTING(    0x05, "1 Coin/12 Credits" )
	PORT_DIPSETTING(    0x00, "Free 99 Credits" )
/* 0x10 is different from the Free 99 Credits.
   When you start the game the credits decrease using the Free 99,
   while they stay at 99 using the 0x10 dip. */
	PORT_DIPNAME( 0x10, 0x10, DEF_STR( Free_Play ) )
	PORT_DIPSETTING(    0x10, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_DIPNAME( 0x20, 0x20, "Invert Buttons" )
	PORT_DIPSETTING(    0x20, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_DIPNAME( 0x40, 0x40, DEF_STR( Unknown ) )
	PORT_DIPSETTING(    0x40, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_DIPNAME( 0x80, 0x80, "Trackball" )
	PORT_DIPSETTING(    0x80, "Small" )
	PORT_DIPSETTING(    0x00, "Large" )

	PORT_START	/* DIPSW2 */
	PORT_DIPNAME( 0x03, 0x03, DEF_STR( Lives ) )
	PORT_DIPSETTING(    0x02, "2" )
	PORT_DIPSETTING(    0x03, "3" )
	PORT_DIPSETTING(    0x01, "5" )
	PORT_BITX( 0,       0x00, IPT_DIPSWITCH_SETTING | IPF_CHEAT, "Infinite", IP_KEY_NONE, IP_JOY_NONE )
	PORT_DIPNAME( 0x0c, 0x0c, DEF_STR( Bonus_Life ) )
	PORT_DIPSETTING(    0x0c, "20k 50k" )
	PORT_DIPSETTING(    0x08, "30k 100k" )
	PORT_DIPSETTING(    0x04, "50k 150k" )
	PORT_DIPSETTING(    0x00, "70K" )
	PORT_DIPNAME( 0x30, 0x30, DEF_STR( Difficulty ) )
	PORT_DIPSETTING(    0x30, "Easy" )
	PORT_DIPSETTING(    0x20, "Normal" )
	PORT_DIPSETTING(    0x10, "Hard" )
	PORT_DIPSETTING(    0x00, "Very Hard" )
	PORT_DIPNAME( 0x40, 0x40, DEF_STR( Unknown ) )
	PORT_DIPSETTING(    0x40, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_DIPNAME( 0x80, 0x80, DEF_STR( Demo_Sounds ) )
	PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x80, DEF_STR( On ) )
INPUT_PORTS_END



static struct GfxLayout text_layout =
{
	8,8,	/* 8*8 characters */
	1024,	/* 1024 characters */
	2,	/* 4 bits per pixel */
	{ 0,4 },//0x4000*8+0, 0x4000*8+4, 0, 4 },
	{ 3, 2, 1, 0, 8+3, 8+2, 8+1, 8+0},
	{ 0*16, 1*16, 2*16, 3*16, 4*16, 5*16, 6*16, 7*16 },
	16*8	/* every char takes 16 consecutive bytes */
};

static struct GfxLayout tile_layout =
{
	16,16,  /* 16*16 tiles */
	4*1024,   /* 1024 tiles */
	4,      /* 4 bits per pixel */
	{ 0, 4, 0x40000*8+0, 0x40000*8+4 },
	{ 3, 2, 1, 0, 8+3, 8+2, 8+1, 8+0,
			32*8+3, 32*8+2, 32*8+1, 32*8+0, 33*8+3, 33*8+2, 33*8+1, 33*8+0 },
	{ 0*16, 1*16, 2*16, 3*16, 4*16, 5*16, 6*16, 7*16,
			8*16, 9*16, 10*16, 11*16, 12*16, 13*16, 14*16, 15*16 },
	64*8
};

static struct GfxLayout sprite_layout =
{
	16,16,	/* 16*16 sprites */
	4*1024,	/* 1024 sprites */
	4,	/* 4 bits per pixel */
	{ 0, 4, 0x40000*8+0, 0x40000*8+4 },	/* the two bitplanes for 4 pixels are packed into one byte */
	{ 3, 2, 1, 0, 8+3, 8+2, 8+1, 8+0,
			16+3, 16+2, 16+1, 16+0, 24+3, 24+2, 24+1, 24+0 },
	{ 30*16, 28*16, 26*16, 24*16, 22*16, 20*16, 18*16, 16*16,
			14*16, 12*16, 10*16,  8*16,  6*16,  4*16,  2*16,  0*16 },
	64*8	/* every sprite takes 64 consecutive bytes */
};

static struct GfxDecodeInfo cabal_gfxdecodeinfo[] = {
	{ REGION_GFX1, 0, &text_layout,		0, 1024/4 },
	{ REGION_GFX2, 0, &tile_layout,		32*16, 16 },
	{ REGION_GFX3, 0, &sprite_layout,	16*16, 16 },
	{ -1 }
};

static struct MachineDriver machine_driver_cabal =
{
	{
		{
			CPU_M68000,
			10000000, /* 10 Mhz */
			readmem_cpu,writemem_cpu,0,0,
			m68_level1_irq,1
		},
		{
			CPU_Z80 | CPU_AUDIO_CPU,
			3579545,	/* 3.579545 Mhz */
			readmem_sound,writemem_sound,0,0,
			ignore_interrupt,0
		},
	},
	60, DEFAULT_REAL_60HZ_VBLANK_DURATION,
	1, /* CPU slices per frame */
	seibu_sound_init_1, /* init machine */

	/* video hardware */
	256, 256, { 0, 255, 16, 255-16 },

	cabal_gfxdecodeinfo,
	1024,1024,
	0,
	
	VIDEO_TYPE_RASTER | VIDEO_MODIFIES_PALETTE,
	0,
	generic_vh_start,
	generic_vh_stop,
	cabal_vh_screenrefresh,

	/* sound hardware */
	SOUND_SUPPORTS_STEREO,0,0,0,
	{
	    {
		SOUND_YM2151,
		&ym2151_interface
	    },
	    {
		    SOUND_ADPCM,
		    &adpcm_interface
	    }
	}
};

static struct MachineDriver machine_driver_cabalbl =
{
	{
		{
			CPU_M68000,
			10000000, /* 10 Mhz */
			cabalbl_readmem_cpu,cabalbl_writemem_cpu,0,0,
			m68_level1_irq,1
		},
		{
			CPU_Z80 | CPU_AUDIO_CPU,
			3579545,	/* 3.579545 Mhz */
			cabalbl_readmem_sound,cabalbl_writemem_sound,0,0,
			interrupt,1
		},
	},
	60, DEFAULT_REAL_60HZ_VBLANK_DURATION,
	10, /* CPU slices per frame */
	cabal_init_machine, /* init machine */

	/* video hardware */
	256, 256, { 0, 255, 16, 255-16 },

	cabal_gfxdecodeinfo,
	1024,1024,
	0,

	VIDEO_TYPE_RASTER | VIDEO_MODIFIES_PALETTE,
	0,
	generic_vh_start,
	generic_vh_stop,
	cabal_vh_screenrefresh,

	/* sound hardware */
	SOUND_SUPPORTS_STEREO,0,0,0,
	{
		{
			SOUND_YM2151,//_ALT,
			&ym2151_interface_bl
		},
		{
			SOUND_ADPCM,
			&adpcm_interface
		}
	}
};

ROM_START( cabal )
	ROM_REGION( 0x50000, REGION_CPU1 )	/* 64k for cpu code */
	ROM_LOAD_EVEN( "h7_512.bin",      0x00000, 0x10000, 0x8fe16fb4 )
	ROM_LOAD_ODD ( "h6_512.bin",      0x00000, 0x10000, 0x6968101c )
	ROM_LOAD_EVEN( "k7_512.bin",      0x20000, 0x10000, 0x562031a2 )
	ROM_LOAD_ODD ( "k6_512.bin",      0x20000, 0x10000, 0x4fda2856 )

	ROM_REGION( 0x10000 * 2, REGION_CPU2 )	/* 64k for sound cpu code */
	ROM_LOAD( "4-3n",           0x0000, 0x2000, 0x4038eff2 )
	ROM_LOAD( "3-3p",           0x8000, 0x8000, 0xd9defcbf )

	ROM_REGION( 0x4000,  REGION_GFX1 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "t6_128.bin",     0x00000, 0x04000, 0x1ccee214 ) /* characters */

	/* the gfx ROMs were missing in this set, I'm using the ones from */
	/* the bootleg */
	ROM_REGION( 0x80000, REGION_GFX2 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "cabal_17.bin",   0x00000, 0x10000, 0x3b6d2b09 ) /* tiles, planes0,1 */
	ROM_LOAD( "cabal_16.bin",   0x10000, 0x10000, 0x77bc7a60 )
	ROM_LOAD( "cabal_18.bin",   0x20000, 0x10000, 0x0bc50075 )
	ROM_LOAD( "cabal_19.bin",   0x30000, 0x10000, 0x67e4fe47 )
	ROM_LOAD( "cabal_15.bin",   0x40000, 0x10000, 0x1023319b ) /* tiles, planes2,3 */
	ROM_LOAD( "cabal_14.bin",   0x50000, 0x10000, 0x420b0801 )
	ROM_LOAD( "cabal_12.bin",   0x60000, 0x10000, 0x543fcb37 )
	ROM_LOAD( "cabal_13.bin",   0x70000, 0x10000, 0xd28d921e )

	ROM_REGION( 0x80000, REGION_GFX3 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "cabal_05.bin",   0x00000, 0x10000, 0x4e49c28e ) /* sprites, planes0,1 */
	ROM_LOAD( "cabal_06.bin",   0x10000, 0x10000, 0x6a0e739d )
	ROM_LOAD( "cabal_07.bin",   0x20000, 0x10000, 0x581a50c1 )
	ROM_LOAD( "cabal_08.bin",   0x30000, 0x10000, 0x702735c9 )
	ROM_LOAD( "cabal_04.bin",   0x40000, 0x10000, 0x34d3cac8 ) /* sprites, planes2,3 */
	ROM_LOAD( "cabal_03.bin",   0x50000, 0x10000, 0x7065e840 )
	ROM_LOAD( "cabal_02.bin",   0x60000, 0x10000, 0x0e1ec30e )
	ROM_LOAD( "cabal_01.bin",   0x70000, 0x10000, 0x55c44764 )

	ROM_REGION( 0x20000, REGION_SOUND1 )	/* Samples? */
	ROM_LOAD( "2-1s",           0x00000, 0x10000, 0x850406b4 )
	ROM_LOAD( "1-1u",           0x10000, 0x10000, 0x8b3e0789 )
ROM_END

ROM_START( cabal2 )
	ROM_REGION( 0x50000, REGION_CPU1 )	/* 64k for cpu code */
	ROM_LOAD_EVEN( "9-7h",            0x00000, 0x10000, 0xebbb9484 )
	ROM_LOAD_ODD ( "7-6h",            0x00000, 0x10000, 0x51aeb49e )
	ROM_LOAD_EVEN( "8-7k",            0x20000, 0x10000, 0x4c24ed9a )
	ROM_LOAD_ODD ( "6-6k",            0x20000, 0x10000, 0x681620e8 )

	ROM_REGION( 0x10000 * 2, REGION_CPU2 )	/* 64k for sound cpu code */
	ROM_LOAD( "4-3n",           0x0000, 0x2000, 0x4038eff2 )
	ROM_LOAD( "3-3p",           0x8000, 0x8000, 0xd9defcbf )

	ROM_REGION( 0x4000,  REGION_GFX1 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "5-6s",           0x00000, 0x04000, 0x6a76955a ) /* characters */

	/* the gfx ROMs were missing in this set, I'm using the ones from */
	/* the bootleg */
	ROM_REGION( 0x80000, REGION_GFX2 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "cabal_17.bin",   0x00000, 0x10000, 0x3b6d2b09 ) /* tiles, planes0,1 */
	ROM_LOAD( "cabal_16.bin",   0x10000, 0x10000, 0x77bc7a60 )
	ROM_LOAD( "cabal_18.bin",   0x20000, 0x10000, 0x0bc50075 )
	ROM_LOAD( "cabal_19.bin",   0x30000, 0x10000, 0x67e4fe47 )
	ROM_LOAD( "cabal_15.bin",   0x40000, 0x10000, 0x1023319b ) /* tiles, planes2,3 */
	ROM_LOAD( "cabal_14.bin",   0x50000, 0x10000, 0x420b0801 )
	ROM_LOAD( "cabal_12.bin",   0x60000, 0x10000, 0x543fcb37 )
	ROM_LOAD( "cabal_13.bin",   0x70000, 0x10000, 0xd28d921e )

	ROM_REGION( 0x80000, REGION_GFX3 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "cabal_05.bin",   0x00000, 0x10000, 0x4e49c28e ) /* sprites, planes0,1 */
	ROM_LOAD( "cabal_06.bin",   0x10000, 0x10000, 0x6a0e739d )
	ROM_LOAD( "cabal_07.bin",   0x20000, 0x10000, 0x581a50c1 )
	ROM_LOAD( "cabal_08.bin",   0x30000, 0x10000, 0x702735c9 )
	ROM_LOAD( "cabal_04.bin",   0x40000, 0x10000, 0x34d3cac8 ) /* sprites, planes2,3 */
	ROM_LOAD( "cabal_03.bin",   0x50000, 0x10000, 0x7065e840 )
	ROM_LOAD( "cabal_02.bin",   0x60000, 0x10000, 0x0e1ec30e )
	ROM_LOAD( "cabal_01.bin",   0x70000, 0x10000, 0x55c44764 )

	ROM_REGION( 0x20000, REGION_SOUND1 )	/* Samples? */
	ROM_LOAD( "2-1s",           0x00000, 0x10000, 0x850406b4 )
	ROM_LOAD( "1-1u",           0x10000, 0x10000, 0x8b3e0789 )
ROM_END

ROM_START( cabalbl )
	ROM_REGION( 0x50000, REGION_CPU1 )	/* 64k for cpu code */
	ROM_LOAD_EVEN( "cabal_24.bin",    0x00000, 0x10000, 0x00abbe0c )
	ROM_LOAD_ODD ( "cabal_22.bin",    0x00000, 0x10000, 0x78c4af27 )
	ROM_LOAD_EVEN( "cabal_23.bin",    0x20000, 0x10000, 0xd763a47c )
	ROM_LOAD_ODD ( "cabal_21.bin",    0x20000, 0x10000, 0x96d5e8af )

	ROM_REGION( 0x10000, REGION_CPU2 )	/* 64k for sound cpu code */
	ROM_LOAD( "cabal_11.bin",    0x0000, 0x10000, 0xd308a543 )

	ROM_REGION( 0x4000,  REGION_GFX1 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "5-6s",           0x00000, 0x04000, 0x6a76955a ) /* characters */

	ROM_REGION( 0x80000, REGION_GFX2 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "cabal_17.bin",   0x00000, 0x10000, 0x3b6d2b09 ) /* tiles, planes0,1 */
	ROM_LOAD( "cabal_16.bin",   0x10000, 0x10000, 0x77bc7a60 )
	ROM_LOAD( "cabal_18.bin",   0x20000, 0x10000, 0x0bc50075 )
	ROM_LOAD( "cabal_19.bin",   0x30000, 0x10000, 0x67e4fe47 )
	ROM_LOAD( "cabal_15.bin",   0x40000, 0x10000, 0x1023319b ) /* tiles, planes2,3 */
	ROM_LOAD( "cabal_14.bin",   0x50000, 0x10000, 0x420b0801 )
	ROM_LOAD( "cabal_12.bin",   0x60000, 0x10000, 0x543fcb37 )
	ROM_LOAD( "cabal_13.bin",   0x70000, 0x10000, 0xd28d921e )

	ROM_REGION( 0x80000, REGION_GFX3 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "cabal_05.bin",   0x00000, 0x10000, 0x4e49c28e ) /* sprites, planes0,1 */
	ROM_LOAD( "cabal_06.bin",   0x10000, 0x10000, 0x6a0e739d )
	ROM_LOAD( "cabal_07.bin",   0x20000, 0x10000, 0x581a50c1 )
	ROM_LOAD( "cabal_08.bin",   0x30000, 0x10000, 0x702735c9 )
	ROM_LOAD( "cabal_04.bin",   0x40000, 0x10000, 0x34d3cac8 ) /* sprites, planes2,3 */
	ROM_LOAD( "cabal_03.bin",   0x50000, 0x10000, 0x7065e840 )
	ROM_LOAD( "cabal_02.bin",   0x60000, 0x10000, 0x0e1ec30e )
	ROM_LOAD( "cabal_01.bin",   0x70000, 0x10000, 0x55c44764 )

	ROM_REGION( 0x20000, REGION_SOUND1 )
	ROM_LOAD( "cabal_09.bin",   0x00000, 0x10000, 0x4ffa7fe3 ) /* Z80 code/adpcm data */
	ROM_LOAD( "cabal_10.bin",   0x10000, 0x10000, 0x958789b6 ) /* Z80 code/adpcm data */
ROM_END

#define BIT(x,n) (((x)>>(n))&1)
static UINT8 decrypt_data(int a,int src)
{
    if ( BIT(a,9)  &  BIT(a,8))             src ^= 0x80;
    if ( BIT(a,11) &  BIT(a,4) &  BIT(a,1)) src ^= 0x40;
    if ( BIT(a,11) & ~BIT(a,8) &  BIT(a,1)) src ^= 0x04;
    if ( BIT(a,13) & ~BIT(a,6) &  BIT(a,4)) src ^= 0x02;
    if (~BIT(a,11) &  BIT(a,9) &  BIT(a,2)) src ^= 0x01;

    if (BIT(a,13) &  BIT(a,4)) src = BITSWAP8(src,7,6,5,4,3,2,0,1);
    if (BIT(a, 8) &  BIT(a,4)) src = BITSWAP8(src,7,6,5,4,2,3,1,0);

    return src;
}

static UINT8 decrypt_opcode(int a,int src)
{
    if ( BIT(a,9)  &  BIT(a,8))             src ^= 0x80;
    if ( BIT(a,11) &  BIT(a,4) &  BIT(a,1)) src ^= 0x40;
    if (~BIT(a,13) & BIT(a,12))             src ^= 0x20;
    if (~BIT(a,6)  &  BIT(a,1))             src ^= 0x10;
    if (~BIT(a,12) &  BIT(a,2))             src ^= 0x08;
    if ( BIT(a,11) & ~BIT(a,8) &  BIT(a,1)) src ^= 0x04;
    if ( BIT(a,13) & ~BIT(a,6) &  BIT(a,4)) src ^= 0x02;
    if (~BIT(a,11) &  BIT(a,9) &  BIT(a,2)) src ^= 0x01;

    if (BIT(a,13) &  BIT(a,4)) src = BITSWAP8(src,7,6,5,4,3,2,0,1);
    if (BIT(a, 8) &  BIT(a,4)) src = BITSWAP8(src,7,6,5,4,2,3,1,0);
    if (BIT(a,12) &  BIT(a,9)) src = BITSWAP8(src,7,6,4,5,3,2,1,0);
    if (BIT(a,11) & ~BIT(a,6)) src = BITSWAP8(src,6,7,5,4,3,2,1,0);

    return src;
}


void seibu_sound_decrypt(int cpu_region,int length)
{
    UINT8 *rom = memory_region(cpu_region);
    int diff =  memory_region_length(cpu_region)/2;
    int i;

    memory_set_opcode_base(cpu_region-REGION_CPU1,rom+diff);

    for (i = 0;i < length;i++)
    {
	UINT8 src = rom[i];

	rom[i]      = decrypt_data(i,src);
	rom[i+diff] = decrypt_opcode(i,src);
    }
}

void seibu_adpcm_decrypt(int region)
{
    data8_t *ROM = memory_region(region);
    int i;

    for (i = 0; i < memory_region_length(region); i++)
    {
	ROM[i] = BITSWAP8(ROM[i], 7, 5, 3, 1, 6, 4, 2, 0);
    }
}




static void init_cabal(void)
{
    seibu_sound_decrypt(REGION_CPU2,0x2000);
    seibu_adpcm_decrypt(REGION_SOUND1);
}


GAMEX( 1988, cabal,   0,     cabal,   cabal, cabal, ROT0, "Tad (Fabtek license)", "Cabal (US set 1)", GAME_NO_COCKTAIL )
GAMEX( 1988, cabal2,  cabal, cabal,   cabal, cabal, ROT0, "Tad (Fabtek license)", "Cabal (US set 2)", GAME_NO_COCKTAIL )
GAMEX( 1988, cabalbl, cabalbl, cabalbl, cabalbl, 0, ROT0, "bootleg", "Cabal (bootleg)", GAME_IMPERFECT_SOUND | GAME_NO_COCKTAIL )
