#include "../vidhrdw/toypop.c"

/****************************************

TOYPOP
1986 Namco

driver by Edgardo E. Contini Salvan (pag2806@iperbole.bologna.it)

TOYPOP uses a 6809 main CPU,
another 6809 for the sound and
a 68000 to create the background image.

Libble Rabble should run on the same board

****************************************/

#include "driver.h"
#include "sound/namco.h"
#include "vidhrdw/generic.h"
#include "machine/namcoio.h"


/***************************************************************************

  Custom I/O initialization

***************************************************************************/

static READ_HANDLER( in0_l )	{ return readinputport(0); }		// P1 joystick
static READ_HANDLER( in0_h )	{ return readinputport(0) >> 4; }	// P2 joystick
static READ_HANDLER( in1_l )	{ return readinputport(1); }		// fire and start buttons
static READ_HANDLER( in1_h )	{ return readinputport(1) >> 4; }	// coins
static READ_HANDLER( dipA_l )	{ return readinputport(2); }		// dips A
static READ_HANDLER( dipA_h )	{ return readinputport(2) >> 4; }	// dips A
static READ_HANDLER( dipB_l )	{ return readinputport(3); }		// dips B
static READ_HANDLER( dipB_h )	{ return readinputport(3) >> 4; }	// dips B
static READ_HANDLER( in2_l )	{ return readinputport(4); }		// P1 joystick left in liblrabl
static READ_HANDLER( in2_h )	{ return readinputport(4) >> 4; }	// P2 joystick left in liblrabl
static READ_HANDLER( in3 )		{ return readinputport(5); }		// test, cocktail, optional buttons

static WRITE_HANDLER( out_coin0 )
{
	coin_lockout_global_w(0, data & 4);
	coin_counter_w(0,~data & 8);
}

static WRITE_HANDLER( out_coin1 )
{
	coin_counter_w(1,~data & 1);
}

static WRITE_HANDLER( toypop_flip )
{
	 flip_screen_w(0, data & 1);
}

/* chip #0: player inputs, buttons, coins */
static struct namcoio_interface intf0_coin =
{
	{ in1_h, in0_l, in0_h, in1_l },	/* port read handlers */
	{ out_coin0, out_coin1 }		/* port write handlers */
};
static struct namcoio_interface intf0 =
{
	{ in1_h, in0_l, in0_h, in1_l },	/* port read handlers */
	{ NULL, NULL }					/* port write handlers */
};

/* chip #1: dip switches */
static struct namcoio_interface intf1 =
{
	{ dipA_h, dipB_l, dipB_h, dipA_l },	/* port read handlers */
	{ toypop_flip, NULL }						/* port write handlers */
};

/* chip #2: test/cocktail, optional buttons */
static struct namcoio_interface intf2 =
{
	{ NULL, in2_l, in2_h, in3 },	/* port read handlers */
	{ NULL, NULL }					/* port write handlers */
};

static void init_58c_56_56 (void)
{
	namcoio_init(0, NAMCOIO_58XX, &intf0_coin);
	namcoio_init(1, NAMCOIO_56XX, &intf1);
	namcoio_init(2, NAMCOIO_56XX, &intf2);
}

static void init_58_56_56 (void)
{
	namcoio_init(0, NAMCOIO_58XX, &intf0);
	namcoio_init(1, NAMCOIO_56XX, &intf1);
	namcoio_init(2, NAMCOIO_56XX, &intf2);
}


/***************************************************************************/


static int interrupt_enable_68k;
static data8_t *toypop_m68000_sharedram;


READ_HANDLER( toypop_sound_sharedram_r )
{
	return namco_soundregs[offset];
}

WRITE_HANDLER( toypop_sound_sharedram_w )
{
	if (offset < 0x40)
		namco_15xx_w(offset,data);
	else
		namco_soundregs[offset] = data;
}

//READ16_HANDLER( toypop_m68000_sharedram_r )
READ_HANDLER( toypop_m68000_sharedram_r )
{
	return toypop_m68000_sharedram[offset>>1];
}

//WRITE16_HANDLER( toypop_m68000_sharedram_w )
WRITE_HANDLER( toypop_m68000_sharedram_w )
{
	if (ACCESSING_LSB)
		toypop_m68000_sharedram[offset>>1] = data & 0xff;
}

READ_HANDLER( toypop_main_interrupt_enable_r )
{
	cpu_interrupt_enable(0,1);
	return 0;
}

WRITE_HANDLER( toypop_main_interrupt_enable_w )
{
	cpu_interrupt_enable(0,1);
	cpu_set_irq_line(0, 0, CLEAR_LINE);
}

WRITE_HANDLER( toypop_main_interrupt_disable_w )
{
	cpu_interrupt_enable(0,0);
}

WRITE_HANDLER( toypop_sound_interrupt_enable_acknowledge_w )
{
	cpu_interrupt_enable(1,1);
	cpu_set_irq_line(1, 0, CLEAR_LINE);
}

WRITE_HANDLER( toypop_sound_interrupt_disable_w )
{
	cpu_interrupt_enable(1,0);
}

int toypop_main_interrupt ( void )
{
	namcoio_set_irq_line(0,PULSE_LINE);
	namcoio_set_irq_line(1,PULSE_LINE);
	namcoio_set_irq_line(2,PULSE_LINE);

	//irq0_line_assert();	// this also checks if irq is enabled - IMPORTANT!
						// so don't replace with cpu_set_irq_line(0, 0, ASSERT_LINE);

	return interrupt();
}

WRITE_HANDLER( toypop_sound_clear_w )
{
	cpu_set_reset_line(1, CLEAR_LINE);
}

WRITE_HANDLER( toypop_sound_assert_w )
{
	cpu_set_reset_line(1, ASSERT_LINE);
}

WRITE_HANDLER( toypop_m68000_clear_w )
{
	cpu_set_reset_line(2, CLEAR_LINE);
}

WRITE_HANDLER( toypop_m68000_assert_w )
{
	cpu_set_reset_line(2, ASSERT_LINE);
}

void init_machine_toypop ( void )
{
	cpu_interrupt_enable(0,0);
	cpu_set_irq_line(0, 0, CLEAR_LINE);
	cpu_interrupt_enable(1,0);
	cpu_set_irq_line(1, 0, CLEAR_LINE);
	interrupt_enable_68k = 0;
}

int toypop_m68000_interrupt ( void )
{
	if (interrupt_enable_68k)
		//cpu_set_irq_line(2, 6, HOLD_LINE);
	    return 6;
	else
	    return ignore_interrupt();
}

static WRITE_HANDLER( toypop_68k_irq_trigger_w )
{
	if (interrupt_enable_68k)
		cpu_set_irq_line(2, 6, HOLD_LINE);
}

//WRITE16_HANDLER( toypop_m68000_interrupt_enable_w )
WRITE_HANDLER( toypop_m68000_interrupt_enable_w )
{
	interrupt_enable_68k = 1;
}

//WRITE16_HANDLER( toypop_m68000_interrupt_disable_w )
WRITE_HANDLER( toypop_m68000_interrupt_disable_w )
{
	interrupt_enable_68k = 0;
}







static struct MemoryReadAddress readmem_mainCPU_liblrabl[] =
{
	{ 0x0000, 0x2fff, MRA_RAM },							
	{ 0x6000, 0x63ff, toypop_sound_sharedram_r },			/* shared RAM with the 68000 CPU */
	{ 0x6800, 0x683f, namcoio_r  },					/* custom I/O */
	{ 0x7800, 0x7800, watchdog_reset_r },
	{ 0x8000, 0xffff, MRA_ROM },					/* ROM code */
	{ -1 }	/* end of table */
};

static struct MemoryReadAddress readmem_mainCPU_toypop[] =
{
	{ 0x0000, 0x2fff, MRA_RAM },							
	{ 0x6000, 0x603f, namcoio_r  },					/* custom I/O */
	{ 0x6800, 0x6bff, toypop_sound_sharedram_r },			/* shared RAM with the 68000 CPU */
	{ 0x7000, 0x7000, toypop_main_interrupt_enable_r },		/* enable interrupt */
	{ 0x8000, 0xffff, MRA_ROM },					/* ROM code */
	{ -1 }	/* end of table */
};


static struct MemoryWriteAddress writemem_mainCPU_liblrabl[] =
{
	{ 0x0000, 0x07ff, toypop_videoram_w, &toypop_videoram},		/* video RAM */
	{ 0x0800, 0x0f7f, MWA_RAM },					/* general RAM, area 1 */
	{ 0x0f80, 0x0fff, MWA_RAM, &spriteram, &spriteram_size },	/* sprite RAM, area 1 */
	{ 0x1000, 0x177f, MWA_RAM },					/* general RAM, area 2 */
	{ 0x1780, 0x17ff, MWA_RAM, &spriteram_2 },			/* sprite RAM, area 2 */
	{ 0x1800, 0x1f7f, MWA_RAM },					/* general RAM, area 3 */
	{ 0x1f80, 0x1fff, MWA_RAM, &spriteram_3 },			/* sprite RAM, area 3 */
	{ 0x2800, 0x2fff, MWA_RAM, &toypop_m68000_sharedram },		/* shared RAM with the 68000 CPU */
	{ 0x6000, 0x63ff, toypop_sound_sharedram_w },			/* shared RAM with sound CPU */
	{ 0x6800, 0x683f, namcoio_w },					/* custom I/O */
	{ 0x7000, 0x7000, toypop_main_interrupt_enable_w },		/* enable interrupt */
	{ 0x7800, 0x7800, toypop_main_interrupt_disable_w },		/* disable interrupt */
	{ 0x8000, 0x8000, toypop_m68000_clear_w },			/* reset 68000 */
	{ 0x8800, 0x8800, toypop_m68000_assert_w },			/* reset 68000 */
	{ 0x9000, 0x9000, toypop_sound_clear_w },			/* sound CPU reset */
	{ 0x9800, 0x9800, toypop_sound_assert_w },			/* sound CPU reset */
	{ 0xa000, 0xa001, toypop_palettebank_w },			/* background image palette */
	{ 0x8000, 0xffff, MWA_ROM },					/* ROM code */
	{ -1 }	/* end of table */
};

static struct MemoryWriteAddress writemem_mainCPU_toypop[] =
{
	{ 0x0000, 0x07ff, toypop_videoram_w, &toypop_videoram},		/* video RAM */
	{ 0x0800, 0x0f7f, MWA_RAM },					/* general RAM, area 1 */
	{ 0x0f80, 0x0fff, MWA_RAM, &spriteram, &spriteram_size },	/* sprite RAM, area 1 */
	{ 0x1000, 0x177f, MWA_RAM },					/* general RAM, area 2 */
	{ 0x1780, 0x17ff, MWA_RAM, &spriteram_2 },			/* sprite RAM, area 2 */
	{ 0x1800, 0x1f7f, MWA_RAM },					/* general RAM, area 3 */
	{ 0x1f80, 0x1fff, MWA_RAM, &spriteram_3 },			/* sprite RAM, area 3 */
	{ 0x2000, 0x27ff, MWA_RAM },					/* general RAM, area 4 */
	{ 0x2800, 0x2fff, MWA_RAM, &toypop_m68000_sharedram },		/* shared RAM with the 68000 CPU */
	{ 0x6000, 0x603f, namcoio_w },					/* custom I/O */
	{ 0x6800, 0x6bff, toypop_sound_sharedram_w },			/* shared RAM with sound CPU */
	{ 0x7000, 0x7000, toypop_main_interrupt_disable_w },		/* disable interrupt */
	{ 0x8000, 0x8000, toypop_m68000_clear_w },			/* reset 68000 */
	{ 0x8800, 0x8800, toypop_m68000_assert_w },			/* reset 68000 */
	{ 0x9000, 0x9000, toypop_sound_clear_w },			/* sound CPU reset */
	{ 0x9800, 0x9800, toypop_sound_assert_w },			/* sound CPU reset */
	{ 0xa000, 0xa001, toypop_palettebank_w },			/* background image palette */
	{ 0x8000, 0xffff, MWA_ROM },					/* ROM code */
	{ -1 }	/* end of table */
};


/*************************************
 *
 *      Sound CPU memory handlers
 *
 *************************************/

static struct MemoryReadAddress readmem_soundCPU[] =
{
	{ 0x0000, 0x03ff, toypop_sound_sharedram_r },			/* shared RAM with the main CPU */
	{ 0xe000, 0xffff, MRA_ROM },		    			/* ROM code */

	{ -1 }	/* end of table */
};

static struct MemoryWriteAddress writemem_soundCPU[] =
{
	{ 0x0000, 0x03ff, toypop_sound_sharedram_w, &namco_soundregs },	/* sound control registers */
	{ 0x2000, 0x2000, toypop_sound_interrupt_disable_w },		/* ??? toypop doesn't write here */
	{ 0x4000, 0x4000, toypop_sound_interrupt_enable_acknowledge_w },
	{ 0x6000, 0x6000, watchdog_reset_w },				/* watchdog */
	{ 0xe000, 0xffff, MWA_ROM },					/* ROM code */
	{ -1 }	/* end of table */
};


/*************************************
 *
 *      68k CPU memory handlers
 *
 *************************************/

static struct MemoryReadAddress toypop_readmem_68k[] =
{
	{ 0x000000, 0x007fff, MRA_ROM },				/* ROM code */
	//{ 0x080000, 0x080001, toypop_cycle_r },			/* speed hack */
	{ 0x080000, 0x0bffff, MRA_BANK1 },				/* RAM */
	{ 0x100000, 0x100fff, toypop_m68000_sharedram_r },		/* shared RAM with the main CPU */
	{ 0x180000, 0x187fff, toypop_merged_background_r },		/* RAM merged with the background image */
	{ 0x190000, 0x1dffff, MRA_BANK2 },				/* RAM */
	{ -1 }	/* end of table */
};

static struct MemoryWriteAddress toypop_writemem_68k[] =
{
	{ 0x000000, 0x007fff, MWA_ROM },				/* ROM code */
	//{ 0x080000, 0x0bffff, MWA_BANK1, &toypop_speedup },		/* RAM */
	{ 0x080000, 0x0bffff, MWA_BANK1 },				/* RAM */
	{ 0x100000, 0x100fff, toypop_m68000_sharedram_w },		/* shared RAM with the main CPU */
	{ 0x180000, 0x187fff, toypop_merged_background_w },		/* RAM that has to be merged with the background image */
	{ 0x18fffc, 0x18ffff, toypop_flipscreen_w },			/* flip mode */
	{ 0x190000, 0x1dffff, MWA_BANK2, &toypop_bg_image },		/* RAM */
	{ 0x300000, 0x300001, toypop_m68000_interrupt_enable_w },	/* interrupt enable */
	{ 0x380000, 0x380001, toypop_m68000_interrupt_disable_w },	/* interrupt disable */
	{ -1 }	/* end of table */
};

//////////////////////////////////////////////////////////////////////////////////
//

INPUT_PORTS_START( liblrabl )
            /* The inputs are not memory mapped, they are handled by three I/O chips. */
            PORT_START      /* 58XX #0 pins 22-29 */
	    PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_JOYSTICKRIGHT_UP    | IPF_8WAY )
            PORT_BIT(0x02, IP_ACTIVE_LOW, IPT_JOYSTICKRIGHT_RIGHT | IPF_8WAY )
            PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_JOYSTICKRIGHT_DOWN  | IPF_8WAY )
            PORT_BIT(0x08, IP_ACTIVE_LOW, IPT_JOYSTICKRIGHT_LEFT  | IPF_8WAY )
            PORT_BIT(0x10, IP_ACTIVE_LOW, IPT_JOYSTICKRIGHT_UP    | IPF_8WAY | IPF_COCKTAIL )
            PORT_BIT(0x20, IP_ACTIVE_LOW, IPT_JOYSTICKRIGHT_RIGHT | IPF_8WAY | IPF_COCKTAIL )
            PORT_BIT(0x40, IP_ACTIVE_LOW, IPT_JOYSTICKRIGHT_DOWN  | IPF_8WAY | IPF_COCKTAIL )
            PORT_BIT(0x80, IP_ACTIVE_LOW, IPT_JOYSTICKRIGHT_LEFT  | IPF_8WAY | IPF_COCKTAIL )

            PORT_START      /* 58XX #0 pins 30-33 and 38-41 */
	    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_BUTTON1 )
            PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER2 )
            PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_START1 )
            PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_START2 )
            PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_COIN1 )
            PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_COIN2 )
            PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNUSED )
            PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_SERVICE1 )

	    PORT_START      /* 56XX #1 pins 22-29 */
	    PORT_DIPNAME( 0x03, 0x03, DEF_STR( Lives ) )
	    PORT_DIPSETTING(    0x02, "1" )
	    PORT_DIPSETTING(    0x00, "2" )
	    PORT_DIPSETTING(    0x03, "3" )
	    PORT_DIPSETTING(    0x01, "5" )
	    PORT_DIPNAME( 0x1c, 0x1c, DEF_STR( Bonus_Life ) )
	    PORT_DIPSETTING(    0x04, "40k 120k and every 120k" )
	    PORT_DIPSETTING(    0x1c, "40k 120k 200k 400k..." )
	    PORT_DIPSETTING(    0x10, "40k 120k 200k" )
	    PORT_DIPSETTING(    0x0c, "40k 140k 250k 400k..." )
	    PORT_DIPSETTING(    0x18, "40k 150k and every 150k" )
	    PORT_DIPSETTING(    0x14, "50k 150k 300k 500k..." )
	    PORT_DIPSETTING(    0x08, "50k 150k 300k" )
	    PORT_DIPSETTING(    0x00, "None" )
	    PORT_DIPNAME( 0xe0, 0xe0, DEF_STR( Coin_A ) )
	    PORT_DIPSETTING(    0x80, DEF_STR( 3C_1C ) )
	    PORT_DIPSETTING(    0xc0, DEF_STR( 2C_1C ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( 3C_2C ) )
	    PORT_DIPSETTING(    0xe0, DEF_STR( 1C_1C ) )
	    PORT_DIPSETTING(    0x40, DEF_STR( 2C_3C ) )
	    PORT_DIPSETTING(    0x60, DEF_STR( 1C_2C ) )
	    PORT_DIPSETTING(    0xa0, DEF_STR( 1C_3C ) )
	    PORT_DIPSETTING(    0x20, DEF_STR( 1C_6C ) )

	    PORT_START      /* 56XX #1 pins 30-33 and 38-41 */
	    PORT_DIPNAME( 0x01, 0x01, "Freeze" )
	    PORT_DIPSETTING(    0x01, DEF_STR( Off ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	    PORT_DIPNAME( 0x02, 0x02, "Rack Test" )
	    PORT_DIPSETTING(    0x02, DEF_STR( Off ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	    PORT_DIPNAME( 0x04, 0x04, DEF_STR( Demo_Sounds ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
	    PORT_DIPSETTING(    0x04, DEF_STR( On ) )
	    PORT_DIPNAME( 0x18, 0x18, DEF_STR( Coin_B ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( 2C_1C ) )
	    PORT_DIPSETTING(    0x18, DEF_STR( 1C_1C ) )
	    PORT_DIPSETTING(    0x08, DEF_STR( 1C_5C ) )
	    PORT_DIPSETTING(    0x10, DEF_STR( 1C_7C ) )
	    PORT_DIPNAME( 0x20, 0x20, "Practice" )
	    PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
	    PORT_DIPSETTING(    0x20, DEF_STR( On ) )
	    PORT_DIPNAME( 0xc0, 0xc0, DEF_STR( Difficulty ) )
	    PORT_DIPSETTING(    0xc0, "A" )
	    PORT_DIPSETTING(    0x40, "B" )

	    PORT_START      /* 56XX #2 pins 22-29 */
	    PORT_BIT(0x01, IP_ACTIVE_LOW, IPT_JOYSTICKLEFT_UP    | IPF_8WAY )
            PORT_BIT(0x02, IP_ACTIVE_LOW, IPT_JOYSTICKLEFT_RIGHT | IPF_8WAY )
            PORT_BIT(0x04, IP_ACTIVE_LOW, IPT_JOYSTICKLEFT_DOWN  | IPF_8WAY )
            PORT_BIT(0x08, IP_ACTIVE_LOW, IPT_JOYSTICKLEFT_LEFT  | IPF_8WAY )
            PORT_BIT(0x10, IP_ACTIVE_LOW, IPT_JOYSTICKLEFT_UP    | IPF_8WAY | IPF_COCKTAIL )
            PORT_BIT(0x20, IP_ACTIVE_LOW, IPT_JOYSTICKLEFT_RIGHT | IPF_8WAY | IPF_COCKTAIL )
            PORT_BIT(0x40, IP_ACTIVE_LOW, IPT_JOYSTICKLEFT_DOWN  | IPF_8WAY | IPF_COCKTAIL )
            PORT_BIT(0x80, IP_ACTIVE_LOW, IPT_JOYSTICKLEFT_LEFT  | IPF_8WAY | IPF_COCKTAIL )

            PORT_START      /* 56XX #2 pins 30-33 */
	    PORT_BIT( 0x03, IP_ACTIVE_LOW, IPT_UNUSED )
            PORT_DIPNAME( 0x04, 0x04, DEF_STR( Cabinet ) )
            PORT_DIPSETTING(    0x04, DEF_STR( Upright ) )
            PORT_DIPSETTING(    0x00, DEF_STR( Cocktail ) )
            PORT_SERVICE( 0x08, IP_ACTIVE_LOW )
INPUT_PORTS_END



INPUT_PORTS_START( toypop )
	    /* The inputs are not memory mapped, they are handled by three I/O chips. */
	    PORT_START      /* 58XX #0 pins 22-29 */
	    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_4WAY )
	    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_4WAY )
	    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_4WAY )
	    PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_4WAY )
	    PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_4WAY | IPF_PLAYER2 )
	    PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_4WAY | IPF_PLAYER2 )
	    PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_4WAY | IPF_PLAYER2 )
	    PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_4WAY | IPF_PLAYER2 )

	    PORT_START      /* 58XX #0 pins 30-33 and 38-41 */
	    PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_BUTTON1 )
	    PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_PLAYER2 )
	    PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_START1 )
	    PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_START2 )
	    PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_COIN1 )
	    PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_COIN2 )
	    PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNUSED )
	    PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_SERVICE1 )

	    PORT_START      /* 56XX #1 pins 22-29 */
	    PORT_DIPNAME( 0x03, 0x03, DEF_STR( Lives ) )
	    PORT_DIPSETTING(    0x02, "1" )
	    PORT_DIPSETTING(    0x01, "2" )
	    PORT_DIPSETTING(    0x03, "3" )
	    PORT_DIPSETTING(    0x00, "5" )
	    PORT_DIPNAME( 0x0c, 0x0c, DEF_STR( Coin_B ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( 3C_1C ) )
	    PORT_DIPSETTING(    0x04, DEF_STR( 2C_1C ) )
	    PORT_DIPSETTING(    0x0c, DEF_STR( 1C_1C ) )
	    PORT_DIPSETTING(    0x08, DEF_STR( 1C_2C ) )
	    PORT_DIPNAME( 0x30, 0x30, DEF_STR( Coin_A ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( 3C_1C ) )
            PORT_DIPSETTING(    0x10, DEF_STR( 2C_1C ) )
            PORT_DIPSETTING(    0x30, DEF_STR( 1C_1C ) )
            PORT_DIPSETTING(    0x20, DEF_STR( 1C_2C ) )
            PORT_DIPNAME( 0x40, 0x00, DEF_STR( Flip_Screen ) )
            PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
            PORT_DIPSETTING(    0x40, DEF_STR( On ) )
            PORT_SERVICE( 0x80, IP_ACTIVE_LOW )

	    PORT_START      /* 56XX #1 pins 30-33 and 38-41 */
	    PORT_DIPNAME( 0x01, 0x01, "Freeze" )
	    PORT_DIPSETTING(    0x01, DEF_STR( Off ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	    PORT_DIPNAME( 0x02, 0x02, "Level Select" )
	    PORT_DIPSETTING(    0x02, DEF_STR( Off ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	    PORT_DIPNAME( 0x04, 0x04, "2 Players Game" )
	    PORT_DIPSETTING(    0x00, "1 Credit" )
	    PORT_DIPSETTING(    0x04, "2 Credits" )
	    PORT_DIPNAME( 0x08, 0x08, DEF_STR( Demo_Sounds ) )
	    PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
	    PORT_DIPSETTING(    0x08, DEF_STR( On ) )
	    PORT_DIPNAME( 0x10, 0x10, "Entering" )  // ???
	    PORT_DIPSETTING(    0x00, DEF_STR( Off ) )
	    PORT_DIPSETTING(    0x10, DEF_STR( On ) )
	    PORT_DIPNAME( 0x60, 0x60, DEF_STR( Difficulty ) )
	    PORT_DIPSETTING(    0x40, "Easy" )
	    PORT_DIPSETTING(    0x60, "Normal" )
	    PORT_DIPSETTING(    0x20, "Hard" )
	    PORT_DIPSETTING(    0x00, "Very hard" )
	    PORT_DIPNAME( 0x80, 0x80, DEF_STR( Bonus_Life ) )
	    PORT_DIPSETTING(    0x80, "Every 15000 points" )
	    PORT_DIPSETTING(    0x00, "Every 20000 points" )

	    PORT_START      /* 56XX #2 pins 22-29 */
	    PORT_BIT(0xff, IP_ACTIVE_LOW, IPT_UNUSED )


	    PORT_START      /* 56XX #2 pins 22-29 */
	    PORT_BIT(0xff, IP_ACTIVE_LOW, IPT_UNUSED )
            PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_UNUSED )
            PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_UNUSED )     // would be Cabinet, but this game has no cocktail mode
            PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_SERVICE )    // service mode again
INPUT_PORTS_END



///////////////////////////////////////////////////////////////////////////////////

static struct GfxLayout toypop_charlayout =
{
	8,8,             /* 8*8 characters */
	RGN_FRAC(1,1),   /* 512 characters */
	2,             /* 2 bits per pixel */
	{ 0, 4 },      /* the two bitplanes for 4 pixels are packed into one byte */
	{ 8*8+0, 8*8+1, 8*8+2, 8*8+3, 0, 1, 2, 3 },   /* bits are packed in groups of four */
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },   /* v offset */
	16*8           /* every char takes 16 bytes */
};

static struct GfxLayout toypop_spritelayout =
{
	16,16,		   /* 16*16 sprites */
	RGN_FRAC(1,1),     /* 256 sprites */
	2,                 /* 2 bits per pixel */
	{ 0, 4 },   /* the two bitplanes for 4 pixels are packed into one byte */
	{ 0, 1, 2, 3, 8*8, 8*8+1, 8*8+2, 8*8+3, 16*8+0, 16*8+1, 16*8+2, 16*8+3,
	24*8+0, 24*8+1, 24*8+2, 24*8+3 },
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8,
	32 * 8, 33 * 8, 34 * 8, 35 * 8, 36 * 8, 37 * 8, 38 * 8, 39 * 8 },
	64*8    /* every sprite takes 64 bytes */
};

static struct GfxDecodeInfo toypop_gfxdecodeinfo[] =
{
	{ REGION_GFX1, 0, &toypop_charlayout,      0, 128 },
	{ REGION_GFX2, 0, &toypop_spritelayout, 128*4, 256 },
	{ -1 } /* end of array */
};

/////////////////////////////////////////////////////////////////////////////////////

static struct namco_interface namco_interface =
{
	24000,	/* sample rate (approximate value) */
	8,	/* number of voices */
	100,	/* playback volume */
	REGION_SOUND1	/* memory region */
};

static struct MachineDriver machine_driver_toypop =
{
	/* basic machine hardware */
	{
		{
			CPU_M6809,
			1536000,	/* 1.536 Mhz */
			readmem_mainCPU_toypop,writemem_mainCPU_toypop,0,0,
			toypop_main_interrupt,1
		},
		{
			CPU_M6809 | CPU_AUDIO_CPU,
			1536000,	/* 1.536 Mhz */
			readmem_soundCPU,writemem_soundCPU,0,0,
			interrupt,1
		},
		{
			CPU_M68000,
			6144000,	/* 6.144 Mhz */
			toypop_readmem_68k,toypop_writemem_68k,0,0,
			toypop_m68000_interrupt,1
		}
	},
	60, DEFAULT_60HZ_VBLANK_DURATION,	/* frames per second, vblank duration */
	100,    /* 100 CPU slices per frame - an high value to ensure proper */
			/* synchronization of the CPUs */
	init_machine_toypop,

	/* video hardware */
	18*16, 14*16, { 0*16, 18*16-1, 0*16, 14*16-1 },
	toypop_gfxdecodeinfo,
	256,128*4+64*4,
	toypop_vh_convert_color_prom,

	VIDEO_TYPE_RASTER,
	0,
	toypop_vh_start,
	toypop_vh_stop,
	toypop_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0,
	{
		{
			SOUND_NAMCO,
			&namco_interface
		}
	}
};


static struct MachineDriver machine_driver_liblrabl =
{
	/* basic machine hardware */
	{
		{
			CPU_M6809,
			1536000,	/* 1.536 Mhz */
			readmem_mainCPU_liblrabl,writemem_mainCPU_liblrabl,0,0,
			toypop_main_interrupt,1
		},
		{
			CPU_M6809 | CPU_AUDIO_CPU,
			1536000,	/* 1.536 Mhz */
			readmem_soundCPU,writemem_soundCPU,0,0,
			interrupt,1
		},
		{
			CPU_M68000,
			6144000,	/* 6.144 Mhz */
			toypop_readmem_68k,toypop_writemem_68k,0,0,
			toypop_m68000_interrupt,1
		}
	},
	60, DEFAULT_60HZ_VBLANK_DURATION,	/* frames per second, vblank duration */
	100,    /* 100 CPU slices per frame - an high value to ensure proper */
			/* synchronization of the CPUs */
	init_machine_toypop,

	/* video hardware */
	18*16, 14*16, { 0*16, 18*16-1, 0*16, 14*16-1 },
	toypop_gfxdecodeinfo,
	256,128*4+64*4,
	toypop_vh_convert_color_prom,

	VIDEO_TYPE_RASTER,
	0,
	toypop_vh_start,
	toypop_vh_stop,
	toypop_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0,
	{
		{
			SOUND_NAMCO,
			&namco_interface
		}
	}
};


ROM_START( liblrabl )
	ROM_REGION( 0x10000, REGION_CPU1 )	/* 64k for the first CPU */
	ROM_LOAD( "5b.rom",   0x8000, 0x4000, 0xda7a93c2 )
	ROM_LOAD( "5c.rom",   0xc000, 0x4000, 0x6cae25dc )

	ROM_REGION( 0x10000, REGION_CPU2 )	/* 64k for the second CPU */
	ROM_LOAD( "2c.rom",   0xe000, 0x2000, 0x7c09e50a )

	ROM_REGION( 0x8000, REGION_CPU3 )		/* 32k for the third CPU */
	ROM_LOAD_EVEN( "8c.rom",   0x0000, 0x4000, 0xa00cd959 )
	ROM_LOAD_ODD( "10c.rom",   0x0000, 0x4000, 0x09ce209b )

	/* temporary space for graphics (disposed after conversion) */
	ROM_REGION( 0x2000, REGION_GFX1 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "5p.rom",   0x0000, 0x2000, 0x3b4937f0 )	/* characters */

	ROM_REGION( 0x4000, REGION_GFX2 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "9t.rom",   0x0000, 0x4000, 0xa88e24ca )	/* sprites */

	ROM_REGION( 0x0600, REGION_PROMS )	/* color proms */
	ROM_LOAD( "lr1-3.1r", 0x0000, 0x0100, 0xf3ec0d07 )	// palette: red component
	ROM_LOAD( "lr1-2.1s", 0x0100, 0x0100, 0x2ae4f702 )	// palette: green component
	ROM_LOAD( "lr1-1.1t", 0x0200, 0x0100, 0x7601f208 )	// palette: blue component
	ROM_LOAD( "lr1-5.5l", 0x0300, 0x0100, 0x940f5397 )	/* characters */
	ROM_LOAD( "lr1-6.2p", 0x0400, 0x0200, 0xa6b7f850 )	/* sprites */

	ROM_REGION( 0x0100, REGION_SOUND1 )	/* sound prom */
	ROM_LOAD( "lr1-4.3d", 0x0000, 0x0100, 0x16a9166a )
ROM_END

ROM_START( toypop )
	ROM_REGION( 0x10000, REGION_CPU1 )	/* 64k for the first CPU */
	ROM_LOAD( "tp1-2.5b", 0x8000, 0x4000, 0x87469620 )
	ROM_LOAD( "tp1-1.5c", 0xc000, 0x4000, 0xdee2fd6e )

	ROM_REGION( 0x10000, REGION_CPU2 )	/* 64k for the second CPU */
	ROM_LOAD( "tp1-3.2c", 0xe000, 0x2000, 0x5f3bf6e2 )

	ROM_REGION( 0x8000, REGION_CPU3 )		/* 32k for the third CPU */
	ROM_LOAD_EVEN( "tp1-4.8c", 0x0000, 0x4000, 0x76997db3 )
	ROM_LOAD_ODD( "tp1-5.10c", 0x0000, 0x4000, 0x37de8786 )

	/* temporary space for graphics (disposed after conversion) */
	ROM_REGION( 0x2000, REGION_GFX1 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "tp1-7.5p", 0x0000, 0x2000, 0x95076f9e )	/* characters */

	ROM_REGION( 0x4000, REGION_GFX2 | REGIONFLAG_DISPOSE )
	ROM_LOAD( "tp1-6.9t", 0x0000, 0x4000, 0x481ffeaf )	/* sprites */

	ROM_REGION( 0x0600, REGION_PROMS )	/* color proms */
	ROM_LOAD( "tp1-3.1r", 0x0000, 0x0100, 0xcfce2fa5 )	// palette: red component
	ROM_LOAD( "tp1-2.1s", 0x0100, 0x0100, 0xaeaf039d )	// palette: green component
	ROM_LOAD( "tp1-1.1t", 0x0200, 0x0100, 0x08e7cde3 )	// palette: blue component
	ROM_LOAD( "tp1-4.5l", 0x0300, 0x0100, 0x74138973 )	/* characters */
	ROM_LOAD( "tp1-5.2p", 0x0400, 0x0200, 0x4d77fa5a )	/* sprites */

	ROM_REGION( 0x0100, REGION_SOUND1 )	/* sound prom */
	ROM_LOAD( "lr1-4.3d", 0x0000, 0x0100, 0x16a9166a )
ROM_END


GAME( 1983, liblrabl, 0, liblrabl,  liblrabl, 58_56_56,	ROT0, "Namco", "Libble Rabble")
GAME( 1986, toypop,   0, toypop,    toypop, 58c_56_56,	ROT0, "Namco", "Toypop" )
