enum
{
    VECTOR_INIT,
    RST10_ASSERT,
    RST10_CLEAR,
    RST18_ASSERT,
    RST18_CLEAR
};


static int sound_cpu=1;
static UINT8 main2sub[2],sub2main[2];
static int last[4];
static int main2sub_pending,sub2main_pending;
static int start, end, start1, end1;

READ_HANDLER( seibu_main_word_r );
WRITE_HANDLER( seibu_main_word_w );


static void setvector_callback(int param)
{
    static int irq1,irq2;

    switch(param)
    {
	case VECTOR_INIT:
	    irq1 = irq2 = 0xff;
	    break;

	case RST10_ASSERT:
	    irq1 = 0xd7;
	    break;

	case RST10_CLEAR:
	    irq1 = 0xff;
	    break;

	case RST18_ASSERT:
	    irq2 = 0xdf;
	    break;

	case RST18_CLEAR:
	    irq2 = 0xff;
	    break;
    }

    cpu_irq_line_vector_w(sound_cpu,0,irq1 & irq2);
    if ((irq1 & irq2) == 0xff)      /* no IRQs pending */
	cpu_set_irq_line(sound_cpu,0,CLEAR_LINE);
    else    /* IRQ pending */
	cpu_set_irq_line(sound_cpu,0,ASSERT_LINE);
}



void seibu_ym3812_irqhandler(int linestate)
{
    if (linestate)
	timer_set(TIME_NOW,RST10_ASSERT,setvector_callback);
    else
	timer_set(TIME_NOW,RST10_CLEAR,setvector_callback);
}


static WRITE_HANDLER( track_reset_w )
{
    int i;

    for (i = 0;i < 4;i++)
    last[i] = readinputport(3+i);
}

static READ_HANDLER( track_r )
{
    switch (offset>>1)
    {
	default:
	case 0: return ((readinputport(3) - last[0]) & 0x00ff) | ((readinputport(5) - last[2]) << 8);           /* X lo */
	case 1: return ((readinputport(3) - last[0]) >> 8)     | ((readinputport(5) - last[2]) & 0xff00);       /* X hi */
	case 2: return ((readinputport(4) - last[1]) & 0x00ff) | ((readinputport(6) - last[3]) << 8);           /* Y lo */
	case 3: return ((readinputport(4) - last[1]) >> 8)     | ((readinputport(6) - last[3]) & 0xff00);       /* Y hi */
    }
}


WRITE_HANDLER( cabal_sound_irq_trigger_word_w )
{
	seibu_main_word_w(8,data);

	/* spin for a while to let the Z80 read the command, otherwise coins "stick" */
	cpu_spinuntil_time(TIME_IN_USEC(50));
}

/* Use this if the sound cpu is cpu 1 */
void seibu_sound_init_1(void)
{
	sound_cpu=1;
	setvector_callback(VECTOR_INIT);
}

READ_HANDLER( seibu_soundlatch_r )
{
	return main2sub[offset];
}

WRITE_HANDLER( seibu_main_data_w )
{
	sub2main[offset] = data;
}

WRITE_HANDLER( seibu_coin_w )
{
	coin_counter_w(0,data & 1);
	coin_counter_w(1,data & 2);
}

READ_HANDLER( seibu_main_data_pending_r )
{
	return sub2main_pending ? 1 : 0;
}

WRITE_HANDLER( seibu_irq_clear_w )
{
	setvector_callback(VECTOR_INIT);
}

WRITE_HANDLER( seibu_rst10_ack_w )
{
    /* Unused for now */
}

WRITE_HANDLER( seibu_rst18_ack_w )
{
    timer_set(TIME_NOW,RST18_CLEAR,setvector_callback);
}


READ_HANDLER( seibu_main_word_r )
{
    switch (offset)
    {
    case 4:
    case 6:
	return sub2main[(offset/2)-2];
    case 10:
	return main2sub_pending ? 1 : 0;
    default:
	return 0xffff;
    }
}

#define ACCESSING_LSB (!(data & 0x00ff0000))
WRITE_HANDLER( seibu_main_word_w )
{
    if (ACCESSING_LSB)
    {
	switch (offset)
	{
	    case 0:
	    case 2:
		main2sub[offset/2] = data;
		break;
	    case 8:
		timer_set(TIME_NOW,RST18_ASSERT,setvector_callback);
		break;
	    case 12:
		/* just a guess */
		sub2main_pending = 0;
		main2sub_pending = 1;
		break;
	}
    }
}


WRITE_HANDLER( seibu_adpcm_adr_1_w )
{
    if (offset)
    {
	end = data<<8;
    }
    else
    {
	start = data<<8;
    }
}

WRITE_HANDLER( seibu_adpcm_ctl_1_w )
{
    switch (data)
    {
	case 0:
	    ADPCM_stop(0);
	    break;
	case 2:
	    break;
	case 1:
	    ADPCM_play(0, start, end-start);
	    break;
    }
}

WRITE_HANDLER( seibu_adpcm_adr_2_w )
{
    if (offset)
    {
	end1 = data<<8;
    }
    else
    {
	start1 = data<<8;
    }
}

WRITE_HANDLER( seibu_adpcm_ctl_2_w )
{
    switch (data)
    {
	case 0:
	    ADPCM_stop(1);
	    break;
	case 2:
	    break;
	case 1:
	    start1 += 0x10000;
	    end1 += 0x10000;
	    ADPCM_play(1, start1, end1-start1);
	    break;
    }
}

#define BIT(data,bit) (((data)>>(bit))&1)
#define BITSWAP8(val,B7,B6,B5,B4,B3,B2,B1,B0) \
	       ((BIT(val,B7) << 7) | \
		(BIT(val,B6) << 6) | \
		(BIT(val,B5) << 5) | \
		(BIT(val,B4) << 4) | \
		(BIT(val,B3) << 3) | \
		(BIT(val,B2) << 2) | \
		(BIT(val,B1) << 1) | \
		(BIT(val,B0) << 0))


static UINT8 decrypt_data(int a,int src)
{
    if ( BIT(a,9)  &  BIT(a,8))             src ^= 0x80;
    if ( BIT(a,11) &  BIT(a,4) &  BIT(a,1)) src ^= 0x40;
    if ( BIT(a,11) & ~BIT(a,8) &  BIT(a,1)) src ^= 0x04;
    if ( BIT(a,13) & ~BIT(a,6) &  BIT(a,4)) src ^= 0x02;
    if (~BIT(a,11) &  BIT(a,9) &  BIT(a,2)) src ^= 0x01;

    if (BIT(a,13) &  BIT(a,4)) src = BITSWAP8(src,7,6,5,4,3,2,0,1);
    if (BIT(a, 8) &  BIT(a,4)) src = BITSWAP8(src,7,6,5,4,2,3,1,0);

    return src;
}

static UINT8 decrypt_opcode(int a,int src)
{
    if ( BIT(a,9)  &  BIT(a,8))             src ^= 0x80;
    if ( BIT(a,11) &  BIT(a,4) &  BIT(a,1)) src ^= 0x40;
    if (~BIT(a,13) & BIT(a,12))             src ^= 0x20;
    if (~BIT(a,6)  &  BIT(a,1))             src ^= 0x10;
    if (~BIT(a,12) &  BIT(a,2))             src ^= 0x08;
    if ( BIT(a,11) & ~BIT(a,8) &  BIT(a,1)) src ^= 0x04;
    if ( BIT(a,13) & ~BIT(a,6) &  BIT(a,4)) src ^= 0x02;
    if (~BIT(a,11) &  BIT(a,9) &  BIT(a,2)) src ^= 0x01;

    if (BIT(a,13) &  BIT(a,4)) src = BITSWAP8(src,7,6,5,4,3,2,0,1);
    if (BIT(a, 8) &  BIT(a,4)) src = BITSWAP8(src,7,6,5,4,2,3,1,0);
    if (BIT(a,12) &  BIT(a,9)) src = BITSWAP8(src,7,6,4,5,3,2,1,0);
    if (BIT(a,11) & ~BIT(a,6)) src = BITSWAP8(src,6,7,5,4,3,2,1,0);

    return src;
}


void seibu_sound_decrypt(int cpu_region,int length)
{
    UINT8 *rom = memory_region(cpu_region);
    int diff =  memory_region_length(cpu_region)/2;
    int i;

    memory_set_opcode_base(cpu_region-REGION_CPU1,rom+diff);

    for (i = 0;i < length;i++)
    {
	UINT8 src = rom[i];

	rom[i]      = decrypt_data(i,src);
	rom[i+diff] = decrypt_opcode(i,src);
    }
}

void seibu_adpcm_decrypt(int region)
{
    data8_t *ROM = memory_region(region);
    int i;

    for (i = 0; i < memory_region_length(region); i++)
    {
	ROM[i] = BITSWAP8(ROM[i], 7, 5, 3, 1, 6, 4, 2, 0);
    }
}
