#include "driver.h"
#include "vidhrdw/generic.h"
#include "vidhrdw/taitoic.h"

/* all these games require only 1 chip */
#define TC0100SCN_GFX_NUM 1
/* actual spriteram size varies per game however they all only use a portion */
#define PC090OJ_ACTIVE_RAM_SIZE 0x800

static UINT16 PC090OJ_sprite_ctrl = 0, PC090OJ_ctrl = 0;
static data16_t *buffered_spriteram16;
static UINT16 *spriteptr;


/**********************************************************/

void cadash_vh_stop (void)
{
	free(buffered_spriteram16);
	buffered_spriteram16 = 0;

	//free(spriteram);
	spriteram = 0;

	TC0100SCN_vh_stop();

	TC0110PCR_vh_stop();
}


static int core_vh_start (int xoffset, int use_sprite_buffer)
{
	/* Make room before allocating space for the large tilemaps */
	free_disposable_regions();

	buffered_spriteram16 = malloc(spriteram_size);
	//spriteram = malloc(spriteram_size);
	//if (!buffered_spriteram16 || !spriteram)
	if (!buffered_spriteram16)
		return 1;

	/* clear it all out */
	memset(buffered_spriteram16, 0, spriteram_size);
	memset(spriteram, 0, spriteram_size);

	if (TC0100SCN_vh_start(1,TC0100SCN_GFX_NUM,xoffset))
	{
		cadash_vh_stop();
		return 1;
	}

	if (TC0110PCR_vh_start())
	{
		cadash_vh_stop();
		return 1;
	}

	if (use_sprite_buffer)
	    spriteptr = buffered_spriteram16;
	else
	    spriteptr = (UINT16*)spriteram;

	return 0;
}

int cadash_vh_start (void)
{
	return core_vh_start(1,1);
}

int bonze_vh_start (void)
{
	return core_vh_start(0,0);
}

int asuka_vh_start (void)
{
	return core_vh_start(0,0);
}

int galmedes_vh_start (void)
{
	return core_vh_start(1,0);
}

/********************************************************
          SPRITE READ AND WRITE HANDLERS
********************************************************/

WRITE_HANDLER( cadash_spritectrl_w )
{
	PC090OJ_sprite_ctrl = ((data & 0x3c) >> 2) | ((data & 0x1) << 15);
}


WRITE_HANDLER( cadash_spriteram_w )
{
	COMBINE_WORD_MEM(&spriteram[offset],data);
}

READ_HANDLER( cadash_spriteram_r )
{
	return READ_WORD (&spriteram[offset]);
}

WRITE_HANDLER( asuka_spriteram_w )
{
	COMBINE_WORD_MEM(&spriteram[offset],data);
}

WRITE_HANDLER( asuka_spriteflip_w )
{
	/* Bit 0 is flip control, others seem unused */
	PC090OJ_ctrl = data & 0x01;
}


/*********************************************************
				PALETTE
*********************************************************/

void cadash_update_palette (void)
{
	int i,j,offs;
	UINT32 color, sprite_colbank = (PC090OJ_sprite_ctrl & 0xf) << 4;
	UINT16 data, tilenum, palette_map[256];

	memset (palette_map, 0, sizeof (palette_map));


	for (offs = (PC090OJ_ACTIVE_RAM_SIZE/2)-4;offs >=0;offs -= 4)
	{
		data = spriteptr[offs];
		color = (data &0x000f) | sprite_colbank;

		data = spriteptr[offs+2];
		tilenum = data &0x1fff;

		if (tilenum)
		{
			palette_map[color] |= Machine->gfx[0]->pen_usage[tilenum];
		}
	}

	/* Tell MAME about the color usage */
	for (i = 0;i < 256;i++)
	{
		int usage = palette_map[i];

		if (usage)
		{
			if (palette_map[i] & (1 << 0))
				palette_used_colors[i<<4] = PALETTE_COLOR_USED;
			for (j = 1; j < 16; j++)
				if (palette_map[i] & (1 << j))
					palette_used_colors[(i<<4) + j] = PALETTE_COLOR_USED;
		}
	}
}




/************************************************************
			SPRITE DRAW ROUTINE

		(information from Raine)

		OBJECT RAM
		----------

		- 8 bytes/sprite
		- 256 sprites (0x800 bytes)
		- First sprite has *highest* priority

		-----+--------+-------------------------
		Byte | Bit(s) | Use
		-----+76543210+-------------------------
		  0  |.x......| Flip Y Axis
		  0  |x.......| Flip X Axis
		  1  |....xxxx| Colour Bank
		  2  |.......x| Sprite Y
		  3  |xxxxxxxx| Sprite Y
		  4  |...xxxxx| Sprite Tile
		  5  |xxxxxxxx| Sprite Tile
		  6  |.......x| Sprite X
		  7  |xxxxxxxx| Sprite X
		-----+--------+-------------------------

		SPRITE CONTROL
		--------------

		-----+--------+-------------------------
		Byte | Bit(s) | Use
		-----+76543210+-------------------------
		  0  |.......x| ?
		  0  |......x.| Write Acknowledge?
		  0  |..xxxx..| Colour Bank Offset
		  0  |xx......| Unused
		  1  |...xxxxx| Unused
		  1  |..x.....| BG1:Sprite Priority
		  1  |.x......| Priority?
		  1  |x.......| Unused
		-----+--------+-------------------------

********************************************************/


/* This should be in taitoic.c but this is the only driver that uses PC090OJ_draw_sprites()  */
static void PC090OJ_draw_sprites(struct osd_bitmap *bitmap, int pri_type)
{
	int offs, priority = 0;
	int sprite_colbank = (PC090OJ_sprite_ctrl & 0xf) << 4;

	switch (pri_type)
	{
	    case 0x00:
		priority = 0;   /* sprites over top bg layer */
		break;

	    case 0x01:
		priority = 1;   /* sprites under top bg layer */
		break;

	    case 0x02:
		priority = PC090OJ_sprite_ctrl >> 15;   /* variable sprite/tile priority */
		break;
	}


	for (offs = 0;offs < PC090OJ_ACTIVE_RAM_SIZE/2;offs += 4)
	{
	    int flipx, flipy;
	    int x, y;
	    int data,code,color;

	    code = spriteptr[offs+2] & 0x1fff;
	    if (!code) continue;

	    data = spriteptr[offs];
	    flipy = (data & 0x8000) >> 15;
	    flipx = (data & 0x4000) >> 14;
	    color = (data & 0x000f) | sprite_colbank;

	    x = spriteptr[offs+3] & 0x1ff;   /* mask verified with Rainbowe board */
	    y = spriteptr[offs+1] & 0x1ff;   /* mask verified with Rainbowe board */

	    /* treat coords as signed */
	    if (x>0x140) x -= 0x200;
	    if (y>0x140) y -= 0x200;
#if 0
	    if (PC090OJ_ctrl & 1)        /* sprites flipscreen */
	    {
		x = 320 - x - 16;
		y = 256 - y - 16;
		flipx = !flipx;
		flipy = !flipy;
	    }
#endif

	    //y += PC090OJ_yoffs;
	    y += 8;

	    pdrawgfx(
		    bitmap,Machine->gfx[0],
		    code,
		    color,
		    flipx,flipy,
		    x,y,
		    &Machine->visible_area,TRANSPARENCY_PEN,0,
		    priority ? 0xfc : 0xf0);
	}
}



/**************************************************************
				SCREEN REFRESH
**************************************************************/

void cadash_vh_eof_callback(void)
{
	// Only copy the active portion of sprite ram
	memcpy32((int*)buffered_spriteram16,(int*)spriteram,PC090OJ_ACTIVE_RAM_SIZE);
}

void asuka_vh_eof_callback(void)
{
	//static int myref = 0;
	//debug_printf("screenrefresh %d\n", myref++);
}

void cadash_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh)
{
	UINT32 layer[3];

	TC0100SCN_tilemap_update();

	palette_init_used_colors();
	cadash_update_palette();
	palette_used_colors[0] |= PALETTE_COLOR_VISIBLE;
	if (palette_recalc())
		tilemap_mark_all_pixels_dirty(ALL_TILEMAPS);

	// Get bottom layer
	layer[0] = TC0100SCN_bottomlayer(0);
	layer[1] = layer[0]^1;
	layer[2] = 2;

	TC0100SCN_tilemap_render(1);

	fillbitmap(priority_bitmap,0,NULL);

	if (TC0100SCN_is_layer_disabled(0, layer[0]))
	{
	    /* Ensure screen blanked when bottom layer not drawn due to disable bit */
	    fillbitmap(bitmap, palette_transparent_pen, &Machine -> visible_area);
	}
	else
	{
	    TC0100SCN_tilemap_draw(bitmap,0,layer[0],TILEMAP_IGNORE_TRANSPARENCY|1<<16);    //background
	}

	TC0100SCN_tilemap_draw(bitmap,0,layer[1],2<<16);				// foreground
	TC0100SCN_tilemap_draw(bitmap,0,layer[2],4<<16);		// In RAM generated character set, always on top

	PC090OJ_draw_sprites(bitmap,0);

}


void asuka_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh)
{
	UINT32 layer[2];

	TC0100SCN_tilemap_notxt_update();

	palette_init_used_colors();

	cadash_update_palette();
	palette_used_colors[0] |= PALETTE_COLOR_VISIBLE;
	if (palette_recalc())
		tilemap_mark_all_pixels_dirty(ALL_TILEMAPS);

	// Get bottom layer_
	layer[0] = TC0100SCN_bottomlayer(0);
	layer[1] = layer[0]^1;

	TC0100SCN_tilemap_render(0);

	if (TC0100SCN_is_layer_disabled(0, layer[0]))
	{
	    /* Ensure screen blanked when bottom layer not drawn due to disable bit */
	    fillbitmap(bitmap, palette_transparent_pen, &Machine -> visible_area);
	}
	else
	{
	    TC0100SCN_tilemap_draw(bitmap,0,layer[0],TILEMAP_IGNORE_TRANSPARENCY|1<<16);    //background
	}

	TC0100SCN_tilemap_draw(bitmap,0,layer[1],2<<16);				// foreground

	PC090OJ_draw_sprites(bitmap,2);

	/* The text foreground tilemap is not used */
}

void mofflott_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh)
{
	UINT32 layer[3];

	TC0100SCN_tilemap_update();

	palette_init_used_colors();
	cadash_update_palette();
	palette_used_colors[0] |= PALETTE_COLOR_VISIBLE;
	if (palette_recalc())
		tilemap_mark_all_pixels_dirty(ALL_TILEMAPS);

	// Get bottom layer
	layer[0] = TC0100SCN_bottomlayer(0);
	layer[1] = layer[0]^1;
	layer[2] = 2;

	TC0100SCN_tilemap_render(1);

	// The priorities in Cadash seem to really screw up rendering, no tilemaps get drawn
	// Remove priority and reordering the sprites/txt tilemap seems to have done the same thing
	// Slaanesh
	
	fillbitmap(priority_bitmap,0,NULL);

	if (TC0100SCN_is_layer_disabled(0, layer[0]))
	{
	    /* Ensure screen blanked when bottom layer not drawn due to disable bit */
	    fillbitmap(bitmap, palette_transparent_pen, &Machine -> visible_area);
	}
	else
	{
	    TC0100SCN_tilemap_draw(bitmap,0,layer[0],TILEMAP_IGNORE_TRANSPARENCY|1<<16);    //background
	}


	TC0100SCN_tilemap_draw(bitmap,0,layer[1],2<<16);				// foreground
	TC0100SCN_tilemap_draw(bitmap,0,layer[2],4<<16);		// In RAM generated character set, always on top

	PC090OJ_draw_sprites(bitmap,2);

}
