/***************************************************************************

  vidhrdw/jack.c

  Functions to emulate the video hardware of the machine.

***************************************************************************/

#include "driver.h"
#include "vidhrdw/generic.h"

static struct tilemap *bg_tilemap;
unsigned char *joinem_scrollram;
int joinem_palette_bank;


void joinem_vh_convert_color_prom(unsigned char *palette, unsigned short *colortable,const unsigned char *color_prom)
{
    int i;

    for (i = 0; i < Machine->drv->total_colors; i++)
    {
	int bit0, bit1, bit2;
	int r, g, b;

	bit0 = (color_prom[i] >> 0) & 0x01;
	bit1 = (color_prom[i] >> 1) & 0x01;
	bit2 = (color_prom[i] >> 2) & 0x01;
	*(palette++) = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
	r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
	bit0 = (color_prom[i] >> 3) & 0x01;
	bit1 = (color_prom[i] >> 4) & 0x01;
	bit2 = (color_prom[i] >> 5) & 0x01;
	*(palette++) = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
	g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
	bit0 = 0;
	bit1 = (color_prom[i] >> 6) & 0x01;
	bit2 = (color_prom[i] >> 7) & 0x01;
	*(palette++) = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
	b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
    }
}


WRITE_HANDLER( joinem_scroll_w )
{
    switch (offset & 3)
    {
	// byte 0: column scroll
	case 0:
	    //m_bg_tilemap->set_scrolly(offset >> 2, -data);
	    tilemap_set_scrolly(bg_tilemap, offset >> 2, -data);
	    break;

	// byte 1/2/3: no effect?
	default:
	    break;
    }

    joinem_scrollram[offset] = data;
}

WRITE_HANDLER( unclepoo_videoram_w )
{
	videoram[offset] = data;
	tilemap_mark_tile_dirty(bg_tilemap, offset);
}

WRITE_HANDLER( unclepoo_colorram_w )
{
	colorram[offset] = data;
	tilemap_mark_tile_dirty(bg_tilemap, offset);
}


WRITE_HANDLER( jack_paletteram_w )
{
	/* RGB output is inverted */
	paletteram_BBGGGRRR_w(offset,~data);
}


READ_HANDLER( jack_flipscreen_r )
{
	flip_screen_w(0, offset);
	return 0;
}

WRITE_HANDLER( jack_flipscreen_w )
{
	flip_screen_w(0, offset);
}


void jack_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh)
{
	int offs;


	if (palette_recalc() || full_refresh)
		memset(dirtybuffer,1,videoram_size);

	/* for every character in the Video RAM, check if it has been modified */
	/* since last time and update it accordingly. */
	for (offs = videoram_size - 1;offs >= 0;offs--)
	{
		if (dirtybuffer[offs])
		{
			int sx,sy;


			dirtybuffer[offs] = 0;

			sx = offs / 32;
			sy = 31 - offs % 32;

			if (flip_screen)
			{
				sx = 31 - sx;
				sy = 31 - sy;
			}

			drawgfx(tmpbitmap,Machine->gfx[0],
					videoram[offs] + ((colorram[offs] & 0x18) << 5),
					colorram[offs] & 0x07,
					flip_screen,flip_screen,
					8*sx,8*sy,
					&Machine->visible_area,TRANSPARENCY_NONE,0);
		}
	}


	/* copy the temporary bitmap to the screen */
	copybitmap(bitmap,tmpbitmap,0,0,0,0,&Machine->visible_area,TRANSPARENCY_NONE,0);

	/* draw sprites */
	for (offs = spriteram_size - 4;offs >= 0;offs -= 4)
	{
		int sx,sy,num, color,flipx,flipy;

		sx    = spriteram[offs + 1];
		sy    = spriteram[offs];
		num   = spriteram[offs + 2] + ((spriteram[offs + 3] & 0x08) << 5);
		color = spriteram[offs + 3] & 0x07;
		flipx = (spriteram[offs + 3] & 0x80);
		flipy = (spriteram[offs + 3] & 0x40);

		if (flip_screen)
		{
			sx = 248 - sx;
			sy = 248 - sy;
			flipx = !flipx;
			flipy = !flipy;
		}

		drawgfx(bitmap,Machine->gfx[0],
				num,
				color,
				flipx,flipy,
				sx,sy,
				&Machine->visible_area,TRANSPARENCY_PEN,0);
	}
}



static void joinem_get_bg_tile_info (int tile_index)
{
    int code = videoram[tile_index] + ((colorram[tile_index] & 0x03) << 8);
    int color = (colorram[tile_index] & 0x38) >> 3 | joinem_palette_bank;

    SET_TILE_INFO(0, code, color);
}

static unsigned int tilemap_scan_cols_flipy(unsigned int col,unsigned int row,unsigned int num_cols,unsigned int num_rows)
{
    /* logical (col,row) -> memory offset */
    return (col * num_rows) + (num_rows - 1 - row);
}


static void joinem_draw_sprites( struct osd_bitmap *bitmap )
{
    int offs;

    for (offs = spriteram_size - 4;offs >= 0;offs -= 4)
    {
	int sy = spriteram[offs];
	int sx = spriteram[offs + 1];
	int code = spriteram[offs + 2] | ((spriteram[offs + 3] & 0x03) << 8);
	int color = (spriteram[offs + 3] & 0x38) >> 3 | joinem_palette_bank;
	int flipx = (spriteram[offs + 3] & 0x80) >> 7;
	int flipy = (spriteram[offs + 3] & 0x40) >> 6;

	if (flip_screen)
	{
	    sx = 248 - sx;
	    sy = 248 - sy;
	    flipx = !flipx;
	    flipy = !flipy;
	}

	drawgfx(bitmap,Machine->gfx[0],
			code,
			color,
			flipx,flipy,
			sx,sy,
			&Machine->visible_area,TRANSPARENCY_PEN,0);
    }
}


int unclepoo_vh_start ( void )
{
    bg_tilemap = tilemap_create(joinem_get_bg_tile_info, tilemap_scan_cols_flipy, TILEMAP_OPAQUE, 8, 8, 32, 32);
    if (!bg_tilemap)
	return 1;
    tilemap_set_scroll_cols(bg_tilemap, 32);

    if (generic_vh_start())
	return 1;
    else
	return 0;
}

void unclepoo_vh_stop ( Void )
{
    generic_vh_stop();
}

void unclepoo_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh)
{
    tilemap_update(ALL_TILEMAPS);
    tilemap_render(ALL_TILEMAPS);
    tilemap_draw(bitmap, bg_tilemap, 0);
    joinem_draw_sprites(bitmap);
}
