/***************************************************************************

						-= Jaleco Mega System 1 =-

				driver by	Luca Elia (eliavit@unina.it)


Note:	if MAME_DEBUG is defined, pressing Z with:

		Q		shows scroll 1
		W		shows scroll 2
		E		shows scroll 3
		A		shows sprites with attribute 0-3
		S		shows sprites with attribute 4-7
		D		shows sprites with attribute 8-b
		F		shows sprites with attribute c-f

		Keys can be used togheter!


**********  There are 3 scrolling layers, 2 bytes per tile info:

* Note: MS1-Z has 2 layers only.

  A page is 256x256, approximately the visible screen size. Each layer is
  made up of 8 pages (8x8 tiles) or 32 pages (16x16 tiles). The number of
  horizontal  pages and the tiles size  is selectable, using the  layer's
  control register. I think that when tiles are 16x16 a layer can be made
  of 16x2, 8x4, 4x8 or 2x16 pages (see below). When tile size is 8x8 we
  have two examples to guide the choice:

  the copyright screen of p47j (0x12) should be 4x2 (unless it's been hacked :)
  the ending sequence of 64th street (0x13) should be 2x4.

  I don't see a relation.


MS1-A MS1-B MS1-C
-----------------

					Scrolling layers:

90000 50000 e0000	Scroll 1
94000 54000 e8000	Scroll 2
98000 58000 f0000	Scroll 3					* Note: missing on MS1-Z

Tile format:	fedc------------	Palette
				----ba9876543210	Tile Number



84000 44000 c2208	Layers Enable				* Note: missing on MS1-Z?

	fedc ---- ---- ---- ? (unused?)
	---- ba-- ---- ----	? (used!)
	---- --9- ---- ----	Sprites below fg (sprites over fg on MS1-C?)
	---- ---8 ---- ----	Swap txt with fg (swap bg with fg on MS1-C?)
	---- ---- 7654 ----	? (unused?)
	---- ---- ---- 3---	Enable Sprites
	---- ---- ---- -210	Enable Layer 321



84200 44200 c2000	Scroll 1 Control
84208 44208 c2008	Scroll 2 Control
84008 44008 c2100	Scroll 3 Control		* Note: missing on MS1-Z

Offset:		00					   Scroll X
			02					   Scroll Y
			04 fedc ba98 765- ---- ? (unused?)
			   ---- ---- ---4 ---- 16x16 Tiles
			   ---- ---- ---- 32-- ? (used, by p47!)
			   ---- ---- ---- --10 N: Layer H pages = 16 / (2^N)



84300 44300 c2308	Screen Control

	fedc ba9- ---- ---- ? (unused?)
	---- ---8 ---- ---- Portrait F/F (?FullFill?)
	---- ---- 765- ---- ? (unused?)
	---- ---- ---4 ---- ? (used, see p47j copyright screen!)
	---- ---- ---- 321- ? (unused?)
	---- ---- ---- ---0	Screen V Flip



**********  There are 256*4 colors (256*3 for MS1-Z):

Colors		MS1-A/C			MS1-Z

000-0ff		Scroll 1		Scroll 1
100-1ff		Scroll 2		Sprites
200-2ff		Scroll 3		Scroll 2
300-3ff		Sprites			-

88000 48000 f8000	Palette

	fedc--------3---	Red
	----ba98-----2--	Blue
	--------7654--1-	Green
	---------------0	? (used, not RGB! [not changed in fades])


**********  There are 256 sprites (128 for MS1-Z):

RAM[8000]	Sprite Data	(16 bytes/entry. 128? entries)

Offset:		0-6						? (used, but as normal RAM, I think)
			08 	fed- ---- ---- ----	?
				---c ---- ---- ----	mosaic sol.	(?)
				---- ba98 ---- ----	mosaic		(?)
				---- ---- 7--- ----	y flip
				---- ---- -6-- ----	x flip
				---- ---- --45 ----	?
				---- ---- ---- 3210	color code (* bit 3 = priority *)
			0A						H position
			0C						V position
			0E	fedc ---- ---- ----	? (used by p47j, 0-8!)
				---- ba98 7654 3210	Number



Object RAM tells the hw how to use Sprite Data (missing on MS1-Z).
This makes it possible to group multiple small sprite, up to 256,
into one big virtual sprite (up to a whole screen):

8e000 4e000 d2000	Object RAM (8 bytes/entry. 256*4 entries)

Offset:		00	Index into Sprite Data RAM
			02	H	Displacement
			04	V	Displacement
			06	Number	Displacement

Only one of these four 256 entries is used to see if the sprite is to be
displayed, according to this latter's flipx/y state:

Object RAM entries:		Useb by sprites with:

000-0ff					No Flip
100-1ff					Flip X
200-2ff					Flip Y
300-3ff					Flip X & Y




No? No? c2108	Sprite Bank

	fedc ba98 7654 321- ? (unused?)
	---- ---- ---- ---0 Sprite Bank



84100 44100 c2200 Sprite Control

			fedc ba9- ---- ---- ? (unused?)
			---- ---8 ---- ---- Enable sprite priority effect ? (see p47j sprite test 1!)
			---- ---- 765- ---- ? (unused?)
			---- ---- ---4 ----	Enable Effect (?)
			---- ---- ---- 3210	Effect Number (?)

I think bit 4 enables some sort of color cycling for sprites having priority
bit set. See code of p7j at 6488,  affecting the rotating sprites before the
Jaleco logo is shown: values 11-1f, then 0. I fear the Effect Number is an
offset to be applied over the pens used by those sprites. As for bit 8, it's
not used during game, but it is turned on when sprite/foreground priority is
tested, along with Effect Number being 1, so ...


**********  Priority (ouch!)

Sprite order is from first in Sprite Data RAM (frontmost) to last.
* Note: the opposite on MS1-Z

Foreground isn't splittable in a "back" part using, say, pens 0-7 and a
"front part" using pens 8-15 usually. Sprites aren't splittable either.
But level A of 64th street has fg that is splittable. I think bit 0 of
the color in paletteram has a role also.


Here's how I draw things (surely incomplete/inaccurate):

	MS1-Z/A/B		MS1-C
bg  Scroll RAM 1	Scroll RAM 2
fg  Scroll RAM 2	Scroll RAM 1
txt Scroll RAM 3	Scroll RAM 3

MS1-Z	-
MS1-A	bit 8 of 84000 is on  -> swap fg, txt
MS1-B	bit 8 of 44000 is on  -> swap fg, txt
MS1-C	bit 8 of c2208 is off -> swap bg, fg


bit x is sprite priority enable:			bit 8 of 84100 44100 c2200
bit 9 is a foreground priority control:		bit 9 of 84000 44000 !c2208

					bg
	if bit x
		if !bit 9	fg
					sprites (priority bit 1)
		if  bit 9	fg
					sprites (priority bit 0)
	else
		if !bit 9	fg
					sprites (priority doesn't affect order)
		if  bit 9	fg

					txt (missing on MS1-Z)

***************************************************************************/

#include "vidhrdw/generic.h"
#include "options.h"

#define NBR_LAYERS 3

extern struct GameDriver driver_avspirit;
extern unsigned char *scrollram[NBR_LAYERS],*scrollram_dirty[NBR_LAYERS];		// memory pointers
extern unsigned char *objectram, *videoregs, *ram;
extern int scrollx[NBR_LAYERS],scrolly[NBR_LAYERS],scrollflag[NBR_LAYERS],nx[NBR_LAYERS],ny[NBR_LAYERS];	// video registers
extern int megasys1_active_layers, spritebank, screenflag, spriteflag,hardware_type;
extern int bg, fg, txt;

struct osd_bitmap *scroll_bitmap[NBR_LAYERS];

struct priority
{
	struct GameDriver *driver;
	int priorities[16];
};

int megasys1_layers_order[16];


/* these are for debug purposes */
#ifdef MAME_DEBUG
int debugsprites;
#endif


void mark_dirty(int n)
{
    memset(scrollram_dirty[n],1,256*256/64*8);
}

void mark_dirty_all(void)
{
	mark_dirty(0);
	mark_dirty(1);
	mark_dirty(2);
}

int megasys1_vh_start(void)
{
	int i;
	free_disposable_regions();
    //debug_printf("megasys1_vh_start()\n");

    /* allocate a dirty page for each scrolling layer */

	for (i = 0; i < 3; i++)
	 if ((scrollram_dirty[i] = malloc(256*4*256*2/64))==0) return 1;

    /*  temporary bitmaps are not created here. They are instead
       created on the fly when refreshing the screen. */

	mark_dirty_all();

	for (i = 0; i < NBR_LAYERS; i++)
	{
	    debug_printf("Init scroll_bitmap[%d]\n", i);
	    scroll_bitmap[i] = 0;
	}

	return 0;
}

void megasys1_vh_stop(void)
{
	int i;

	for (i = 0; i < NBR_LAYERS; i++)
	{
	    if (scroll_bitmap[i])
	    {
		osd_free_bitmap(scroll_bitmap[i]);
		debug_printf("Clearing scroll_bitmap[%d]\n", i);
		scroll_bitmap[i] = 0;
	    }
	    if (scrollram_dirty[i])
	    {
		free(scrollram_dirty[i]);
		scrollram_dirty[i] = 0;
	    }
	}
}


/*

 Draw sprites in the given bitmap. Priority may be:

 0	Draw sprites whose priority bit is 0
 1	Draw sprites whose priority bit is 1
-1	Draw sprites regardless of their priority

 Sprite Data:

	Offset		Data

 	00-07						?
	08 		fed- ---- ---- ----	?
			---c ---- ---- ----	mosaic sol.	(?)
			---- ba98 ---- ----	mosaic		(?)
			---- ---- 7--- ----	y flip
			---- ---- -6-- ----	x flip
			---- ---- --45 ----	?
			---- ---- ---- 3210	color code (bit 3 = priority)
	0A		H position
	0C		V position
	0E		Number

*/
static void draw_sprites(struct osd_bitmap *bitmap, int priority)
{
int color,code,sx,sy,attr,sprite,offs;
unsigned char *spritedata, *objectdata;

    //debug_printf("draw_sprites()\n");
/* objram: 0x100*4 entries		spritedata: 0x80? entries */

	/* move the bit to the relevant position (and invert it) */
	if (priority != -1)	priority = (priority ^ 1) << 3;

	/* sprite order is from first in Sprite Data RAM (frontmost) to last */

	if (hardware_type != 'Z')
	{
		for (sprite = 0x80-1; sprite >= 0 ; sprite --)
		{
			spritedata = &spriteram[sprite*0x10];

			attr = READ_WORD(&spritedata[0x08]);

			if ( (attr & 0x08) == priority ) continue;
#ifdef MAME_DEBUG
if ( (debugsprites) && (((attr & 0x0f)/4) != (debugsprites-1)) ) continue;
#endif

			objectdata = &objectram[((attr & 0xc0) >> 6) * 0x800];

			for (offs = 0x000; offs < 0x800 ; offs += 8)
			{
				/* seek a reference to current sprite */
				if ( sprite != READ_WORD(&objectdata[offs+0x00]) ) continue;

				/* apply the position displacements */
				sx = ( READ_WORD(&spritedata[0x0A]) + READ_WORD(&objectdata[offs+0x02]) ) % 512;
				sy = ( READ_WORD(&spritedata[0x0C]) + READ_WORD(&objectdata[offs+0x04]) ) % 512;

				if (sx > 256-1) sx -= 512;
				if (sy > 256-1) sy -= 512;

				/* out of screen */
				if ((sx > 256-1) ||(sy > 256-1) ||(sx < 0-15) ||(sy < 0-15)) continue;

				code  = READ_WORD(&spritedata[0x0E]) + READ_WORD(&objectdata[offs+0x06]);
				color = (attr & 0x0F);

				drawgfx(bitmap,Machine->gfx[3],
					(code & 0xfff ) + (spritebank << 12),
					color,
					attr & 0x40,attr & 0x80,
					sx, sy,
					&Machine->visible_area,
					TRANSPARENCY_PEN,15);
			}	/* offs */
		}	/* sprite */
	}	/* non Z hw */
	else
	{

		/* MS1-Z just draws Sprite Data, and in reverse order */

		for (sprite = 0; sprite < 0x80 ; sprite ++)
		{
			spritedata = &spriteram[sprite*0x10];

			attr = READ_WORD(&spritedata[0x08]);
			if ( (attr & 0x08) == priority ) continue;

			//sx = READ_WORD(&spritedata[0x0A]) % 512;
			//sy = READ_WORD(&spritedata[0x0C]) % 512;
			sx = READ_WORD(&spritedata[0x0A]) & 511;
			sy = READ_WORD(&spritedata[0x0C]) & 511;

			if (sx > 256-1) sx -= 512;
			if (sy > 256-1) sy -= 512;

			/* out of screen */
			if ((sx > 256-1) ||(sy > 256-1) ||(sx < 0-15) ||(sy < 0-15)) continue;

			code  = READ_WORD(&spritedata[0x0E]);
			color = (attr & 0x0F);

			drawgfx(bitmap,Machine->gfx[2],
				code,
				color,
				attr & 0x40,attr & 0x80,
				sx, sy,
				&Machine->visible_area,
				TRANSPARENCY_PEN,15);
		}	/* sprite */
	}	/* Z hw */

}



/***************************************************************************

  Draw the game screen in the given osd_bitmap.

***************************************************************************/

void megasys1_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh)
{
int offs,code,sx,sy,attr,i,j;
unsigned char *current_ram, *current_dirty;
int current_mask;
int mask[3] = {0xfff,0xfff,0xfff};

PEN_USAGE_DATATYPE *pen_usage;
int color_codes_start, color, colmask[16];

    //debug_printf("megasys1_vh_screenrefresh()\n");

    int active_layers1;

	active_layers1 = megasys1_active_layers;

	switch (hardware_type)
	{
		case 'A':
		{
			bg = 0;
			if (megasys1_active_layers & 0x0100)	{fg = 2;	txt = 1;}
			else						{fg = 1;	txt = 2;}
		}	break;

		case 'B':
		{
		    if (Machine->gamedrv != &driver_avspirit)
		    {
			bg = 0;
			if (megasys1_active_layers & 0x0100)     {fg = 2;        txt = 1;}
			else                                            {fg = 1;        txt = 2;}
		    }
		    else
		    {
			txt = 2;
			if (megasys1_active_layers & 0x0100)     {bg = 0;        fg = 1;}
			else                                            {bg = 1;        fg = 0;}
		    }
		}       break;


		case 'C':
		{
		    megasys1_active_layers ^= 0x0200;        /* sprites/fg priority swapped */
		    txt = 2;
		    if (megasys1_active_layers & 0x0100)     {bg = 0;        fg = 1;}
		    else                                            {bg = 1;        fg = 0;}
		}       break;

	}
    int active_layers = megasys1_active_layers;
#if 0
    int active_layers = 0;
    int pri;

    /* get layers order */
    pri = megasys1_layers_order[(megasys1_active_layers & 0x0f0f) >> 8];

    if (pri == 0xfffff) pri = 0x04132;

    /* see what layers are really active (layers 4 & f will do no harm) */
    for (i = 0;i < 5;i++)
    active_layers |= 1 << ((pri >> (4*i)) & 0x0f);

    megasys1_active_layers &= active_layers;
    megasys1_active_layers |= 1 << ((pri & 0xf0000) >> 16); // bottom layer can't be disabled

    //debug_printf("pri=%x megasys1_active_layers=%x\n", pri, megasys1_active_layers);
#endif


/* Palette stuff */

	palette_init_used_colors();

/* We consider whole layers, redraw whole layers: worst case is slow! */

	for (j = 0 ; j < NBR_LAYERS ; j++ )
	{
		/* let's skip disabled layers */
		if (!(active_layers & (1 << j )))	continue;

		pen_usage = Machine->gfx[j]->pen_usage;
		color_codes_start = Machine->drv->gfxdecodeinfo[j].color_codes_start;

		for (color = 0 ; color < 16 ; color++) colmask[color] = 0;

		for (offs  = 0 ; offs < 0x4000 ; offs += 2)
		{
			color = ( READ_WORD(&scrollram[j][offs]) & 0xF000 ) >> 12;
			code  =   READ_WORD(&scrollram[j][offs]) & mask[j];

			if (scrollflag[j] & 0x10)
					colmask[color] |= pen_usage[code % Machine->gfx[j]->total_elements];
			else{	colmask[color] |= pen_usage[(code*4+0)%Machine->gfx[j]->total_elements];
					colmask[color] |= pen_usage[(code*4+1)%Machine->gfx[j]->total_elements];
					colmask[color] |= pen_usage[(code*4+2)%Machine->gfx[j]->total_elements];
					colmask[color] |= pen_usage[(code*4+3)%Machine->gfx[j]->total_elements];}
		}

		for (color = 0; color < 16; color++)
		{
			if (colmask[color])
			{
				for (i = 0; i < 16; i++)
					if (colmask[color] & (1 << i))
						palette_used_colors[16 * color + i + color_codes_start] = PALETTE_COLOR_USED;
			}
		}

		if (j != bg)	/* background colors are all used */
		{
			for (color = 0; color < 16; color++)
				palette_used_colors[16 * color + 15 + color_codes_start] = PALETTE_COLOR_TRANSPARENT;
		}
	}



	/* Sprites */

	for (color = 0 ; color < 16 ; color++) colmask[color] = 0;

	pen_usage = Machine->gfx[3]->pen_usage;
	color_codes_start = Machine->drv->gfxdecodeinfo[3].color_codes_start;

	for (offs = 0; offs < 0x2000 ; offs += 8)
	{
	    int sprite;
	    unsigned char* spritedata;

		sprite = READ_WORD(&objectram[offs+0x00]);
		spritedata = &spriteram[(sprite&0x7F)*16];

		attr = READ_WORD(&spritedata[0x08]);
		if ( (attr & 0xc0) != ((offs/0x800)<<6) ) continue;

		sx = ( READ_WORD(&spritedata[0x0A]) + READ_WORD(&objectram[offs+0x02]) ) % 512;
		sy = ( READ_WORD(&spritedata[0x0C]) + READ_WORD(&objectram[offs+0x04]) ) % 512;

		if (sx > 256-1) sx -= 512;
		if (sy > 256-1) sy -= 512;

		if ((sx > 256-1) ||(sy > 256-1) ||(sx < 0-15) ||(sy < 0-15)) continue;

		code  = READ_WORD(&spritedata[0x0E]) + READ_WORD(&objectram[offs+0x06]);
		code  =	(code & 0xfff ) + (spritebank << 12);
		color = (attr & 0x0F);

		colmask[color] |= pen_usage[code];
	}


	for (color = 0; color < 16; color++)
	 for (i = 0; i < 16; i++)
	  if (colmask[color] & (1 << i)) palette_used_colors[16 * color + i + color_codes_start] = PALETTE_COLOR_USED;

	if (palette_recalc()) mark_dirty_all();


/* Allocate off screen bitmaps */

	for (i = 0; i < NBR_LAYERS ; i++)
	{
		nx[i] = 16 / ( 1 << (scrollflag[i] & 0x3) );
		ny[i] = 32 / nx[i];

		if (scrollflag[i]&0x10)
		{
			if (ny[i] > 4)
			{
				nx[i] /= 1;
				ny[i] /= 4;
			}
			else
			{
				nx[i] /= 2;
				ny[i] /= 2;
			}
		}

		/* let's skip disabled layers */
		if ( (active_layers & (1 << i )) && (scroll_bitmap[i] == 0) )
		{
			debug_printf("scroll_bitmap[%d]=%x\n", i, scroll_bitmap[i]);
			scroll_bitmap[i] = bitmap_alloc_depth(256*nx[i], 256*ny[i], 4); // Needs only a 4-bit depth bitmap
			if (scroll_bitmap[i])
			{
				debug_printf("vidhrdw/megasys1.c: created %dx%d screen[%d]\n",nx[i],ny[i],i);
			}
			else
			{
				debug_printf("vidhrdw/megasys1.c: No mem for %dx%d screen[%d]\n",nx[i],ny[i],i);
				return;	//bad
			}

		}

	}


/* Update the scrolling layers */

	for (i = 0; i < NBR_LAYERS ; i++)
	{
		/* let's skip disabled layers */
		if (!(active_layers & (1 << i )))	continue;

		current_ram   = scrollram[i];
		current_dirty = scrollram_dirty[i];
		current_mask  = mask[i];

		if (scrollflag[i] & 0x10)	/* 8x8 tiles */
		{
		 for (j=0 ; j < nx[i]*ny[i] ; j++)
		  for (sx = (j%nx[i])*256; sx < (j%nx[i])*256+256; sx += 8)
		   for (sy = (j/nx[i])*256; sy < (j/nx[i])*256+256; sy += 8)
		   {
			if (current_dirty[0])
			{
				current_dirty[0] = 0;
				code = READ_WORD(&current_ram[0]);
				drawgfx(scroll_bitmap[i],Machine->gfx[i],
					code & current_mask,
					(code & 0xF000)>>12,
					0,0,
					sx,sy,
					0,TRANSPARENCY_NONE,0);
			};
			current_dirty++;	current_ram+=2;
		   }
		}
		else	/* 16x16 tiles */
		{
		 for (j=0 ; j < nx[i]*ny[i] ; j++)
		  for (sx = (j%nx[i])*256; sx < (j%nx[i])*256+256; sx += 16)
		   for (sy = (j/nx[i])*256; sy < (j/nx[i])*256+256; sy += 16)
		   {
			if (current_dirty[0])
			{
				current_dirty[0] = 0;
				code = READ_WORD(&current_ram[0]);
				attr = (code >> 12) & 0x0F;
				code = (code & current_mask) * 4;
				drawgfx(scroll_bitmap[i],Machine->gfx[i],
					code+0,
					attr,
					0,0,sx+0,sy+0,0,TRANSPARENCY_NONE,0);
				drawgfx(scroll_bitmap[i],Machine->gfx[i],
					code+1,
					attr,
					0,0,sx+0,sy+8,0,TRANSPARENCY_NONE,0);
				drawgfx(scroll_bitmap[i],Machine->gfx[i],
					code+2,
					attr,
					0,0,sx+8,sy+0,0,TRANSPARENCY_NONE,0);
				drawgfx(scroll_bitmap[i],Machine->gfx[i],
					code+3,
					attr,
					0,0,sx+8,sy+8,0,TRANSPARENCY_NONE,0);
			};
			current_dirty++;	current_ram+=2;
		   }
		} /* 16x16 tiles */
	} /* i */

/* copy each (enabled) layer */


#define copylayer(_n_,_transparency_,_pen_)\
	if (active_layers & (1 << _n_ ))\
	{\
		copyscrollbitmap(bitmap, scroll_bitmap[_n_],\
				1,&scrollx[_n_],1,&scrolly[_n_],\
				&Machine->visible_area,\
				_transparency_,_pen_);\
	}


	/* Copy the background */
	copylayer(bg,TRANSPARENCY_NONE,0)
	else	osd_clearbitmap(Machine->scrbitmap);

	/* If priority effect is active .. */
	if (spriteflag & 0x0100)
	{
		/* Copy the foreground below ... */
		if (!(active_layers &0x0200))	copylayer(fg,TRANSPARENCY_PEN,palette_transparent_pen);

		/* draw sprites with color >= 8 */
		if (active_layers & 0x08)	draw_sprites(bitmap,1);

		/* ... or over sprites with color >= 8 */
		if (active_layers &0x0200)	copylayer(fg,TRANSPARENCY_PEN,palette_transparent_pen);

		/* Draw sprites with color 0-7 */
		if (active_layers & 0x08)	draw_sprites(bitmap,0);
	}
	else
	{
		/* Copy the foreground below ... */
		if (!(active_layers &0x0200))	copylayer(fg,TRANSPARENCY_PEN,palette_transparent_pen);

		/* Draw sprites regardless of their color */
		if (active_layers & 0x08)	draw_sprites(bitmap,-1);

		/* ... or over sprites */
		if (active_layers &0x0200)	copylayer(fg,TRANSPARENCY_PEN,palette_transparent_pen);
	}

	/* Copy the text layer */
	copylayer(txt,TRANSPARENCY_PEN,palette_transparent_pen);

	megasys1_active_layers = active_layers1;
}



/***************************************************************************
						Convert the Priority Prom
***************************************************************************/


extern struct GameDriver driver_64street;
extern struct GameDriver driver_chimerab;
extern struct GameDriver driver_cybattlr;
extern struct GameDriver driver_bigstrik;
extern struct GameDriver driver_iganinju;
extern struct GameDriver driver_tshingen;
#if 0
extern struct GameDriver driver_astyanax;
extern struct GameDriver driver_hachoo;
extern struct GameDriver driver_kickoff;
extern struct GameDriver driver_soldamj;
#endif

/*
	Layers order encoded as an int like: 0x01234, where

	0:	Scroll 0
	1:	Scroll 1
	2:	Scroll 2
	3:	Sprites with color 0-7
		(*every sprite*, if sprite splitting is not active)
	4:	Sprites with color 8-f
	f:	empty placeholder (we can't use 0!)

	and the bottom layer is on the left (e.g. 0).

	The special value 0xfffff means that the order is either unknown
	or no simple stack of layers can account for the values in the prom.
	(the default value, 0x04132, will be used in those cases)

*/

static struct priority priorities[] =
{
	{	&driver_64street,
		{ 0xfffff,0x03142,0xfffff,0x04132,0xfffff,0x04132,0xfffff,0xfffff,
		  0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
	{	&driver_chimerab,
		{ 0x14032,0x04132,0x14032,0x04132,0xfffff,0xfffff,0xfffff,0xfffff,
		  0xfffff,0xfffff,0x01324,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
	{	&driver_cybattlr,
		{ 0x04132,0xfffff,0xfffff,0xfffff,0x14032,0xfffff,0xfffff,0xfffff,
		  0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0x04132 }
	},
	{	&driver_bigstrik,	/* like 64street ? */
		{ 0xfffff,0x03142,0xfffff,0x04132,0xfffff,0x04132,0xfffff,0xfffff,
		  0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
	{	&driver_iganinju,
		{ 0x04132,0xfffff,0xfffff,0x01423,0xfffff,0xfffff,0xfffff,0xfffff,
		  0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
	{	&driver_tshingen,
		{ 0x04132,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,
		  0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
#if 0
	{	&driver_astyanax,
		{ 0x04132,0x03142,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,
		  0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
	{	&driver_hachoo,
		{ 0x24130,0x01423,0xfffff,0x02413,0x04132,0xfffff,0x24130,0x13240,
		  0x24103,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
	{	&driver_kickoff,
		{ 0x04132,0x02413,0x03142,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,
		  0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
	{	&driver_soldamj,
		{ 0x04132,0xfffff,0xfffff,0x01423,0xfffff,0xfffff,0xfffff,0x20413,
		  0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff,0xfffff }
	},
#endif
	{	0	}	// end of list: use the prom's data
};


/*
	Convert the 512 bytes in the Priority Prom into 16 ints, encoding
	the layers order for 16 selectable priority schemes.

	INPUT (to the video chip):

		4 pixels: 3 layers(012) + 1 sprite (3)
		(there are low and high priority sprites which
		are split when the "split sprites" bit is set)

	addr =	( (low pri sprite & split sprites ) << 0 ) +
			( (pixel 0 is enabled and opaque ) << 1 ) +
			( (pixel 1 is enabled and opaque ) << 2 ) +
			( (pixel 2 is enabled and opaque ) << 3 ) +
			( (pixel 3 is enabled and opaque ) << 4 ) +
			( (layers_enable bits 11-8  ) << 5 )

	OUTPUT (to video):
		1 pixel, the one from layer: PROM[addr] (layer can be 0-3)

	This scheme can generate a wealth of funky priority schemes
	while we can account for just a simple stack of transparent
	layers like: 01324. That is: bottom layer is 0, then 1, then
	sprites (low priority sprites if sprite splitting is active,
	every sprite if not) then layer 2 and high priority sprites
	(only if sprite splitting is active).

	Hence, during the conversion process we make sure each of the
	16 priority scheme in the prom is a "simple" one like the above
	and log a warning otherwise. The feasibility criterion is such:

	the opaque pens of the top layer must be above any other layer.
	The transparent pens of the top layer must be either totally
	opaque or totally transparent with respects to the other layers:
	when the bit relative to the top layer is not set, the top layer's
	code must be either always absent (transparent case) or always
	present (opaque case) in the prom.

	NOTE: This can't account for orders starting like: 030..
	as found in peekaboo's prom. That's where sprites go below
	the bottom layer's opaque pens, but above its transparent
	pens.
*/

void megasys1_convert_prom(unsigned char *palette, unsigned short *colortable,const unsigned char *prom)
{
	int pri_code, offset, i, order;

	/* First check if we have an hand-crafted priority scheme
	   available (this should happen only if no good dump
	   of the prom is known) */

	i = 0;
	while (	priorities[i].driver &&
			priorities[i].driver != Machine->gamedrv &&
			priorities[i].driver != Machine->gamedrv->clone_of)
		i++;

	if (priorities[i].driver)
	{
		memcpy (megasys1_layers_order, priorities[i].priorities, 16 * sizeof(int));

		debug_printf("WARNING: using hand-crafted priorities scheme\n");

		return;
	}

	/* Otherwise, perform the conversion from the prom itself */

	for (pri_code = 0; pri_code < 0x10 ; pri_code++)	// 16 priority codes
	{
		int layers_order[2];	// 2 layers orders (split sprites on/off)

		for (offset = 0; offset < 2; offset ++)
		{
			int enable_mask = 0xf;	// start with every layer enabled

			layers_order[offset] = 0xfffff;

			do
			{
				int top = prom[pri_code * 0x20 + offset + enable_mask * 2] & 3;	// this must be the top layer
				int top_mask = 1 << top;

				int	result = 0;		// result of the feasibility check for this layer

				for (i = 0; i < 0x10 ; i++)	// every combination of opaque and transparent pens
				{
					int opacity	=	i & enable_mask;	// only consider active layers
					int layer	=	prom[pri_code * 0x20 + offset + opacity * 2];

					if (opacity)
					{
						if (opacity & top_mask)
						{
							if (layer != top )	result |= 1; 	// error: opaque pens aren't always opaque!
						}
						else
						{
							if (layer == top)	result |= 2;	// transparent pen is opaque
							else				result |= 4;	// transparent pen is transparent
						}
					}
				}

				/*	note: 3210 means that layer 0 is the bottom layer
					(the order is reversed in the hand-crafted data) */

				layers_order[offset] = ( (layers_order[offset] << 4) | top ) & 0xfffff;
				enable_mask &= ~top_mask;

				if (result & 1)
				{
					//logerror("WARNING, pri $%X split %d - layer %d's opaque pens not totally opaque\n",pri_code,offset,top);

					layers_order[offset] = 0xfffff;
					break;
				}

				if  ((result & 6) == 6)
				{
					//logerror("WARNING, pri $%X split %d - layer %d's transparent pens aren't always transparent nor always opaque\n",pri_code,offset,top);

					layers_order[offset] = 0xfffff;
					break;
				}

				if (result == 2)	enable_mask = 0; // totally opaque top layer

			}	while (enable_mask);

        }	// offset

		/* merge the two layers orders */

		order = 0xfffff;

		for (i = 5; i > 0 ; )	// 5 layers to write
		{
			int layer;
			int layer0 = layers_order[0] & 0x0f;
			int layer1 = layers_order[1] & 0x0f;

			if (layer0 != 3)	// 0,1,2 or f
			{
				if (layer1 == 3)
				{
					layer = 4;
					layers_order[0] <<= 4;	// layer1 won't change next loop
				}
				else
				{
					layer = layer0;
					if (layer0 != layer1)
					{
						//logerror("WARNING, pri $%X - 'sprite splitting' does not simply split sprites\n",pri_code);

						order = 0xfffff;
						break;
					}

				}
			}
			else	// layer0 = 3;
			{
				if (layer1 == 3)
				{
					layer = 0x43;			// 4 must always be present
					order <<= 4;
					i --;					// 2 layers written at once
				}
				else
				{
					layer = 3;
					layers_order[1] <<= 4;	// layer1 won't change next loop
				}
			}

			/* reverse the order now */
			order = (order << 4 ) | layer;

			i --;		// layer written

			layers_order[0] >>= 4;
			layers_order[1] >>= 4;

		}	// merging

		megasys1_layers_order[pri_code] = order & 0xfffff;	// at last!

	}	// pri_code



	/* log the priority schemes */
	//for (i = 0; i < 16; i++)
	//	debug_printf("PROM %X] %05x\n", i, megasys1_layers_order[i]);


}
