#include "driver.h"
#include "state.h"
#include "vidhrdw/generic.h"
#include "vidhrdw/taitoic.h"

#define TC0100SCN_GFX_NUM 1

void othunder_vh_stop(void);
unsigned char *othunder_ram8;
UINT16 *othunder_ram;

struct tempsprite
{
	int gfx;
	int code,color;
	int flipx,flipy;
	int x,y;
	int zoomx,zoomy;
	int primask;
};
static struct tempsprite *spritelist;

static int taito_hide_pixels;

static unsigned short *spriteram16;

/* Cross hairs co-ordinates */
static int screenx_p1, screeny_p1;
static int screenx_p2, screeny_p2;

/**********************************************************/

static int othunder_core_vh_start (void)
{
	/* Make room before allocating space for the large tilemaps */
	free_disposable_regions();

	memset(spriteram, 0, spriteram_size);

	// Create 16-bit pointers to 8-bit versions
	spriteram16 = (unsigned short *)spriteram;
	othunder_ram = (UINT16 *)othunder_ram8;

	/* Up to $800/8 big sprites, requires 0x100 * sizeof(*spritelist)
	   Multiply this by 32 to give room for the number of small sprites,
	   which are what actually get put in the structure. */

	spritelist = malloc(0x2000 * sizeof(*spritelist));
	if (!spritelist)
	{
		return 1;
	}

	if (TC0100SCN_vh_start(1,TC0100SCN_GFX_NUM,taito_hide_pixels))
	{
		othunder_vh_stop();
		return 1;
	}

	if (TC0110PCR_vh_start())
	{
		othunder_vh_stop();
		return 1;
	}

	return 0;
}

int othunder_vh_start (void)
{
	/* There is a problem here. 4 is correct for text layer/sprite
	   alignment, but the bg layers [or one of them] are wrong */

	taito_hide_pixels = 4;
	return (othunder_core_vh_start());
}

void othunder_vh_stop (void)
{
	if (spritelist)
	{
	    free(spritelist);
	    spritelist = 0;
	}

	TC0100SCN_vh_stop();

	TC0110PCR_vh_stop();
}


/*********************************************************
				PALETTE
*********************************************************/

static void othunder_update_palette (void)
{
	int i,j;
	data16_t *spritemap = (data16_t *)memory_region(REGION_USER1);
	UINT16 tile_mask = (Machine->gfx[0]->total_elements) - 1;
	int map_offset,sprite_chunk,code;
	int offs,data,tilenum,color;
	unsigned short palette_map[256];
	memset (palette_map, 0, 256 * sizeof (unsigned short));

	for (offs = (spriteram_size/2)-4;offs >=0;offs -= 4)
	{
		data = spriteram16[offs+2];
		color = (data &0xff00) >> 8;

		data = spriteram16[offs+3];
		tilenum = data &0x1fff;

		if (tilenum)
		{
			map_offset = tilenum << 5;

			for (sprite_chunk=0;sprite_chunk<32;sprite_chunk++)
			{
				//i = sprite_chunk % 4;   /* 4 chunks across */
				//j = sprite_chunk / 4;   /* 8 chunks down */
				i = sprite_chunk & 3;   /* 4 chunks across */
				j = sprite_chunk >> 2;   /* 8 chunks down */

				code = spritemap[map_offset + i + (j<<2)] &tile_mask;

				palette_map[color] |= Machine->gfx[0]->pen_usage[code];
			}
		}
	}

	/* Tell MAME about the color usage */
	for (i = 0;i < 256;i++)
	{
		int usage = palette_map[i];

		if (usage)
		{
			if (palette_map[i] & (1 << 0))
				palette_used_colors[i * 16 + 0] = PALETTE_COLOR_USED;
			for (j = 1; j < 16; j++)
				if (palette_map[i] & (1 << j))
					palette_used_colors[i * 16 + j] = PALETTE_COLOR_USED;
		}
	}
}




/************************************************************
			SPRITE DRAW ROUTINE

It draws a series of small tiles ("chunks") together to
create a big sprite. The spritemap rom provides the lookup
table for this. We look up the 16x8 sprite chunks from
the spritemap rom, creating each 64x64 sprite as follows:

	 0  1  2  3
	 4  5  6  7
	 8  9 10 11
	12 13 14 15
	16 17 18 19
	20 21 22 23
	24 25 26 27
	28 29 30 31

The game makes heavy use of sprite zooming.

	***

NB: unused portions of the spritemap rom contain hex FF's.
It is a useful coding check to warn in the log if these
are being accessed. [They can be inadvertently while
spriteram is being tested, take no notice of that.]


	Othunder (modified table from Raine)

	Byte | Bit(s) | Description
	-----+76543210+-------------------------------------
	  0  |xxxxxxx.| ZoomY (0 min, 63 max - msb unused as sprites are 64x64)
	  0  |.......x| Y position (High)
	  1  |xxxxxxxx| Y position (Low)
	  2  |x.......| Sprite/BG Priority (0=sprites high)
	  2  |.x......| Flip X
	  2  |..?????.| unknown/unused ?
	  2  |.......x| X position (High)
	  3  |xxxxxxxx| X position (Low)
	  4  |xxxxxxxx| Palette bank
	  5  |?.......| unknown/unused ?
	  5  |.xxxxxxx| ZoomX (0 min, 63 max - msb unused as sprites are 64x64)
	  6  |x.......| Flip Y
	  6  |.??.....| unknown/unused ?
	  6  |...xxxxx| Sprite Tile high (2 msbs unused - 3/4 of spritemap rom empty)
	  7  |xxxxxxxx| Sprite Tile low

********************************************************/


static void othunder_draw_sprites_16x8(struct osd_bitmap *bitmap,int *primasks,int y_offs)
{
	data16_t *spritemap = (data16_t *)memory_region(REGION_USER1);
	UINT16 tile_mask = (Machine->gfx[0]->total_elements) - 1;
	int offs, data, tilenum, color, flipx, flipy;
	int x, y, priority, curx, cury;
	int sprites_flipscreen = 0;
	int zoomx, zoomy, zx, zy;
	int sprite_chunk,map_offset,code,j,k,px,py;
	int bad_chunks;

	/* pdrawgfx() needs us to draw sprites front to back, so we have to build a list
	   while processing sprite ram and then draw them all at the end */
	struct tempsprite *sprite_ptr = spritelist;

	for (offs = (spriteram_size/2)-4;offs >=0;offs -= 4)
	{
		data = spriteram16[offs+0];
		zoomy = (data & 0xfe00) >> 9;
		y = data & 0x1ff;

		data = spriteram16[offs+1];
		flipx = (data & 0x4000) >> 14;
		priority = (data & 0x8000) >> 15;
		x = data & 0x1ff;

		data = spriteram16[offs+2];
		color = (data & 0xff00) >> 8;
		zoomx = (data & 0x7f);

		data = spriteram16[offs+3];
		tilenum = data & 0x1fff;	// $80000 spritemap rom maps up to $2000 64x64 sprites
		flipy = (data & 0x8000) >> 15;

		if (!tilenum) continue;

		map_offset = tilenum << 5;

		zoomx += 1;
		zoomy += 1;

		y += y_offs;

		/* treat coords as signed */
		if (x>0x140) x -= 0x200;
		if (y>0x140) y -= 0x200;

		bad_chunks = 0;

		for (sprite_chunk=0;sprite_chunk<32;sprite_chunk++)
		{
			k = sprite_chunk & 3;   /* 4 chunks per row */
			j = sprite_chunk >> 2;   /* 8 rows */

			px = k;
			py = j;
			if (flipx)  px = 3-k;	/* pick tiles back to front for x and y flips */
			if (flipy)  py = 7-j;

			code = spritemap[map_offset + px + (py<<2)] &tile_mask;

			if (code==0xffff)
			{
				bad_chunks += 1;
				continue;
			}

			curx = x + ((k*zoomx)>>2);
			cury = y + ((j*zoomy)>>3);

			zx= x + (((k+1)*zoomx)>>2) - curx;
			zy= y + (((j+1)*zoomy)>>3) - cury;

			if (sprites_flipscreen)
			{
				/* -zx/y is there to fix zoomed sprite coords in screenflip.
				   drawgfxzoom does not know to draw from flip-side of sprites when
				   screen is flipped; so we must correct the coords ourselves. */

				curx = 320 - curx - zx;
				cury = 256 - cury - zy;
				flipx = !flipx;
				flipy = !flipy;
			}

			sprite_ptr->code = code;
			sprite_ptr->color = color;
			sprite_ptr->flipx = flipx;
			sprite_ptr->flipy = flipy;
			sprite_ptr->x = curx;
			sprite_ptr->y = cury;
			sprite_ptr->zoomx = zx << 12;
			sprite_ptr->zoomy = zy << 13;

			if (primasks)
			{
				sprite_ptr->primask = primasks[priority];
				sprite_ptr++;
			}
			else
			{
				drawgfxzoom(bitmap,Machine->gfx[0],
						sprite_ptr->code,
						sprite_ptr->color,
						sprite_ptr->flipx,sprite_ptr->flipy,
						sprite_ptr->x,sprite_ptr->y,
						&Machine->visible_area,TRANSPARENCY_PEN,0,
						sprite_ptr->zoomx,sprite_ptr->zoomy);
			}
		}

		if (bad_chunks)
		    debug_printf("Sprite number %04x had %02x invalid chunks\n",tilenum,bad_chunks);
	}


	/* this happens only if primsks != NULL */
	while (sprite_ptr != spritelist)
	{
		sprite_ptr--;

		pdrawgfxzoom(bitmap,Machine->gfx[0],
				sprite_ptr->code,
				sprite_ptr->color,
				sprite_ptr->flipx,sprite_ptr->flipy,
				sprite_ptr->x,sprite_ptr->y,
				&Machine->visible_area,TRANSPARENCY_PEN,0,
				sprite_ptr->zoomx,sprite_ptr->zoomy,
				sprite_ptr->primask);
	}
}

unsigned short swapInt16(unsigned short x) 
{ 
      return   ((x << 8) & 0xFF00) | ((x >> 8) & 0x00FF); 
}

#define ENDIAN_READ(A)	A
#define ENDIAN_MASK	0xffff0000
#define ENDIAN_SHIFT	16

static void calc_crosshair(int player)
{
	int rawx, rawy, centrex, centrey;

	if (player == 1)
	{
	    /* calculate p1 screen co-ords by matching routine at $A932 */
	    rawx = ENDIAN_READ(othunder_ram[0x2848/2]);
	    centrex = ENDIAN_READ(othunder_ram[0xa046/2]);
	    if (rawx <= centrex)
	    {
		    rawx = centrex - rawx;
		    screenx_p1 = rawx * ENDIAN_READ(othunder_ram[0xa04e/2]) + (((rawx * ENDIAN_READ(othunder_ram[0xa050/2])) & ENDIAN_MASK) >> ENDIAN_SHIFT);
		    screenx_p1 = 0xa0 - screenx_p1;
		    if (screenx_p1 < 0) screenx_p1 = 0;
	    }
	    else
	    {
		    if (rawx > ENDIAN_READ(othunder_ram[0xa028/2])) rawx = ENDIAN_READ(othunder_ram[0xa028/2]);
		    rawx -= centrex;
		    screenx_p1 = rawx * ENDIAN_READ(othunder_ram[0xa056/2]) + (((rawx * ENDIAN_READ(othunder_ram[0xa058/2])) & ENDIAN_MASK) >> ENDIAN_SHIFT);
		    screenx_p1 += 0xa0;
		    if (screenx_p1 > 0x140) screenx_p1 = 0x140;
	    }
	    rawy = ENDIAN_READ(othunder_ram[0x284a/2]);
	    centrey = ENDIAN_READ(othunder_ram[0xa048/2]);
	    if (rawy <= centrey)
	    {
		    rawy = centrey - rawy;
		    screeny_p1 = rawy * ENDIAN_READ(othunder_ram[0xa052/2]) + (((rawy * ENDIAN_READ(othunder_ram[0xa054/2])) & ENDIAN_MASK) >> ENDIAN_SHIFT);
		    screeny_p1 = 0x78 - screeny_p1;
		    if (screeny_p1 < 0) screeny_p1 = 0;
	    }
	    else
	    {
		    if (rawy > ENDIAN_READ(othunder_ram[0xa030/2])) rawy = ENDIAN_READ(othunder_ram[0xa030/2]);
		    rawy -= centrey;
		    screeny_p1 = rawy * ENDIAN_READ(othunder_ram[0xa05a/2]) + (((rawy * ENDIAN_READ(othunder_ram[0xa05c/2])) & ENDIAN_MASK) >> ENDIAN_SHIFT);
		    screeny_p1 += 0x78;
		    if (screeny_p1 > 0xf0) screeny_p1 = 0xf0;
	    }

	    // fudge y to show in centre of scope/hit sprite, note that screenx_p1, screeny_p1
	    // were confirmed to match those stored by the game at $82732, $82734
	    screeny_p1 += 2;

	}
	else
	{
	    /* calculate p2 screen co-ords by matching routine at $AA48 */
	    rawx = ENDIAN_READ(othunder_ram[0x284c/2]);
	    centrex = ENDIAN_READ(othunder_ram[0xa04a/2]);
	    if (rawx <= centrex)
	    {
		    rawx = centrex - rawx;
		    screenx_p2 = rawx * ENDIAN_READ(othunder_ram[0xa05e/2]) + (((rawx * ENDIAN_READ(othunder_ram[0xa060/2])) & ENDIAN_MASK) >> ENDIAN_SHIFT);
		    screenx_p2 = 0xa0 - screenx_p2;
		    if (screenx_p2 < 0) screenx_p2 = 0;
	    }
	    else
	    {
		    if (rawx > ENDIAN_READ(othunder_ram[0xa038/2])) rawx = ENDIAN_READ(othunder_ram[0xa038/2]);
		    rawx -= centrex;
		    screenx_p2 = rawx * ENDIAN_READ(othunder_ram[0xa066/2]) + (((rawx * ENDIAN_READ(othunder_ram[0xa068/2])) & ENDIAN_MASK) >> ENDIAN_SHIFT);
		    screenx_p2 += 0xa0;
		    if (screenx_p2 > 0x140) screenx_p2 = 0x140;
	    }
	    rawy = ENDIAN_READ(othunder_ram[0x284e/2]);
	    centrey = ENDIAN_READ(othunder_ram[0xa04c/2]);
	    if (rawy <= centrey)
	    {
		    rawy = centrey - rawy;
		    screeny_p2 = rawy * ENDIAN_READ(othunder_ram[0xa062/2]) + (((rawy * ENDIAN_READ(othunder_ram[0xa064/2])) & ENDIAN_MASK) >> ENDIAN_SHIFT);
		    screeny_p2 = 0x78 - screeny_p2;
		    if (screeny_p2 < 0) screeny_p2 = 0;
	    }
	    else
	    {
		    if (rawy > ENDIAN_READ(othunder_ram[0xa040/2])) rawy = ENDIAN_READ(othunder_ram[0xa040/2]);
		    rawy -= centrey;
		    screeny_p2 = rawy * ENDIAN_READ(othunder_ram[0xa06a/2]) + (((rawy * ENDIAN_READ(othunder_ram[0xa06c/2])) & ENDIAN_MASK) >> ENDIAN_SHIFT);
		    screeny_p2 += 0x78;
		    if (screeny_p2 > 0xf0) screeny_p2 = 0xf0;
	    }

	    // fudge y to show in centre of scope/hit sprite, note that screenx_p2, screeny_p2
	    // were confirmed to match those stored by the game at $82736, $82738
	    screeny_p2 += 2;
	}
}

/**************************************************************
				SCREEN REFRESH
**************************************************************/
void othunder_vh_eof_callback(void)
{
	//calc_crosshair(1);
}


void othunder_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh)
{
	int layer[3];

	TC0100SCN_tilemap_update();

	palette_init_used_colors();
	othunder_update_palette();
	palette_used_colors[0] |= PALETTE_COLOR_VISIBLE;

	if (palette_recalc())
	    tilemap_mark_all_pixels_dirty(ALL_TILEMAPS);

	TC0100SCN_tilemap_render(1);

	layer[0] = TC0100SCN_bottomlayer(0);
	layer[1] = layer[0]^1;
	layer[2] = 2;

	fillbitmap(priority_bitmap,0,NULL);

	if (TC0100SCN_is_layer_disabled(0, layer[0]))
	{
	    /* Ensure screen blanked when bottom layer not drawn due to disable bit */
	    fillbitmap(bitmap, palette_transparent_pen, &Machine -> visible_area);
	}
	else
	{
	    TC0100SCN_tilemap_draw(bitmap,0,layer[0],TILEMAP_IGNORE_TRANSPARENCY|1<<16);
	}

	TC0100SCN_tilemap_draw(bitmap,0,layer[1],2<<16);
	TC0100SCN_tilemap_draw(bitmap,0,layer[2],4<<16);

	/* Sprites can be under/over the layer below text layer */
	{
		int primasks[2] = {0xf0,0xfc};
		othunder_draw_sprites_16x8(bitmap,primasks,3);
	}

	/* Draw artificial gun targets */
	/* player 1 */
	calc_crosshair(1);
//debug_printf("ch1(%d,%d)\n", screenx_p1, screeny_p1);
	draw_crosshair(bitmap,screenx_p1,screeny_p1,&Machine->visible_area);
	/* player 2 */
	//draw_crosshair(bitmap,screenx_p2,screeny_p2,&Machine->visible_area);
}
